# -*- coding: utf-8 -*-
"""
Created on Tue Jan 23 16:22:20 2018

@author: Callie Darsey

My adjust a bounding box class.

Adjustable_BB_03
"""

import cv2
import numpy as np
import labeling_image_components_01 as labeling

class AdjustableBB:
    '''
    Adjustable bounding box class.
    
    references
    https://www.pyimagesearch.com/2015/03/09/capturing-mouse-click-events-with-python-and-opencv/
    https://docs.opencv.org/3.3.1/d7/dfc/group__highgui.html
    '''
    def __init__(self, question):
        '''

        ''' 
        
        # set up the yes, no, quit                  
        self.options = labeling.YesNoQuit(question)
        self.reply = 'no'

        
    def act(self,  title, image, point1, point2, move_window):

        self.title = title
        self.image_orig = np.copy(image)
        self.point1 = point1
        self.point2 = point2

        '''
        For thickness = 2, border is three pixes wide.
        The points ar in the middle pixel.
        If a point is on edge, the boarder is only 2 pixels thick.
        '''
        thickness = 2
        color = (0, 255, 0)
        self.image = np.copy(self.image_orig)
        cv2.rectangle(self.image, point1, point2, color, thickness)
        
        cv2.namedWindow(self.title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
        cv2.moveWindow(self.title, move_window[0], move_window[1])

        cv2.setMouseCallback(self.title, self._click)
        
        cv2.imshow(self.title, self.image)

        while True:
            
            cv2.waitKey(1)
            
            if self.reply == 'yes' or self.reply == 'quit':
                
                break

        
    def get_reply(self):
        
        return self.reply


    def get_points(self):
        '''
        return the points in the form
        numpy.array([point1_x, point1_y, point2_x, point2_y])
        '''
        return np.array([self.point1[0], self.point1[1], \
                         self.point2[0], self.point2[1]])
        
        
    def _click(self, event, x, y, flags, params):
        
        if event == cv2.EVENT_LBUTTONDOWN:
            #print('left mouse down')
            
            # save coordinates and wait to come up to process
            self.down = [x, y]
            
            print(self.down)
            
        elif event == cv2.EVENT_LBUTTONUP:
            #print('left mouse up')
            
            # save label (1) and imshow and such if click is valid 
            self.up = [x, y]
            
            print(self.up)
            
            self._evaluate()
            
    def _valid_down(self, border_checking_min, border_checking_max,
                    connected_border_lo_val, connected_border_hi_val,
                    border_checking_point_val, connected_border_point_val):
        '''
        This function is used to if a click was in a specified bounding box 
        border.
        
        border_checking_min        - min in the border of interest
        border_checking_max        - max in the border of interest
        connected_border_lo_val    - value (val) of the lo border connected to
                                     the border of interest
        connected_border_hi_val    - value (val) of the hi border connected to
                                     the border of interest
        border_checking_point_val  - point value in the dimesion of the border
                                     of interest
        connected_border_point_val - point value in the dimension of the
                                     borders connected to the border of
                                     interest

        Outputs True if the down click was on the bounding box border.
        '''
            
        return border_checking_point_val >= border_checking_min and \
               border_checking_point_val <= border_checking_max and \
               connected_border_point_val > connected_border_lo_val and \
               connected_border_point_val < connected_border_hi_val

    def _adjust_bb(self, lo_min, lo_val, lo_max, hi_min, hi_val, hi_max,
                   new_val, side_dragged):
        '''
        Return the new low (lo) and high (hi) values of the adjusted box.
        
        lo_min, lo_val, lo_max - min, val, and max for the lo border of
                                 interest
                                 
        hi_min, hi_val, hi_max - min, val, and max for the hi border of
                                 interest
        
        new_val                - the value in the dimension of interest at
                                 which the click went up
                                 
        side_dragged           - string 'lo' or 'hi' indicating which side was
                                 dragged
        '''
        
        new_lo_val = lo_val
        new_hi_val = hi_val

        
        if new_val < lo_min:
            
            if side_dragged == 'lo':
                new_lo_val = new_val
            elif side_dragged == 'hi':
                new_lo_val = new_val
                new_hi_val = lo_val
        
        elif new_val > lo_max and new_val < hi_min:
            
            if side_dragged == 'lo':
                new_lo_val = new_val
            elif side_dragged == 'hi':
                new_hi_val = new_val
        
        elif new_val > hi_max:
            
            if side_dragged == 'lo':
                new_lo_val = hi_val
                new_hi_val = new_val
            elif side_dragged == 'hi':
                new_hi_val = new_val
        
        return (new_lo_val, new_hi_val)



            
    def _evaluate(self):
        '''
                                     header y border
                                           (lo)
                             (min) ----------------- (min)
                             (val) ----------------- (val)
                             (max) ----------------- (max)
            (min) (val) (max)                             (min) (val) (max)
              |     |     |                                 |     |     |
              |     |     |                                 |     |     |
left x border |     |     |             inside              |     |     | right x border
    (lo)      |     |     |                                 |     |     |      (hi)
              |     |     |                                 |     |     |
            (min) (val) (max)                             (min) (val) (max)
                             (min) ----------------- (min)
                             (val) ----------------- (val)
                             (max) ----------------- (max)
                                     footer y border
                                           (hi)
        

        '''
        # for the value in the borders
        x_lo_val, x_hi_val = np.sort([self.point1[0], self.point2[0]])
        y_lo_val, y_hi_val = np.sort([self.point1[1], self.point2[1]])
        
        # for the min and max in the borders
        x_lo_min = np.max([x_lo_val - 1, 0])
        x_lo_max = np.min([x_lo_val + 1, self.image.shape[1] - 1])
        
        x_hi_min = np.max([x_hi_val - 1, 0])
        x_hi_max = np.min([x_hi_val + 1, self.image.shape[1] - 1])
        
        y_lo_min = np.max([y_lo_val - 1, 0])
        y_lo_max = np.min([y_lo_val + 1, self.image.shape[0] - 1])
        
        y_hi_min = np.max([y_hi_val - 1, 0])
        y_hi_max = np.min([y_hi_val + 1, self.image.shape[0] - 1])
        
        # force up click within image
        self.up[0] = np.max([self.up[0], 0])
        self.up[0] = np.min([self.up[0], self.image.shape[1] - 1])
        
        self.up[1] = np.max([self.up[1], 0])
        self.up[1] = np.min([self.up[1], self.image.shape[0] - 1])        
                
        # in left border
        if self._valid_down(x_lo_min, x_lo_max, y_lo_val, y_hi_val,
                            self.down[0], self.down[1]):
            
            print('\nin left border')

            x_lo_val, x_hi_val = self._adjust_bb(x_lo_min, x_lo_val, x_lo_max,
                                                 x_hi_min, x_hi_val, x_hi_max,
                                                 self.up[0], 'lo')
                
        # in right border        
        elif self._valid_down(x_hi_min, x_hi_max, y_lo_val, y_hi_val,
                              self.down[0], self.down[1]):
            
            print('\nin right border')
            
            x_lo_val, x_hi_val = self._adjust_bb(x_lo_min, x_lo_val, x_lo_max,
                                                 x_hi_min, x_hi_val, x_hi_max,
                                                 self.up[0], 'hi')
        
        # in header border
        elif self._valid_down(y_lo_min, y_lo_max, x_lo_val, x_hi_val,
                              self.down[1], self.down[0]):
            
            print('\nin header border')
            
            y_lo_val, y_hi_val = self._adjust_bb(y_lo_min, y_lo_val, y_lo_max,
                                                 y_hi_min, y_hi_val, y_hi_max,
                                                 self.up[1], 'lo')

        # in footer border
        elif self._valid_down(y_hi_min, y_hi_max, x_lo_val, x_hi_val,
                              self.down[1], self.down[0]):
            
            print('\nin footer border')
            
            y_lo_val, y_hi_val = self._adjust_bb(y_lo_min, y_lo_val, y_lo_max,
                                                 y_hi_min, y_hi_val, y_hi_max,
                                                 self.up[1], 'hi')
        # inside    
        elif self._valid_down(x_lo_max + 1, x_hi_min - 1, y_lo_max, y_hi_min,
                              self.down[0], self.down[1]) and \
             self._valid_down(x_lo_max + 1, x_hi_min - 1, y_lo_max, y_hi_min,
                              self.up[0], self.up[1]):
                 
                 self.options.ask()
                 print('No longer asking.')
                 self.reply = self.options.get_reply()
                 
                 if self.reply == 'yes' or self.reply == 'quit':
                     
                     cv2.destroyWindow(self.title)
                                          
                     return
                 
        
        self.point1 = (x_lo_val, y_lo_val)
        self.point2 = (x_hi_val, y_hi_val)
        
        thickness = 2
        color = (0, 255, 0)
        self.image = np.copy(self.image_orig)
        cv2.rectangle(self.image, self.point1, self.point2, color, thickness)
        
        cv2.imshow(self.title, self.image)

if __name__ == '__main__':
        
    # Read image
    im_path = "C:\\Users\\Callie\\" + \
                    "ECE_633_Adv_Topics_in_Image_Processing\\" + \
                    "Class_Project\\presentation\\" + \
                    "mouth_movement_not_captured_02.PNG"
    im = cv2.imread(im_path)    

    question = 'Are ready to leave?\n\n\n' + \
               'Yes to close adjustment window and save box.\n\n' + \
               'No to continue adjusting.\n\n' + \
               'Quit to close adjustment window without saving.\n\n'  
               
    cv2.imshow('other', im)
    
    im_copy = np.copy(im)
                 
    abb = AdjustableBB(question)
    abb.act('title', im, (30, 30), (130, 130), (0, 0))
    
    print(abb.get_points())
    print(abb.get_reply())