# -*- coding: utf-8 -*-
"""
Created on Wed May  2 18:46:16 2018

@author: Callie Darsey

Trying LogisticRegression

attempt_log_reg_patches_01
"""

import pickle
import os
import cv2
import numpy as np
import patches_01 as patches
import sklearn.linear_model as sklm
import sklearn.neighbors as skn
import sklearn.model_selection as skm
import sklearn.metrics as skmet
import matplotlib.pyplot as plt

def plot_precision_recall_vs_threshold(precisions, recalls, thresholds):
    '''
    Hands-On Machine Learning with Scikit-Learn & TensorFlow 
    by Aurelien Geron for Chapter 3: Classification page 89
    '''
    plt.plot(thresholds, precisions[:-1], "b--", label="Precision")
    plt.plot(thresholds, recalls[:-1], "g-", label="Recall")
    plt.xlabel("Threshold")
    plt.legend(loc="upper left")
    plt.ylim([0, 1])


# paths
path = './video_copies_clips_track/ROI_masks_patch_detection/'
feature_vectors = path + "block_feature_vectors/"
set1_path = path + 'knn_5/set1/'
set2_path = path + 'knn_5/set2/'
feature_vectors_after_clf = \
    set2_path + "block_feature_vectors_after_classification/"
set3_path = path + 'knn_5/set3/'  

k = 5  
    

'''data set broken into three groups'''
# train hand versus randomly chosen non skin
set1 = [\
'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',]

# test with these and save the classification pictures
set2 = [\
'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',]

'''
MVI_0023_trans__100_200_2132.png OR MVI_0029_trans__75_125_0212.png OR MVI_0013_trans__110_140_0768.png OR (STI WA Apr25 AMVI_0068_trans__860_910_1772.png - classified)
'''

# use as a test set
set3 = [\
'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
'STI WA May09 B-MVI_0006_trans__600_700_0496.data',]

'''
MVI_0095_trans__0_100_0540.png OR MVI_0091_trans__60_85_0480.png OR MVI_0087_trans__600_700_1224.png OR MVI_0110_trans__100_200_2384.png OR MVI_0006_trans__600_700_0496.png
'''




def set1_cv_and_model1_training():
    
    print('***************************************************',
          'set1_cv_and_model1_training',
          '***************************************************')
    

    
    
    # cross validation scores
    
    # train with all
    
    filenames = set1
    
    print('Number of frames: {}\n'.format(len(filenames)))
    
    for block_dim in [2, 3, 4, 5]:
        print('block dimension ({:d}, {:d})'.format(block_dim, block_dim))
        
        hand_feature_vectors_collect = []
        non_skin_feature_vectors_collect = []
            
        for filename in filenames:
            
            print(filename)
        
            with open(feature_vectors + filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                
                hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
            
            # I assume there are more non-skin than hand, so verify
            assert hand_feature_vectors.shape[0] < non_skin_feature_vectors.shape[0]
        
            # shuffle, then take a subset of non-skin
            np.random.shuffle(non_skin_feature_vectors)
            
            non_skin_feature_vectors_take = \
                non_skin_feature_vectors[0:hand_feature_vectors.shape[0], :]
            
            # save for further stacking
            hand_feature_vectors_collect.append(hand_feature_vectors)
            non_skin_feature_vectors_collect.append(non_skin_feature_vectors_take)
            
        feature_vectors_hand = np.vstack(hand_feature_vectors_collect)
        feature_vectors_non_skin = np.vstack(non_skin_feature_vectors_collect)
        
        # hand 1 and non-skin 0
        y = np.hstack([np.ones(feature_vectors_hand.shape[0]),
                       np.zeros(feature_vectors_non_skin.shape[0])])
        
        # feature vectors
        X = np.vstack([feature_vectors_hand, feature_vectors_non_skin])
        
        # shuffle
        indices_shuffled = np.arange(0, y.shape[0])
        np.random.shuffle(indices_shuffled)
        
        y = y[indices_shuffled]
        X = X[indices_shuffled, :]
        print('X.shape', X.shape)
        
        
        clf_knn = skn.KNeighborsClassifier(n_neighbors=k)
        
        cv_predict = skm.cross_val_predict(clf_knn, X, y=y, cv=10, verbose=1)
        
        print('cv_predict.shape', cv_predict.shape)
        
        
        print('confusion matrix:\n', skmet.confusion_matrix(y, cv_predict))
        print('precision:', skmet.precision_score(y, cv_predict))
        print('recall:   ', skmet.recall_score(y, cv_predict))
        print('F1:       ', skmet.f1_score(y, cv_predict))
        print('ROC AUC:  ', skmet.roc_auc_score(y, cv_predict), '\n')
        
        # train with all
        clf_knn.fit(X, y)
        
        #pickle the model
        with open(set1_path + \
                  'knn_{}_block_dim_{:02d}.data'.format(k, block_dim),
                  'wb') as f:            
                pickle.dump((clf_knn), f)






def test_model1_with_set2():
    
    print('***************************************************',
          'test_model1_with_set2',
          '***************************************************')

    filenames = set2
    
    for block_dim in [2, 3, 4, 5]:
        
        print('\nblock dimension ({:d}, {:d})'.format(block_dim, block_dim))
            
        with open(set1_path + \
                  'knn_{}_block_dim_{:02d}.data'.format(k, block_dim),
                  'rb') as f:            
                model = pickle.load(f)
                            
        for filename in filenames:
            
            print(filename)
            
            # associated with ROIs
            with open(feature_vectors + filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                
                hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
                
            y_ROI = np.hstack([np.ones(hand_feature_vectors.shape[0]),
                           np.zeros(non_skin_feature_vectors.shape[0])])
            
            X_ROI = np.vstack([hand_feature_vectors, non_skin_feature_vectors])
            
            y_predicted = model.predict(X_ROI)
                        
            print('confusion matrix:\n', skmet.confusion_matrix(y_ROI, y_predicted))
            print('precision:       ', skmet.precision_score(y_ROI, y_predicted))
            print('recall:          ', skmet.recall_score(y_ROI, y_predicted))
            print('F1:              ', skmet.f1_score(y_ROI, y_predicted))
            print('ROC AUC:         ', skmet.roc_auc_score(y_ROI, y_predicted), '\n')      
            
            # not associated with ROIs
            with open(path + filename, 'rb') as f:
                obj_data, frame = pickle.load(f)
            
            X = patches.get_valid_blocks_as_feature_vectors(frame, \
                np.ones(frame.shape[0:2]), block_dim)
            
            predicted_labels = model.predict(X)
    
            skin_mask = patches.build_mask_from_patch_predictions(frame,
                                                                  predicted_labels,
                                                                  block_dim)
            skin_mask_display = cv2.bitwise_and(frame, frame, mask=skin_mask)
        
    
            cv2.imwrite(set2_path + 'classified_patch_skin_mask_' + \
                        filename[0:-5] + \
                        '_knn_{}__block_dim_{:02d}'.format(k, block_dim) + \
                        '.png', skin_mask_display)    
        



def extract_patches_from_set2_classified():
    
    print('***************************************************',
          'extract_patches_from_set2_classified',
          '***************************************************')
    filenames = set2
    # extract non-skin patches that were misclassified with model trained with set1
        
    print('Number of frames: {}\n'.format(len(filenames)))
    
    for block_dim in [2, 3, 4, 5]:
    
        print('block dimension ({:d}, {:d})'.format(block_dim, block_dim))
               
        for filename in filenames:
            
            print(filename)
            
            # get the roi masks
            with open(path + filename, 'rb') as f:
                
                roi_masks, frame = pickle.load(f)
            
            
            # get classifed as skin mask
            classified_frame = cv2.imread(set2_path + \
                'classified_patch_skin_mask_' + filename[0:-5] + \
                '_knn_{}__block_dim_{:02d}.png'.format(k, block_dim))
            
            classified_skin_mask = (np.sum(classified_frame.astype(np.float),
                                           axis=2) > 0).astype(np.uint8)
            
            # from the ROI info, collect the hand mask and non-skin mask
            hand_mask_collect = []
            non_skin_mask_collect = []
            
            for key in roi_masks.keys():
                
                if 'hand' in key:
                    
                    hand_mask_collect.append(roi_masks[key])
                    
                elif 'non_skin' in key:
                    
                    non_skin_mask_collect.append(roi_masks[key] * \
                                                 classified_skin_mask)
                    
            hand_mask = np.any(hand_mask_collect, axis=0)       
            non_skin_mask = np.any(non_skin_mask_collect, axis=0)
            
            # AND the ROI non-skin mask with the classified as skin mask
            non_skin_mask = np.all([classified_skin_mask, non_skin_mask], axis=0)
            
            hand_feature_vectors = \
                patches.get_valid_blocks_as_feature_vectors(frame, hand_mask,
                                                            block_dim)
        
            non_skin_feature_vectors = \
                patches.get_valid_blocks_as_feature_vectors(frame, non_skin_mask,
                                                            block_dim)
            print('Number of hand feature vector:     {}'.format(\
                  hand_feature_vectors.shape[0]))
            print('Number of non-skin feature vector: {}\n'.format(\
                  non_skin_feature_vectors.shape[0]))
        
            with open(feature_vectors_after_clf + filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'wb') as f:
                
                pickle.dump((hand_feature_vectors, non_skin_feature_vectors), f)


def set2_cv_train():
    
    print('***************************************************',
          'set2_cv_train',
          '***************************************************')

    # train with the extracted
    filenames = set2
    
    print('Number of frames: {}\n'.format(len(filenames)))
    
    for block_dim in [2, 3, 4, 5]:
        print('block dimension ({:d}, {:d})'.format(block_dim, block_dim))
        
        hand_feature_vectors_collect = []
        non_skin_feature_vectors_collect = []
            
        for filename in filenames:
            
            print(filename)
        
            with open(feature_vectors_after_clf + filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                
                hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
            
            # I assume there are more non-skin than hand, so verify
            assert hand_feature_vectors.shape[0] < non_skin_feature_vectors.shape[0]
        
            # shuffle, then take a subset of non-skin
            np.random.shuffle(non_skin_feature_vectors)
            
            non_skin_feature_vectors_take = \
                non_skin_feature_vectors[0:hand_feature_vectors.shape[0], :]
            
            # save for further stacking
            hand_feature_vectors_collect.append(hand_feature_vectors)
            non_skin_feature_vectors_collect.append(non_skin_feature_vectors_take)
            
        feature_vectors_hand = np.vstack(hand_feature_vectors_collect)
        feature_vectors_non_skin = np.vstack(non_skin_feature_vectors_collect)
        
        # hand 1 and non-skin 0
        y = np.hstack([np.ones(feature_vectors_hand.shape[0]),
                       np.zeros(feature_vectors_non_skin.shape[0])])
        
        # feature vectors
        X = np.vstack([feature_vectors_hand, feature_vectors_non_skin])
        
        # shuffle
        indices_shuffled = np.arange(0, y.shape[0])
        np.random.shuffle(indices_shuffled)
        
        y = y[indices_shuffled]
        X = X[indices_shuffled, :]
        print('X.shape', X.shape)
        
        
        clf_knn = skn.KNeighborsClassifier(n_neighbors=5)
        
        cv_predict = skm.cross_val_predict(clf_knn, X, y=y, cv=10, verbose=1)
        
        print('cv_predict.shape', cv_predict.shape)
        
        
        print('confusion matrix:\n', skmet.confusion_matrix(y, cv_predict))
        print('precision:', skmet.precision_score(y, cv_predict))
        print('recall:   ', skmet.recall_score(y, cv_predict))
        print('F1:       ', skmet.f1_score(y, cv_predict))
        print('ROC AUC:  ', skmet.roc_auc_score(y, cv_predict), '\n')
        
        # train with all
        clf_knn.fit(X, y)
        
        #pickle the model
        with open(set2_path + \
                  'knn_{}_block_dim_{:02d}.data'.format(k, block_dim),
                  'wb') as f:            
                pickle.dump((clf_knn), f)











def test_model2_set3():
    
    print('***************************************************',
          'test_model2_set3',
          '***************************************************')    

    filenames = set3
    
    for block_dim in [2, 3, 4, 5]:
        
        print('\nblock dimension ({:d}, {:d})'.format(block_dim, block_dim))
            
        with open(set2_path + \
                  'knn_{}_block_dim_{:02d}.data'.format(k, block_dim),
                  'rb') as f:            
                model = pickle.load(f)
            
        # save for the precision / recall curve
        y_ROI_collect = []
        y_decision_function_collect = []
                
        for filename in filenames:
            
            print(filename)
            
            # associated with ROIs
            with open(feature_vectors + filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                
                hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
                
            y_ROI = np.hstack([np.ones(hand_feature_vectors.shape[0]),
                           np.zeros(non_skin_feature_vectors.shape[0])])
            
            X_ROI = np.vstack([hand_feature_vectors, non_skin_feature_vectors])
            
            y_predicted = model.predict(X_ROI)
            
            print('confusion matrix:\n', skmet.confusion_matrix(y_ROI, y_predicted))
            print('precision:       ', skmet.precision_score(y_ROI, y_predicted))
            print('recall:          ', skmet.recall_score(y_ROI, y_predicted))
            print('F1:              ', skmet.f1_score(y_ROI, y_predicted))
            print('ROC AUC:         ', skmet.roc_auc_score(y_ROI, y_predicted), '\n')      
            
            # not associated with ROIs
            with open(path + filename, 'rb') as f:
                obj_data, frame = pickle.load(f)
            
            X = patches.get_valid_blocks_as_feature_vectors(frame, \
                np.ones(frame.shape[0:2]), block_dim)
            
            predicted_labels = model.predict(X)
    
            skin_mask = patches.build_mask_from_patch_predictions(frame,
                                                                  predicted_labels,
                                                                  block_dim)
            skin_mask_display = cv2.bitwise_and(frame, frame, mask=skin_mask)
        
    
            cv2.imwrite(set3_path + 'classified_patch_skin_mask_' + \
                        filename[0:-5] + \
                        '_knn_{}__block_dim_{:02d}'.format(k, block_dim) + \
                        '.png', skin_mask_display)    



def test_model1_model2_set3():
    
    print('***************************************************',
          'test_model1_model2_set3',
          '***************************************************')    

    # classify with both models
    
    for block_dim in [2, 3, 4, 5]:
        
        print('\nblock dimension ({:d}, {:d})'.format(block_dim, block_dim))
            
        with open(set1_path + \
                  'knn_{}_block_dim_{:02d}.data'.format(k, block_dim),
                  'rb') as f:            
                model1 = pickle.load(f)
    
        with open(set2_path + \
                  'knn_{}_block_dim_{:02d}.data'.format(k, block_dim),
                  'rb') as f:            
                model2 = pickle.load(f)
            
        # save for the precision / recall curve
        y_ROI_collect = []
        y_decision_function_collect = []
                
        for filename in filenames:
            
            print(filename)
            
            # associated with ROIs
            with open(feature_vectors + filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                
                hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
                
            y_ROI = np.hstack([np.ones(hand_feature_vectors.shape[0]),
                           np.zeros(non_skin_feature_vectors.shape[0])])
            
            X_ROI = np.vstack([hand_feature_vectors, non_skin_feature_vectors])
            
            # prediction
            y_predicted_1 = model1.predict(X_ROI)        
            
            ind = np.where(y_predicted_1)[0]
            y_predicted_2= model2.predict(X_ROI[ind, :])        
            pind = np.where(y_predicted_2 == 0)[0]
            
            y_predicted = y_predicted_1.copy()
            
            y_predicted[ind[pind]] = y_predicted_1[pind]
            
            print('confusion matrix:\n', skmet.confusion_matrix(y_ROI, y_predicted))
            print('precision:       ', skmet.precision_score(y_ROI, y_predicted))
            print('recall:          ', skmet.recall_score(y_ROI, y_predicted))
            print('F1:              ', skmet.f1_score(y_ROI, y_predicted))
            print('ROC AUC:         ', skmet.roc_auc_score(y_ROI, y_predicted), '\n')      
            
            # not associated with ROIs
            with open(path + filename, 'rb') as f:
                obj_data, frame = pickle.load(f)
            
            X = patches.get_valid_blocks_as_feature_vectors(frame, \
                np.ones(frame.shape[0:2]), block_dim)
            
            # prediction
            y_predicted_1 = model1.predict(X)        
            
            ind = np.where(y_predicted_1)[0]
            y_predicted_2 = model2.predict(X[ind, :])        
            pind = np.where(y_predicted_2 == 0)[0]
            
            predicted_labels = y_predicted_1.copy()
            
            predicted_labels[ind[pind]] = y_predicted_1[pind]
    
            skin_mask = patches.build_mask_from_patch_predictions(frame,
                                                                  predicted_labels,
                                                                  block_dim)
            skin_mask_display = cv2.bitwise_and(frame, frame, mask=skin_mask)
        
    
            cv2.imwrite(set3_path + 'both_classified_patch_skin_mask_' + \
                        filename[0:-5] + \
                        '_knn_{}__block_dim_{:02d}'.format(k, block_dim) + \
                        '.png', skin_mask_display)    


#set1_cv_and_model1_training()
#test_model1_with_set2()
extract_patches_from_set2_classified()
set2_cv_train()
test_model2_set3()
test_model1_model2_set3()
