# -*- coding: utf-8 -*-
"""
Created on Thu May  3 12:25:38 2018

@author: Callie Darsey

do leave-one-out for testing per frame

attempt_log_reg_AND_knn_with_Cleanup_01
"""

import pickle
import os
import cv2
import numpy as np
import patches_01 as patches
import sklearn.linear_model as sklm
import sklearn.neighbors as skn
import sklearn.ensemble as ske
import sklearn.model_selection as skm
import sklearn.metrics as skmet
import matplotlib.pyplot as plt

def locate_window(title):
    '''
    This function places the window for openCV
    
    '''
    
    move_window = [0, -30]
    cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(title, move_window[0], move_window[1]) 
    
def fill_holes(mask):
    '''
    Fill the holes in connected components in the mask.
    
    References:
        https://www.learnopencv.com/filling-holes-in-an-image-using-opencv-python-c/
        
        http://answers.opencv.org/question/9863/fill-holes-of-a-binary-image/
    '''

    assert mask.dtype == np.uint8, 'fill_holes: mask expected to be UINT8'
    
    # force nonzero values to be 255
    mask[mask != 0] = 255
    
    # make a zero border around image    
    mask_floodfill = np.zeros((mask.shape[0] + 2, mask.shape[1] + 2),
                              dtype=np.uint8)
    mask_floodfill[1:mask.shape[0] + 1, 1:mask.shape[1] + 1] = mask
    
    # starting from the zero border, do a flood fill
    cv2.floodFill(mask_floodfill, None, (0, 0), 255)
    
    # invert making the holes in the connected components 255
    holes = cv2.bitwise_not(mask_floodfill[1:mask.shape[0] + 1,
                                           1:mask.shape[1] + 1])
    
    # fill the holes via OR and return
    return cv2.bitwise_or(mask, holes)
        

def validation():
        
    # paths
    path = './video_copies_clips_track/ROI_masks_patch_detection/'
    feature_vectors = path + "block_feature_vectors/"
    out_path = path + 'log_reg_AND_knn_with_Cleanup_01/'
    
    filenames = [\
    'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
    'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
    'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
    'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
    'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
    'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
    'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
    'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
    'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
    'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
    'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
    'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
    'STI WA May09 B-MVI_0006_trans__600_700_0496.data']    
    
    filenames = np.array(filenames)
    indices = np.arange(len(filenames))
    
    print('number of frames:', len(filenames))
    
    #for block_dim in [2, 3, 4, 5]:
    #for block_dim in [6, 7, 8]:
    for block_dim in [9, 10, 11]:
        
        print('block size {:d} x {:d}'.format(block_dim, block_dim))
        
        collect_val_recall = []
        collect_val_precision = []
    
        for i in indices:
            
            val_test_filename = filenames[i]
            train_filenames = filenames[np.where(indices != i)[0]]
            
            # get the feature vectors
            
            hand_feature_vectors_collect = []
            non_skin_feature_vectors_collect = []       
            
            for filename in train_filenames:
                
                print(filename)
            
                with open(feature_vectors + filename[0:-5] + \
                          '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                    
                    hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
                
                # I assume there are more non-skin than hand, so verify
                assert hand_feature_vectors.shape[0] < non_skin_feature_vectors.shape[0]
            
                # shuffle, then take a subset of non-skin
                np.random.shuffle(non_skin_feature_vectors)
                
                non_skin_feature_vectors_take = \
                    non_skin_feature_vectors[0:hand_feature_vectors.shape[0], :]
                
                # save for further stacking
                hand_feature_vectors_collect.append(hand_feature_vectors)
                non_skin_feature_vectors_collect.append(non_skin_feature_vectors_take)
                
            feature_vectors_hand = np.vstack(hand_feature_vectors_collect)
            feature_vectors_non_skin = np.vstack(non_skin_feature_vectors_collect)
            
            # hand 1 and non-skin 0
            y = np.hstack([np.ones(feature_vectors_hand.shape[0]),
                           np.zeros(feature_vectors_non_skin.shape[0])])
            
            # feature vectors
            X = np.vstack([feature_vectors_hand, feature_vectors_non_skin])
            
            # shuffle
            indices_shuffled = np.arange(0, y.shape[0])
            np.random.shuffle(indices_shuffled)
            
            y = y[indices_shuffled]
            X = X[indices_shuffled, :]
            print('X.shape', X.shape)
    
            # train knn
            clf_knn_default = skn.KNeighborsClassifier()
            clf_knn_default.fit(X, y)
            
            # train logistic regression
            clf_log_reg_default = sklm.LogisticRegression()
            clf_log_reg_default.fit(X, y)
            
            # validation test
        
            print('validation test filename:', val_test_filename)
            
            # associated with ROIs
            with open(feature_vectors + val_test_filename[0:-5] + \
                      '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
                
                hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
                
            y_ROI = np.hstack([np.ones(hand_feature_vectors.shape[0]),
                           np.zeros(non_skin_feature_vectors.shape[0])])
            
            X_ROI = np.vstack([hand_feature_vectors, non_skin_feature_vectors])
            
            y_predicted_by_knn = clf_knn_default.predict(X_ROI)
            y_predicted_by_log_reg = clf_log_reg_default.predict(X_ROI)
            
            y_predicted = np.all([y_predicted_by_knn,
                                  y_predicted_by_log_reg], axis=0)
            
            recall = skmet.recall_score(y_ROI, y_predicted)
            precision = skmet.precision_score(y_ROI, y_predicted)
        
            collect_val_recall.append(recall)
            collect_val_precision.append(precision)  
            
            print('confusion matrix:\n', skmet.confusion_matrix(y_ROI, y_predicted))
            print('precision:       ', precision)
            print('recall:          ', recall)
            print('F1:              ', skmet.f1_score(y_ROI, y_predicted))
            print('ROC AUC:         ', skmet.roc_auc_score(y_ROI, y_predicted), '\n')      
            
            # not associated with ROIs
            with open(path + val_test_filename, 'rb') as f:
                obj_data, frame = pickle.load(f)
            
            X = patches.get_valid_blocks_as_feature_vectors(frame, \
                np.ones(frame.shape[0:2]), block_dim)
            
            predicted_labels_by_knn = clf_knn_default.predict(X)
            predicted_labels_by_log_reg = clf_log_reg_default.predict(X)
            
            predicted_labels = np.all([predicted_labels_by_knn,
                                       predicted_labels_by_log_reg], axis=0)
            
            skin_mask = patches.build_mask_from_patch_predictions(frame,
                                                                  predicted_labels,
                                                                  block_dim)
            skin_mask_display = cv2.bitwise_and(frame, frame, mask=skin_mask)
        
    
            cv2.imwrite(out_path + 'classified_patch_skin_mask_' + \
                        val_test_filename[0:-5] + \
                        '_random_forest__block_dim_{:02d}'.format(block_dim) + \
                        '.png', skin_mask_display)
    
        print('Precision:',
              'min {:.2f}, median {:.2f}, mean {:.2f}, max {:.2f}'.format(\
                   np.min(collect_val_precision),
                   np.median(collect_val_precision),
                   np.mean(collect_val_precision),
                   np.max(collect_val_precision)))    
    
        print('Recall   :',
              'min {:.2f}, median {:.2f}, mean {:.2f}, max {:.2f}'.format(\
                   np.min(collect_val_recall),
                   np.median(collect_val_recall),
                   np.mean(collect_val_recall),
                   np.max(collect_val_recall)))
        
def train_all(block_dim):
    '''
    '''
    # paths
    path = './video_copies_clips_track/ROI_masks_patch_detection/'
    feature_vectors = path + "block_feature_vectors/"
    out_path = path + 'log_reg_AND_knn_with_Cleanup_01/'
    
    filenames = [\
    'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
    'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
    'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
    'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
    'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
    'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
    'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
    'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
    'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
    'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
    'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
    'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
    'STI WA May09 B-MVI_0006_trans__600_700_0496.data']

    # get the feature vectors
            
    hand_feature_vectors_collect = []
    non_skin_feature_vectors_collect = []        
    
    for filename in filenames:
        
        print(filename)
    
        with open(feature_vectors + filename[0:-5] + \
                  '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
            
            hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
        
        # I assume there are more non-skin than hand, so verify
        assert hand_feature_vectors.shape[0] < non_skin_feature_vectors.shape[0]
    
        # shuffle, then take a subset of non-skin
        np.random.shuffle(non_skin_feature_vectors)
        
        non_skin_feature_vectors_take = \
            non_skin_feature_vectors[0:hand_feature_vectors.shape[0], :]
        
        # save for further stacking
        hand_feature_vectors_collect.append(hand_feature_vectors)
        non_skin_feature_vectors_collect.append(non_skin_feature_vectors_take)
        
    feature_vectors_hand = np.vstack(hand_feature_vectors_collect)
    feature_vectors_non_skin = np.vstack(non_skin_feature_vectors_collect)
    
    # hand 1 and non-skin 0
    y = np.hstack([np.ones(feature_vectors_hand.shape[0]),
                   np.zeros(feature_vectors_non_skin.shape[0])])
    
    # feature vectors
    X = np.vstack([feature_vectors_hand, feature_vectors_non_skin])
    
    # shuffle
    indices_shuffled = np.arange(0, y.shape[0])
    np.random.shuffle(indices_shuffled)
    
    y = y[indices_shuffled]
    X = X[indices_shuffled, :]
    print('X.shape', X.shape)

    # train knn
    clf_knn_default = skn.KNeighborsClassifier()
    clf_knn_default.fit(X, y)
    
    # train logistic regression
    clf_log_reg_default = sklm.LogisticRegression()
    clf_log_reg_default.fit(X, y)    
    
    return clf_knn_default, clf_log_reg_default
    

def cleanup(skin_mask):
    '''
    '''
    kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (7, 7))

    cleaned = cv2.morphologyEx(skin_mask, cv2.MORPH_OPEN, kernel, iterations=1)
    
    cleaned = fill_holes(cleaned)
    
    cc  = cv2.connectedComponentsWithStats(cleaned)
    
    area_thresh = 4 * np.count_nonzero(kernel)
    
    cleaned = np.any([label == cc[1] for label in range(1, cc[0]) \
                      if cc[2][label, cv2.CC_STAT_AREA] > area_thresh],
                     axis=0).astype(bool).astype(np.uint8) * 255
    
    kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (3, 3))   
    cleaned = cv2.dilate(cleaned, kernel, iterations=1)   

#    cleaned = fill_holes(skin_mask) 
#    
#    kernel = np.array([[0, 1, 1, 1, 0],
#                       [1, 1, 1, 1, 1],
#                       [1, 1, 1, 1, 1],
#                       [1, 1, 1, 1, 1],
#                       [0, 1, 1, 1, 0]])
#    
#    cleaned = cv2.morphologyEx(cleaned, cv2.MORPH_ERODE, kernel, iterations=1)
#    
#    cc  = cv2.connectedComponentsWithStats(cleaned)
#    
#    area_thresh = 8 * np.count_nonzero(kernel)
#    
#    cleaned = np.any([label == cc[1] for label in range(1, cc[0]) \
#                      if cc[2][label, cv2.CC_STAT_AREA] > area_thresh],
#                     axis=0).astype(bool).astype(np.uint8) * 255    
#    
#    cleaned = cv2.morphologyEx(cleaned, cv2.MORPH_DILATE, kernel,
#                               iterations=3) 

    return cleaned


def get_skin_mask(frame, clf_knn_default, clf_log_reg_default, block_dim):
    '''
    '''
    
    X = patches.get_valid_blocks_as_feature_vectors(frame, \
        np.ones(frame.shape[0:2]), block_dim)
    
    predicted_labels_by_knn = clf_knn_default.predict(X)
    predicted_labels_by_log_reg = clf_log_reg_default.predict(X)
    
    predicted_labels = np.all([predicted_labels_by_knn,
                               predicted_labels_by_log_reg], axis=0)
    
    skin_mask = patches.build_mask_from_patch_predictions(frame,
                                                          predicted_labels,
                                                          block_dim)
    return skin_mask    
    
 

#validation()


#cleanup()  

#block_dim = 2
#
#model_knn_for_block_dim_2, model_log_reg_for_block_dim_2 = train_all(block_dim)
#
## paths
#path = './video_copies_clips_track/ROI_masks_patch_detection/'
#model_path = path + 'log_reg_AND_knn_with_Cleanup_01/model/'
#
#with open(model_path + 'model_knn_for_block_dim_2.data', 'wb') as f:
#    pickle.dump((model_knn_for_block_dim_2), f)
#
#with open(model_path + 'model_log_reg_for_block_dim_2.data', 'wb') as f:
#    pickle.dump((model_log_reg_for_block_dim_2), f)

if __name__ == '__main__':
    
    # paths
    path = './video_copies_clips_track/ROI_masks_patch_detection/'
    out_path = path + 'log_reg_AND_knn_with_Cleanup_01/'
    model_path = path + 'log_reg_AND_knn_with_Cleanup_01/model/'
    
    with open(model_path + 'model_knn_for_block_dim_8.data', 'rb') as f:
        model_knn_for_block_dim_8 = pickle.load(f)
    
    with open(model_path + 'model_log_reg_for_block_dim_8.data', 'rb') as f:
        model_log_reg_for_block_dim_8 = pickle.load(f)
    
    block_dim = 2
    
    
    filenames = [\
    'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
    'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
    'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
    'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
    'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
    'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
    'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
    'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
    'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
    'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
    'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
    'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
    'STI WA May09 B-MVI_0006_trans__600_700_0496.data']   
    
    
    for filename in filenames:
        
        print(filename)
        
        with open(path + filename, 'rb') as f:
                obj_data, frame = pickle.load(f)
        
        skin = cv2.imread(out_path + 'classified_patch_skin_mask_' + \
            filename[0:-5] + \
            '_random_forest__block_dim_{:02d}.png'.format(block_dim))
        
        skin_mask = 255 * np.sum(skin.astype(np.float),
                                 axis=2).astype(np.bool).astype(np.uint8)

#        eis = cv2.getTickCount()
#
#        calc = get_skin_mask(frame, model_knn_for_block_dim_11,
#                              model_log_reg_for_block_dim_11, block_dim)
    
        cleaned = cleanup(skin_mask)
        
#        ees = cv2.getTickCount()
#        print((ees - eis)/ cv2.getTickFrequency())
        
        title = 'image'; locate_window(title); cv2.imshow(title, frame)
        
        title = 'skin'; locate_window(title);
        cv2.imshow(title, cv2.bitwise_and(frame, frame, mask=skin_mask))
    
        title = 'cleaned'; locate_window(title);
        cv2.imshow(title, cv2.bitwise_and(frame, frame, mask=cleaned))

        
        cv2.waitKey(0)
        
 
    cv2.destroyAllWindows()
    
#    # watershed
#    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (3, 3))
#    eroded = cv2.erode(cleaned, kernel, iterations=1)
#
#    kernel = kernel = cv2.getStructuringElement(cv2.MORPH_ELLIPSE, (3, 3))
#    dilated = cv2.dilate(cleaned, kernel, iterations=20)
#    
#    unknown = cv2.subtract(dilated, eroded)
#    
#    cc_eroded = cv2.connectedComponents(eroded)
#
#    markers = cc_eroded[1] + 1
#    markers[unknown == 255] = -1
#    
#    markers_display = np.ones(markers.shape, dtype=np.uint8) * 255
#    markers_display[markers == -1] = 0
#    markers_display[markers == 1] = 100
#    
#    title  = 'markers'; locate_window(title);
#    cv2.imshow(title, markers_display)
#
#    markers = cv2.watershed(frame, markers)
#
#    marker_lines_mask = np.ones_like(markers)
#    marker_lines_mask[markers == -1] = 0
#    kernel = np.array([[1, 1, 1], [1, 1, 1], [1, 1, 1]])
#    marker_lines_mask = cv2.morphologyEx(marker_lines_mask.astype(np.uint8),
#                                         cv2.MORPH_ERODE, kernel)
#    
#    cc_marker_lines_mask = cv2.connectedComponentsWithStats(marker_lines_mask)
#    cleaned = np.copy(cc_marker_lines_mask[1])
#    cleaned[cc_marker_lines_mask[1] == \
#        np.argmax(cc_marker_lines_mask[2][:, cv2.CC_STAT_AREA])] = 0
#    cleaned = cleaned.astype(np.bool).astype(np.uint8) * 255
#    
#    title = 'watershed'; locate_window(title);
#    cv2.imshow(title, cv2.bitwise_and(frame, frame, mask=cleaned))    



