# -*- coding: utf-8 -*-
"""
Created on Sun May  6 13:44:29 2018

@author: Callie Darsey

Oh lordy, play around and see how to make boxes.

boxes_1st_attempt_01
"""

import cv2
import numpy as np
import overlay_01 as overlay
import matplotlib.pyplot as plt

def locate_window(title):
    '''
    This function places the window for openCV
    
    '''
    
    move_window = [0, -30]
    #cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.namedWindow(title, cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(title, move_window[0], move_window[1])
    
def get_seg_mag_sum(segment_flow_mag):
    '''
    '''
    collect = []
    
    for flow_mag_display in segment_flow_mag:
        
        collect.append(flow_mag_display[:, :, 0].astype(np.float64))
        
    segment_sum = \
        (np.sum(collect, axis=0) / len(segment_flow_mag)).astype(np.uint8)
        
    segment_sum_display = \
        np.stack([segment_sum, segment_sum, segment_sum], axis=2)
        
    return segment_sum_display


def get_union(segment_flow_mag):
    '''
    '''
    collect = []
    thresholds = []
    
    for flow_mag_display in segment_flow_mag:
        
        flow_mag_display_sorted = np.sort(flow_mag_display[:, :, 0], axis=None)
        
        highest_value_quarter = flow_mag_display_sorted[\
            -int(len(flow_mag_display_sorted) / 32):]
        
        threshold = np.median(highest_value_quarter)
        
        collect.append(flow_mag_display[:, :, 0] > threshold)
        thresholds.append(threshold)
        
    union_mask = np.any(collect, axis=0).astype(np.uint8)
    
    return union_mask, thresholds 
    
    
def overlay_segment(segment_flow_mag, mask, color_bgr, alpha=0.5):
    '''
    '''
    # make a frame the color passed
    color_frame = np.stack([color_bgr[0] * mask,
                            color_bgr[1] * mask,
                            color_bgr[2] * mask],
                           axis=2) 
    
    # make the mask have three channels
    mask = mask.astype(np.bool)
    mask = np.stack([mask, mask, mask], axis=2)
    
    for i, frame in enumerate(segment_flow_mag):
        
        #initialize the overlay frame to be the same as frame
        overlay_frame = np.copy(frame)

        # color the connected components of interest
        np.copyto(overlay_frame, color_frame, where=mask)
            
        # mix
        segment_flow_mag[i] = \
            cv2.addWeighted(frame, 1 - alpha, overlay_frame, alpha, gamma=0)
    
    return segment_flow_mag    
    

def union_skin(segment_skin):
    
    
    step1 = np.stack(segment_skin, axis=2)
    step2 = np.reshape(step1, (step1.shape[0], step1.shape[1], -1))
    
    union_skin = np.any(step2, axis=2).astype(np.uint8)
    
    return union_skin


def combine(crop1, crop2):
    '''
    '''
    mask1 = np.any(crop1 > 50, axis=2)
    mask2 = np.any(crop2 > 50, axis=2)
    
    intersection = np.logical_and(mask1, mask2)
    xor_mask1 = np.logical_xor(mask1, intersection)
    xor_mask2 = np.logical_xor(mask2, intersection)    

    pred_side_1 = np.copy(crop1)
    pred_side_2 = np.copy(crop2)
    
    np.copyto(pred_side_1, crop2,
              where=np.stack([xor_mask2, xor_mask2, xor_mask2], axis=2))
    np.copyto(pred_side_2, crop1,
              where=np.stack([xor_mask1, xor_mask1, xor_mask1], axis=2))

    alpha = 0.5
    beta = 1.0 - alpha
    gamma = 0.0
    
    output = cv2.addWeighted(pred_side_1, alpha, pred_side_2, beta, gamma)
    
    return output


def get_seg_skin_overlays(segment_cleaned_skin, col_1, col_2, row_1, row_2):
    '''
    '''
    
    crop1 = segment_cleaned_skin[0][row_1 : row_2 + 1, col_1 : col_2 + 1, :]
    
    for index in np.arange(1, len(segment_cleaned_skin)):
        
        crop2 = segment_cleaned_skin[index][row_1 : row_2 + 1,
                                            col_1 : col_2 + 1, :]

        crop1 = combine(crop1, crop2)
        
    return crop1
    

#directory = "./wheeler/dataset/STI_WA_Jun20_CT/MVI_0029_trans/"
#name = "MVI_0029_trans__515_557"
directory = "./wheeler/dataset/part1/MVI_0069_trans/"
name = "MVI_0069_trans__0_100"

cap_sampled = cv2.VideoCapture(directory + name + '.sampled.mp4')
cap_skin = cv2.VideoCapture(directory + name + '.cleaned_skin.mp4')
cap_flow_mag = cv2.VideoCapture(directory + 'flow_' + name + '.flow_mag.mp4')


segment = 45

segment_sampled = []
segment_skin = []
segment_skin_watershed = []
segment_flow_mag = []
segment_flow_projection = []
segment_flow_projection_broken = []
segment_skin_projection = []
segment_union = []
segment_comp = []

count = -1
ind = -1

show = 'frame'
          
stats = 'minimum: {:9.6f},  ' + 'median: {:9.6f},  ' + \
        'mean: {:9.6f},  ' + 'maximum: {:10.6f}'
        
title_display = 'display'; locate_window(title_display);

title_flow_mag_sum = 'flow mag. sum'; locate_window(title_flow_mag_sum);
title_markers_sum_before = 'markers sum before';
locate_window(title_markers_sum_before);
title_markers_sum_after = 'markers sum after';
locate_window(title_markers_sum_after);
            
title_dt_on_union = 'DT on union'; locate_window(title_dt_on_union);
title_dt_on_union_thresh = 'DT on union > 10.0';
locate_window(title_dt_on_union_thresh); 
title_markers = 'markers for DT on union'; locate_window(title_markers); 
title_markers_returned = 'markers returned for DT on union';
locate_window(title_markers_returned);

title_skin_overlay = 'skin overlay'; locate_window(title_skin_overlay)
title_skin_WS_overlay = 'skin watershed overlay';
locate_window(title_skin_WS_overlay)

while(cap_sampled.isOpened() and cap_skin.isOpened() and \
      cap_flow_mag.isOpened()):
    
    success_flag_sampled, frame = cap_sampled.read()
    success_flag_skin, skin = cap_skin.read()
    success_flag_flow_mag, flow_mag_display = cap_flow_mag.read()
    
    if success_flag_sampled == True and success_flag_skin == True and \
        success_flag_flow_mag == True:
        
        count = count + 1
        
        cv2.putText(frame, 'segment ' + str(int(np.floor(count  / 45))),
                    (25, 25), cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 255, 0), 1)
        
        rows, cols, channels = frame.shape
        cv2.putText(frame, str(count * 4), (cols - 100, 25),
                    cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2) 
        
        # watershed skin
        skin_mask_thr = np.any(skin > 50, axis=2).astype(np.uint8)

        
        dilate_skin_mask_thr = cv2.dilate(skin_mask_thr, kernel=np.ones((3,3)),
                                          iterations=80)
        
        unknown_skin_watershed = cv2.subtract(dilate_skin_mask_thr,
                                              skin_mask_thr)
        
        cc_skin_mask_thr = cv2.connectedComponents(skin_mask_thr)
        
        markers_skin_watershed = cc_skin_mask_thr[1] + 1
        
        markers_skin_watershed[unknown_skin_watershed > 0] = -1
        
        # call the watershed
        cv2.watershed(frame, markers_skin_watershed)
        
        cc_markers_skin_watershed = cv2.connectedComponentsWithStats(\
            markers_skin_watershed.astype(np.bool).astype(np.uint8))
            
        skin_watershed = markers_skin_watershed.copy()
        
        skin_watershed[skin_watershed == -1] = 0
        skin_watershed[skin_watershed == np.argmax(\
            cc_markers_skin_watershed[2][:, cv2.CC_STAT_AREA])] = 0
        
        skin_watershed_display = cv2.bitwise_and(frame, frame,
            mask=skin_watershed.astype(np.bool).astype(np.uint8))

        
        segment_sampled.append(frame)
        segment_skin.append(skin)
        segment_skin_watershed.append(skin_watershed_display)
        segment_flow_mag.append(flow_mag_display)
        segment_flow_projection.append(frame)
        segment_flow_projection_broken.append(frame)
        segment_skin_projection.append(frame)
        segment_union.append(frame)
        segment_comp.append(frame)
    
        if len(segment_sampled) > segment:
            segment_sampled.pop(0)
            segment_skin.pop(0)
            segment_skin_watershed.pop(0)
            segment_flow_mag.pop(0)
            segment_flow_projection.pop(0)
            segment_flow_projection_broken.pop(0)
            segment_skin_projection.pop(0)
            segment_union.pop(0)
            segment_comp.pop(0)
        else:
            ind = ind + 1
            
        
        if show == 'frame':            
            cv2.imshow(title_display, segment_sampled[ind])            
        elif show == 'skin':            
            cv2.imshow(title_display, segment_skin[ind])
        elif show == 'skin watershed':
            cv2.imshow(title_display, segment_skin_watershed[ind])
        elif show == 'mag':
            cv2.imshow(title_display, segment_flow_mag[ind])
        elif show == 'union overlay':
            cv2.imshow(title_display, segment_union[ind])
        elif show == 'comp':
            cv2.imshow(title_display, segment_comp[ind])


        # catch a key press
        k = cv2.waitKey(1)
        
        # Esc key press will escape
        if k == 27:
    
            # close all the OpenCV windows
            cv2.destroyAllWindows()
            
            break
        
        # space key will pause
        elif k == 32 or (count != 0 and (count + 1) % segment == 0):

            k = -1

            ''' display the sum of flow magnitude'''
            flow_mag_sum = get_seg_mag_sum(segment_flow_mag)
            
            flow_mag_sum_log_display = \
                (np.log(flow_mag_sum + 1) * 255 / \
                np.max(np.log(flow_mag_sum + 1))).astype(np.uint8)
                
            cv2.imshow(title_flow_mag_sum, flow_mag_sum_log_display)
            
            cv2.imwrite(directory + 'sum_flow_mag_through_time/' + name + \
                        '_segment ' + str(int(np.floor(count  / 45))) + '.png',
                        flow_mag_sum_log_display)
                    
            # make the markers
            marks = (cv2.erode(flow_mag_sum[:,:,0], np.ones((3,3)),
                               iterations=20) > 4).astype(np.uint8)
            
            dlt = (cv2.dilate(flow_mag_sum[:,:,0], kernel=np.ones((3,3)),
                              iterations=20) > 4).astype(np.uint8)
            
            unknown_sum = cv2.subtract(dlt, marks)
            
            cc_sum = cv2.connectedComponents(marks)
            
            markers_sum = cc_sum[1] + 1
            
            markers_sum[unknown_sum > 0] = -1
            
            cv2.imshow(title_markers_sum_before,
                ((markers_sum.astype(np.float64) + 1.0) / \
                 np.max(markers_sum.astype(np.float64) + 1.0) * \
                 (2**16 - 1)).astype(np.uint16))
            
            # call the watershed
            cv2.watershed(flow_mag_sum, markers_sum)

            cv2.imshow(title_markers_sum_after,
                ((markers_sum.astype(np.float64) + 1.0) / \
                 np.max(markers_sum.astype(np.float64) + 1.0) * \
                 (2**16 - 1)).astype(np.uint16))
            
            flow_mag_sum_post_watershed = (markers_sum > 1).astype(np.uint8)
            
            flow_mag_sum_returned = overlay_segment([flow_mag_sum],
                                            flow_mag_sum_post_watershed,
                                            (255, 0, 0), alpha=0.25)
            
#            # display          
#            title_sum = 'sum'; locate_window(title_sum);
#            cv2.imshow(title_sum, flow_mag_sum_returned[0])
            

            '''get the flow magnitude thresholded union over the segment'''
            union_mask, thresholds = get_union(segment_flow_mag)
            
            segment_flow_projection = overlay_segment(segment_flow_projection,
                                                      union_mask,
                                                     (0, 255, 255), alpha=0.25)
            
            
            # get distance transform on union mask
            dt_union_mask = cv2.distanceTransform(union_mask,
                                            distanceType=cv2.DIST_L2,
                                            maskSize=cv2.DIST_MASK_PRECISE)
            
            # normalize the distance transform for to UINT8C3
            dt_union_mask_normalized = \
                ((dt_union_mask - np.min(dt_union_mask)) * (255 - 0) / \
                 (np.max(dt_union_mask) - np.min(dt_union_mask)) + (0)\
                 ).astype(np.uint8)
            
            dt_union_mask_normalized = cv2.merge([dt_union_mask_normalized,
                                                  dt_union_mask_normalized,
                                                  dt_union_mask_normalized])
            
            cv2.imshow(title_dt_on_union, dt_union_mask_normalized)
        
            # prep the markers
            union_mask_reduced = (dt_union_mask > 10.0).astype(np.uint8)
            
            cv2.imshow(title_dt_on_union_thresh,
                       cv2.bitwise_and(dt_union_mask_normalized,
                                       dt_union_mask_normalized,
                                       mask=union_mask_reduced))
            
            dilated = cv2.dilate(union_mask_reduced, np.ones((3, 3)),
                                 iterations=30)
            
            unknown = cv2.subtract(dilated, union_mask_reduced)
            
            cc = cv2.connectedComponents(union_mask_reduced)
            
            markers = cc[1] + 1
            
            markers[unknown > 0] = -1
            
            cv2.imshow(title_markers, ((markers.astype(np.float64) + 1.0) / \
                np.max(markers.astype(np.float64) + 1.0) * \
                (2**16 - 1)).astype(np.uint16))
            
            # call the watershed
            cv2.watershed(dt_union_mask_normalized, markers)

            cv2.imshow(title_markers_returned,
                ((markers.astype(np.float64) + 1.0) / \
                 np.max(markers.astype(np.float64) + 1.0) * \
                 (2**16 - 1)).astype(np.uint16))
            
            union_mask_post_watershed = (markers > 1).astype(np.uint8)
            
            title_WS_comp = 'watershed components on union';
            locate_window(title_WS_comp)
            cv2.imshow(title_WS_comp, union_mask_post_watershed * 255)
            
            # combine with flow mag. sum watershed lines
            lines = (markers_sum==-1)
            
            title_lines_flow_mag_sum_WS = 'lines from flow mag sum WS';
            locate_window(title_lines_flow_mag_sum_WS)
            cv2.imshow(title_lines_flow_mag_sum_WS, lines.astype(np.uint8) * 255)
            
            xor = np.logical_xor(lines, union_mask_post_watershed)
            union_mask_broken = xor.astype(np.int) - lines.astype(np.int)
            union_mask_broken[union_mask_broken == -1] = 0
            
            
            
            segment_flow_projection_broken = \
                overlay_segment(segment_flow_projection_broken,
                                union_mask_broken.astype(np.uint8),
                                (0, 255, 255), alpha=0.25)  
            
            segment_union = overlay_segment(segment_union,
                                            union_mask_broken.astype(np.uint8),
                                            (0, 255, 255), alpha=0.25)
                                    
            print('Stats of Thresholds on the Segment')
            print(stats.format(np.min(thresholds), np.median(thresholds),
                               np.mean(thresholds), np.max(thresholds)))
            
            '''union skin mask'''
            union_skin_mask = union_skin(segment_skin)
            
            segment_skin_projection = overlay_segment(segment_skin_projection,
                union_skin_mask, (0, 0, 255), alpha=0.25)
            
            segment_union = overlay_segment(segment_union, union_skin_mask,
                                            (0, 0, 255), alpha=0.25)
            
            
            '''settle the components'''
            cc = cv2.connectedComponentsWithStats(\
                union_mask_broken.astype(np.uint8), connectivity=4)
            
            component_masks = []
            
            for label in np.arange(1, cc[0]):
                
                comp_mask = (cc[1] == label)
                
                overlap = np.logical_and(comp_mask, union_skin_mask)
                
                overlap_count = np.count_nonzero(overlap)
                
                comp_area = cc[2][label, cv2.CC_STAT_AREA]
                
                if overlap_count > 0:
                    
                    if overlap_count / comp_area < 0.5:
                        
                        component_masks.append(overlap)
                        
                    else:
                        
                        component_masks.append(comp_mask)
                        
            settled_comp_mask = np.any(component_masks,
                                       axis=0).astype(np.uint8)
            
            segment_comp = overlay_segment(segment_comp, settled_comp_mask,
                                           (0, 255, 0), alpha=0.25)
            
            
                        
            # overlay the skin watershed
            rows, cols, channels = segment_skin_watershed[0].shape
            skin_watershed_combined = \
                get_seg_skin_overlays(segment_skin_watershed, 0, cols, 0, rows)
                
#            plt.figure(figsize=(10, 10))
#            plt.imshow(cv2.merge([skin_watershed_combined[:, :, 2],
#                                  skin_watershed_combined[:, :, 1],
#                                  skin_watershed_combined[:, :, 0]]))
#            plt.show() 
            cv2.imshow(title_skin_WS_overlay, skin_watershed_combined)   
                
            # overlay the cleaned skin 
            rows, cols, channels = segment_skin[0].shape
            skin_combined = \
                get_seg_skin_overlays(segment_skin, 0, cols, 0, rows)
                
#            plt.figure(figsize=(10, 10))
#            plt.imshow(cv2.merge([skin_combined[:, :, 2],
#                                  skin_combined[:, :, 1],
#                                  skin_combined[:, :, 0]]))
#            plt.show()
            cv2.imshow(title_skin_overlay, skin_combined)             
                                       

            # until we get another space key
            allow_scrolling = True
            while(allow_scrolling):

                k = cv2.waitKey(1)
                
                if k != -1:
                    print(k)
                
                # space or escape
                if k == 32 or k == 27:
    
                    allow_scrolling = False
                
                # < key
                elif k == 44:
                    
                    if ind > 0:
                        
                        ind = ind - 1
                        
                        if show == 'frame':
                            
                            cv2.imshow(title_display, segment_sampled[ind]) 
                            
                        elif show == 'skin':
                            
                            cv2.imshow(title_display, segment_skin[ind])
                            
                        elif show == 'skin watershed':
                            
                            cv2.imshow(title_display, segment_skin_watershed[ind])
                            
                        elif show == 'mag':
                            
                            cv2.imshow(title_display, segment_flow_mag[ind])
                            
                        elif show == 'union overlay':

                            cv2.imshow(title_display, segment_union[ind]) 
                            
                        elif show == 'comp':
                            
                            cv2.imshow(title_display, segment_comp[ind])

                        elif show == 'flow union':
                        
                            cv2.imshow(title_display,
                                       segment_flow_projection[ind])

                        elif show == 'flow union broken':
                        
                            cv2.imshow(title_display,
                                       segment_flow_projection_broken[ind]) 

                        elif show == 'skin mask union':
                        
                            cv2.imshow(title_display,
                                       segment_skin_projection[ind]) 
                                
                # > key
                elif k == 46:
                    
                    if ind < len(segment_sampled) - 1:
                        
                        ind = ind + 1
                        
                        if show == 'frame':
                            
                            cv2.imshow(title_display, segment_sampled[ind])  
                            
                        elif show == 'skin': 
                            
                            cv2.imshow(title_display, segment_skin[ind])

                        elif show == 'skin watershed':
                            
                            cv2.imshow(title_display, segment_skin_watershed[ind])
                            
                        elif show == 'mag':
                            
                            cv2.imshow(title_display, segment_flow_mag[ind])
                            
                        elif show == 'union overlay':
                            
                            cv2.imshow(title_display, segment_union[ind]) 
                            
                        elif show == 'comp':
                            
                            cv2.imshow(title_display, segment_comp[ind])
                            
                        elif show == 'flow union':
                        
                            cv2.imshow(title_display,
                                       segment_flow_projection[ind])
                            
                        elif show == 'flow union broken':
                        
                            cv2.imshow(title_display,
                                       segment_flow_projection_broken[ind])

                        elif show == 'skin mask union':
                        
                            cv2.imshow(title_display,
                                       segment_skin_projection[ind]) 
                             
                # f key
                elif k == 102:
                    
                    show = 'frame'
                    
                    cv2.imshow(title_display, segment_sampled[ind])            

                        
                # s key
                elif k == 115:
                    
                    show = 'skin'
                        
                    cv2.imshow(title_display, segment_skin[ind])

                        
                # m key
                elif k == 109:
                    
                    show = 'mag'
                    
                    cv2.imshow(title_display, segment_flow_mag[ind])
                    
                # u key
                elif k == 117:
                
                    show = 'union overlay'
                    
                    cv2.imshow(title_display, segment_union[ind])
                    
                # c key
                elif k == 99:
                    
                    show = 'comp'

                    cv2.imshow(title_display, segment_comp[ind])
                    
                # w key
                elif k == 119:
                    
                    show = 'skin watershed'
                            
                    cv2.imshow(title_display, segment_skin_watershed[ind])
                    
                # p key
                elif k == 112:
                    
                    show = 'flow union'
                        
                    cv2.imshow(title_display, segment_flow_projection[ind])

                # b key
                elif k == 98:
                    
                    show = 'flow union broken'
                        
                    cv2.imshow(title_display,
                               segment_flow_projection_broken[ind])  

                # k key
                elif k == 107:
                    
                    show = 'skin mask union'
                        
                    cv2.imshow(title_display,
                               segment_skin_projection[ind]) 

                    
    #            According to "A Byte of Python", "The else block is executed when
    #            the while loop condition becomes False - this may even be the first
    #            time that the condition is checked. If there is an else clause for
    #            a while loop, it is always executed unless you break out of the
    #            loop with a break statement."
            
            else:
                
                # escape
                if k == 27:
        
                    # close all the OpenCV windows
                    cv2.destroyAllWindows()
                    
                    break
                
                # space
                if k == 32:
                    
                    get_back = True
                    
                    while(get_back):
                        
                        if ind + 1 == len(segment_sampled):
                            
                            get_back = False
                            
                        else:
                            
                            ind = ind + 1
                            
                            cv2.imshow(title_display, segment_sampled[ind])
                            
                            k = cv2.waitKey(1)
                            
                            if k == 27:
                                
                                get_back = False
    
                    else:
                        
                        # escape
                        if k == 27:
    
                            # close all the OpenCV windows
                            cv2.destroyAllWindows()
                            
                            break   


        
    else:
        
        print("No more frames")
        
        cap_sampled.release()
        cv2.destroyAllWindows()     
        
        break        
else:
    cap_sampled.release()
    cv2.destroyAllWindows()
    