# -*- coding: utf-8 -*-
"""
Created on Mon Apr  2 17:06:22 2018

@author: Callie Darsey

Figure how to break a region based on motion magnitude
"""


import cv2
import numpy as np
import pickle
import os
import fns_investigating_optical_flow_01 as fns
from matplotlib import pyplot as plt
import get_lessened_features_and_save_steps_01 as get
import overlay_01 as overlay
import component_functions_01 as comp_fns


# video path
common_path = './video_copies_clips_track/copies_and_clips_facilitator_C/'
specific_path = 'STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/'
video_name = 'MVI_0028_trans__200_208.mp4'



regions_path = 'adjust_review_03/region_masks/'

for f1n in range(0, 464, 4):
    
    regions_mask_name = 'region_mask_' + '{:04d}'.format(f1n) + '.data'
    regions_mask_path = common_path + specific_path + regions_path + \
                        regions_mask_name
        
    if os.path.isfile(regions_mask_path):
        
        print('{:082d}'.format(f1n))

        flow_name = 'flow_MVI_0028_trans__200_208_' + '{:04d}'.format(f1n) + \
                     '_' + '{:04d}'.format(f1n + 4) + '.data'
        
        # load pickled flow    
        with open(common_path + specific_path + flow_name, 'rb') as f:
            frame1, frame2, flow = pickle.load(f)
        
        # load pickled region
        with open(regions_mask_path, 'rb') as f:
            region_mask = pickle.load(f)
        
        # calc flow mag
        flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
            
        # save image displaying the regions overlay in bgr
        connectivity = 4
        cc_regions = cv2.connectedComponents(region_mask,
                                             connectivity, cv2.CV_32S)
        
        regions_overlay = overlay.overlay(frame1, cc_regions,
                                          np.unique(cc_regions[1])[1:],
                                          (0, 255, 255), alpha=0.75)
        
        # save and display the trigger regions
        tmp = np.copy(regions_overlay)
        fns.draw_flow(tmp, flow, np.ones_like(region_mask), step=4)
        
        for label in range(1, cc_regions[0]):
            
            flow_mag_region = flow_mag[label == cc_regions[1]]
            
            print('{:80s} size'.format(' '))
            print('{:85d}'.format(flow_mag_region.size))
            print('{:80s} std'.format(' '))
            print('{:85.2f}'.format(np.std(flow_mag_region)))
            print(' ')
            
#            f_toss = plt.figure(figsize=(4, 3))
#            bp = plt.boxplot(flow_mag_region)
#            plt.show()
        
        # prep window
        title = 'display window'
        move_window = [0, -30]
        cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
        cv2.moveWindow(title, move_window[0], move_window[1])
        
        flow_mag_display = (255 * flow_mag / np.max(flow_mag)).astype(np.uint8)
        flow_mag_display_C3 = np.stack([flow_mag_display,
                                        flow_mag_display,
                                        flow_mag_display], axis=2)
    
        
        flow_mag_display_nonzero = flow_mag_display[flow_mag_display != 0]
        bpfm = plt.boxplot(flow_mag_display_nonzero.reshape((-1, 1)))
        plt.close()
        
        low_whisker = np.min(bpfm['whiskers'][0].get_data()[1])
        Q1 = np.min(bpfm['boxes'][0].get_data()[1])
        median = bpfm['medians'][0].get_data()[1][1]
        Q3 = np.max(bpfm['boxes'][0].get_data()[1])
        high_whisker = np.max(bpfm['whiskers'][1].get_data()[1])
        print(low_whisker, Q1, median, Q3, high_whisker)
        
        fg = flow_mag_display >= high_whisker
        unknown = np.all([flow_mag_display > Q3,
                          flow_mag_display < high_whisker], axis=0)
    
        cc_mask = cv2.connectedComponents(fg.astype(np.uint8), connectivity,
                                          cv2.CV_32S)
        
        markers = np.copy(cc_mask[1])
        markers = markers + 1
        markers[unknown] = 0
        markers_orig = np.copy(markers)
        
        markers = cv2.watershed(flow_mag_display_C3, markers)
        
        cv2.imshow(title, flow_mag_display_C3)
        cv2.imshow(title+'2',
            np.sum([fg.astype(np.bool).astype(np.uint8)*255,
                    unknown.astype(np.bool).astype(np.uint8)*100], axis=0, dtype=np.uint8))
#        cv2.imshow(title+'3', markers)
        
        k = cv2.waitKey(0)
        if k == 27:
            break
        
cv2.destroyAllWindows()
        
