# -*- coding: utf-8 -*-
"""
Created on Mon Apr 23 09:33:34 2018

@author: Callie Darsey

Classify skin patches from withheld ROI collected images
"""

import pickle
import os
import numpy as np
import sklearn.neighbors as skn
import sklearn.svm as sksvm
import sklearn.model_selection as skm
import sklearn.metrics as skmet
import sklearn.linear_model as skl
import patches_01 as patches
import cv2
import matplotlib.pyplot as plt

def plot_precision_recall_vs_threshold(precisions, recalls, thresholds):
    '''
    Hands-On Machine Learning with Scikit-Learn & TensorFlow 
    by Aurelien Geron for Chapter 3: Classification page 89
    '''
    plt.plot(thresholds, precisions[:-1], "b--", label="Precision")
    plt.plot(thresholds, recalls[:-1], "g-", label="Recall")
    plt.xlabel("Threshold")
    plt.legend(loc="upper left")
    plt.ylim([0, 1])

path = './video_copies_clips_track/ROI_masks_patch_detection/'
tm = path + 'trained_models/'
bfv = path + "block_feature_vectors/"
wh = path + 'classified_witheld/'

#sgdtm = './wheeler/sgd/'
sgdtm = './wheeler/sgd_second/'

model_dir = './wheeler/svc_default/initial_training/'

#'''test for initial training'''
#filenames = [\
#'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
#'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
##'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
#'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
##'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
#'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
##'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
#'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
##'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
#'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
##'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
#'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
##'STI WA May09 B-MVI_0006_trans__600_700_0496.data',
#]

'''test for second training'''
filenames = [\
#'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
#'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
#'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
#'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
#'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
#'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
#'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
'STI WA May09 B-MVI_0006_trans__600_700_0496.data',
]

k = 5

for block_dim in [2, 3, 4, 5]:
    
    print('\nblock dimension ({:d}, {:d})'.format(block_dim, block_dim))
    
#    with open(tm + 'knn_{:03}_block_dim_{:02d}.data'.format(k, block_dim), \
#              'rb') as f:
        
    with open(model_dir + \
              'svc_default_block_dim_{:02d}.data'.format(block_dim), \
              'rb') as f:
        
        model = pickle.load(f)
        
    # save for the precision / recall curve
    y_ROI_collect = []
    y_decision_function_collect = []
            
    for filename in filenames:
        
        print(filename)
        
        # associated with ROIs
        with open(bfv + filename[0:-5] + \
                  '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
            
            hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
            
        y_ROI = np.hstack([np.ones(hand_feature_vectors.shape[0]),
                       np.zeros(non_skin_feature_vectors.shape[0])])
        
        X_ROI = np.vstack([hand_feature_vectors, non_skin_feature_vectors])
        
        #y_predicted = model.predict(X_ROI)
        y_decision_function = model.decision_function(X_ROI)
        y_predicted = (y_decision_function > 0).astype(np.float64)
        
        print('y_ROI.shape              ', y_ROI.shape)
        print('y_decision_function.shape', y_decision_function.shape)
        
        y_ROI_collect.append(y_ROI)
        y_decision_function_collect.append(y_decision_function)
        
        print('\nconfusion matrix:\n', skmet.confusion_matrix(y_ROI, y_predicted))
        print('precision:', skmet.precision_score(y_ROI, y_predicted))
        print('recall:', skmet.recall_score(y_ROI, y_predicted))
        print('F1:', skmet.f1_score(y_ROI, y_predicted))
        print('ROC AUC:', skmet.roc_auc_score(y_ROI, y_predicted))      
        
        # not associated with ROIs
        with open(path + filename, 'rb') as f:
            obj_data, frame = pickle.load(f)
        
        X = patches.get_valid_blocks_as_feature_vectors(frame, \
            np.ones(frame.shape[0:2]), block_dim)
        
        #predicted_labels = model.predict(X)
        y_decision_function = model.decision_function(X)
        predicted_labels = (y_decision_function > 0).astype(np.float64)

        skin_mask = patches.build_mask_from_patch_predictions(frame,
                                                              predicted_labels,
                                                              block_dim)
        skin_mask_display = cv2.bitwise_and(frame, frame, mask=skin_mask)
    
#        cv2.imwrite(wh + 'classified_patch_skin_mask_' + \
#                    filename[0:-5] + \
#                    'knn_{:03}_block_dim_{:02d}'.format(k, block_dim) + \
#                    '.png', skin_mask_display)

        cv2.imwrite(wh + 'classified_patch_skin_mask_' + \
                    filename[0:-5] + \
                    '_svc_default_block_dim_{:02d}'.format(block_dim) + \
                    '.png', skin_mask_display)    
    
    y_ROI_collect = np.hstack(y_ROI_collect)
    y_decision_function_collect = np.hstack(y_decision_function_collect)
    
    precisions, recalls, thresholds = \
        skmet.precision_recall_curve(y_ROI_collect, y_decision_function_collect)
    
    plot_precision_recall_vs_threshold(precisions, recalls, thresholds)
    plt.show()
    
        
'''
runfile('C:/Users/Callie/Thesis/an_investigation_of_optical_flow/classify_withheld_01.py', wdir='C:/Users/Callie/Thesis/an_investigation_of_optical_flow')
Reloaded modules: patches_01

block dimension (2, 2)
STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data

confusion matrix:
 [[171757  21408]
 [   141   3191]]
precision: 0.129720720354
recall: 0.957683073229
F1: 0.228491640113
ROC AUC: 0.923427771181
STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data

confusion matrix:
 [[137006  32663]
 [    83   3160]]
precision: 0.088211484242
recall: 0.974406413814
F1: 0.161777504736
ROC AUC: 0.890948145582
STI Polk May04 F-MVI_0079_trans__390_415_0148.data

confusion matrix:
 [[184635  22060]
 [   253   3798]]
precision: 0.14687910898
recall: 0.937546284868
F1: 0.25397037681
ROC AUC: 0.915409490677
STI WA Apr18 B-MVI_0029_trans__75_125_0212.data

confusion matrix:
 [[182482  17584]
 [   130   2480]]
precision: 0.12360446571
recall: 0.950191570881
F1: 0.218752756461
ROC AUC: 0.931150287455
STI WA Mar28 C-MVI_0013_trans__110_140_0768.data

confusion matrix:
 [[187999  21610]
 [   129   2623]]
precision: 0.108240828622
recall: 0.953125
F1: 0.194404298684
ROC AUC: 0.925014140912
STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data

confusion matrix:
 [[157432  13105]
 [   379   5926]]
precision: 0.311386684883
recall: 0.939888977002
F1: 0.467792863909
ROC AUC: 0.931521741531
STI WA May09 A-MVI_0110_trans__100_200_2384.data

confusion matrix:
 [[192525  20161]
 [   605   2442]]
precision: 0.108038755917
recall: 0.801444043321
F1: 0.190409356725
ROC AUC: 0.853325860183

block dimension (3, 3)
STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data

confusion matrix:
 [[75680  9581]
 [   33  1366]]
precision: 0.124783045583
recall: 0.976411722659
F1: 0.221286246558
ROC AUC: 0.932019562787
STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data

confusion matrix:
 [[59672 14901]
 [   17  1333]]
precision: 0.0821116175927
recall: 0.987407407407
F1: 0.151615104641
ROC AUC: 0.893794889522
STI Polk May04 F-MVI_0079_trans__390_415_0148.data

confusion matrix:
 [[80596 10561]
 [   65  1637]]
precision: 0.134202328251
recall: 0.961809635723
F1: 0.235539568345
ROC AUC: 0.92297728624
STI WA Apr18 B-MVI_0029_trans__75_125_0212.data

confusion matrix:
 [[80283  8040]
 [   48  1014]]
precision: 0.111994698476
recall: 0.954802259887
F1: 0.200474495848
ROC AUC: 0.931886371613
STI WA Mar28 C-MVI_0013_trans__110_140_0768.data

confusion matrix:
 [[83029  9613]
 [   36  1122]]
precision: 0.104517931998
recall: 0.968911917098
F1: 0.188682418229
ROC AUC: 0.932573443059
STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data

confusion matrix:
 [[69055  6151]
 [  188  2457]]
precision: 0.285432156134
recall: 0.928922495274
F1: 0.436683551053
ROC AUC: 0.923566904101
STI WA May09 A-MVI_0110_trans__100_200_2384.data

confusion matrix:
 [[85148  9090]
 [  221  1048]]
precision: 0.103373446439
recall: 0.825847123719
F1: 0.183746822127
ROC AUC: 0.864694609632

block dimension (4, 4)
STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data

confusion matrix:
 [[42121  5451]
 [    9   737]]
precision: 0.119101486749
recall: 0.987935656836
F1: 0.212575713874
ROC AUC: 0.936675723819
STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data

confusion matrix:
 [[32849  8747]
 [   48   665]]
precision: 0.0706544836379
recall: 0.932678821879
F1: 0.131358024691
ROC AUC: 0.861197089563
STI Polk May04 F-MVI_0079_trans__390_415_0148.data

confusion matrix:
 [[44700  6307]
 [   19   885]]
precision: 0.123053392659
recall: 0.978982300885
F1: 0.218626482213
ROC AUC: 0.927666302873
STI WA Apr18 B-MVI_0029_trans__75_125_0212.data

confusion matrix:
 [[44238  5040]
 [   23   530]]
precision: 0.0951526032316
recall: 0.958408679928
F1: 0.173117752736
ROC AUC: 0.928065900904
STI WA Mar28 C-MVI_0013_trans__110_140_0768.data

confusion matrix:
 [[46150  5763]
 [   22   595]]
precision: 0.0935828877005
recall: 0.964343598055
F1: 0.170609318996
ROC AUC: 0.926665471133
STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data

confusion matrix:
 [[37591  4074]
 [  132  1264]]
precision: 0.236792806294
recall: 0.905444126074
F1: 0.375408375408
ROC AUC: 0.903832107439
STI WA May09 A-MVI_0110_trans__100_200_2384.data

confusion matrix:
 [[46962  5638]
 [   98   564]]
precision: 0.0909384069655
recall: 0.851963746224
F1: 0.164335664336
ROC AUC: 0.872388717218

block dimension (5, 5)
STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data

confusion matrix:
 [[26561  3782]
 [   24   419]]
precision: 0.0997381575815
recall: 0.945823927765
F1: 0.18044788975
ROC AUC: 0.910591165016
STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data

confusion matrix:
 [[20985  5486]
 [   24   399]]
precision: 0.0677994902294
recall: 0.943262411348
F1: 0.126506024096
ROC AUC: 0.86800837314
STI Polk May04 F-MVI_0079_trans__390_415_0148.data

confusion matrix:
 [[28164  4166]
 [   29   520]]
precision: 0.110968843363
recall: 0.947176684882
F1: 0.198662846227
ROC AUC: 0.90915901983
STI WA Apr18 B-MVI_0029_trans__75_125_0212.data

confusion matrix:
 [[28128  3115]
 [   14   301]]
precision: 0.0881147540984
recall: 0.955555555556
F1: 0.161350844278
ROC AUC: 0.927926611116
STI WA Mar28 C-MVI_0013_trans__110_140_0768.data

confusion matrix:
 [[29209  3796]
 [   13   366]]
precision: 0.08793849111
recall: 0.965699208443
F1: 0.161197974015
ROC AUC: 0.925343165803
STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data

confusion matrix:
 [[23616  2789]
 [   78   764]]
precision: 0.215029552491
recall: 0.907363420428
F1: 0.347667804323
ROC AUC: 0.900869742783
STI WA May09 A-MVI_0110_trans__100_200_2384.data

confusion matrix:
 [[29927  3662]
 [   53   347]]
precision: 0.086555250686
recall: 0.8675
F1: 0.157405307326
ROC AUC: 0.879238106225
'''