# -*- coding: utf-8 -*-
"""
Created on Sat Feb 17 15:57:18 2018

@author: Callie Darsey

Given  the resized color frame, its flow, and exemplar(s) information for the
frame, form the three versions of  keep (1) / toss (0) components

component_exploration_02
"""



import pickle
import numpy as np
import cv2
import skin_detector_01 as skin_detector
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import overlay_01 as overlay
import component_functions_01 as comp_fns
import example_of_DFT_01 as dft_hist_fns
import component_exploration_02 as label_fns
import exploring_histogram_creation_01 as my_print


def force_neg_pi_to_pi(value):
    '''
    This function takes the value and adds or subtracts pi until a value in
    [-pi, pi] is reached and returned.
    '''
    while value > np.pi or value < -np.pi:
        
        if value > np.pi:
            
            value = value - 2 * np.pi
            
        elif value < -np.pi:
            
            value = value + 2 * np.pi
            
    return value
    

def get_flow_mag_in_phase_bins(flow, mask, start, bin_size):
    '''
    Return a histogram with flow phase bins [start, start + 1 * bin_size),
    [start + 2 * bin_size), ..., [start + n * size, start) adjusted such that
    the bins capture angles around the unit circle taking values [-pi, pi].
    
    For example, if start = -np.pi / 8 and bin_size = np.pi / 4, the bins are
    
    [    -pi / 8,      pi / 8),
    [     pi / 8,  3 * pi / 8),
    [ 3 * pi / 8,  5 * pi / 8),
    [ 5 * pi / 8,  7 * pi / 8),
    [ 7 * pi / 8, -7 * pi / 8),
    [-7 * pi / 8, -5 * pi / 8),
    [-5 * pi / 8, -3 * pi / 8),
    [-3 * pi / 8,     -pi / 8)
    
    Three versions of this histogram are returned.
    
    1) The value in each bin will be the sum of the flow magnitude values of
       pixels whose flow phase was within the phase bin range.
    
    2) The histogram described in 1) with each value divided by the phase bin
       count (if the count is nonzero)
       
    3) The histogram described in 1) with each value divided by the mask (i.e.
        component) area

    Inputs:
        
    flow - two channel array holding the bin sizes
    mask - mask of the connected component
    start - angle in radians to start the phase bins at
    bin_size - angle in radians of the bin widths

    '''
    assert bin_size <= 2 * np.pi and bin_size > 0
    
    # force start to be in [-pi, pi]
    start = force_neg_pi_to_pi(start)
    
    # set up the bin edges
    bin_edges = [start]
    
    angles_travelled = 0
    while True:
         
        angles_travelled = angles_travelled + bin_size
        
        if angles_travelled >= 2 * np.pi:
            
            bin_edges.append(start)
            
            break
        
        else:

            new_edge = force_neg_pi_to_pi(bin_edges[-1] + bin_size)
            
            bin_edges.append(new_edge)

    # get the area of the mask (i.e. the area of the component)
    area = np.count_nonzero(mask)

    # get the flow into magnitude and angle form
    flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
    flow_phase = cv2.phase(flow[:, :, 0], flow[:, :, 1])
        
    if area == 16703:
        
        with open('flow_mag_phase.data', 'wb') as f:
            pickle.dump((flow_mag[mask > 0], flow_phase[mask > 0]), f)

    # find the sums and counts for the bins
    
    bin_sums = np.zeros((1, len(bin_edges)-1), dtype=np.float32)
    bin_counts = np.zeros((1, len(bin_edges)-1), dtype=np.float32)
    
    r_ind, c_ind = np.where(mask)
    
    for r, c in zip(r_ind, c_ind):
        
        flow_phase_value = force_neg_pi_to_pi(flow_phase[r, c])      
        
        for  i, edge_pair in enumerate(zip(bin_edges[0:-1], bin_edges[1:])):
            
            edge_inclusive, edge_exclusive = edge_pair
            
            # The bin does not cross pi on unit circle
            if edge_inclusive < edge_exclusive:
                
                if edge_inclusive <= flow_phase_value and \
                                     flow_phase_value < edge_exclusive:
                                       
                    bin_sums[0, i] = bin_sums[0, i] + flow_mag[r, c]
                    
                    bin_counts[0, i] = bin_counts[0, i] + 1
                    
                    break
                    
            # The bin crosses pi on the unit circle
            elif edge_inclusive > edge_exclusive:
                
                if edge_inclusive <= flow_phase_value and \
                                     flow_phase_value <= np.pi or \
                           -np.pi <= flow_phase_value and \
                                     flow_phase_value < edge_exclusive:
                
                    bin_sums[0, i] = bin_sums[0, i] + flow_mag[r, c]
                    
                    bin_counts[0, i] = bin_counts[0, i] + 1
                    
                    break
    
    # divide sums by bin counts            
    bin_sums_div_by_count = np.copy(bin_sums)

    bin_sums_div_by_count[bin_counts != 0] = \
        bin_sums_div_by_count[bin_counts != 0] / bin_counts[bin_counts != 0]
        
    # divide sums by area
    bin_sums_div_by_area = bin_sums / area
    
    return bin_sums, bin_sums_div_by_count, bin_sums_div_by_area
        
    
    
    
    
    
    
    
    

   
'''
path to pickled the resized color frame, its flow, and exemplar(s) information
'''
path_specific = 'STI WA Mar07 A/00011_trans/'
video_name = '00011_trans__320_360.mp4'
target_frame = 2

path_common = './video_copies_clips_track/copies_and_clips_wave_02/'


# pickle
pickle_path = path_common + path_specific + 'frame_flow_exemplars__' + \
              video_name[0:-4] + '__' + str(target_frame) + '.data'
            
with open(pickle_path, 'rb') as f:
    frame, flow, boxes = pickle.load(f)
    
# get gray frame
frame_gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
    
    
# get components
way_1, way_2, way_3 = label_fns.get_cc_info(frame, flow, boxes)
    

# overlay (after separating into keep and toss) (WAY #1 for components)
cc_dt_flow_keep, keep_labels, toss_labels = way_1

overlay_motion_keep = overlay.overlay(frame_gray, cc_dt_flow_keep,
                                      keep_labels, (0, 255, 0), alpha=0.5)

overlay_motion_keep = overlay.overlay(overlay_motion_keep, cc_dt_flow_keep,
                                      toss_labels, (0, 0, 255), alpha=0.5)


## overlay (after separating into keep and toss) (WAY #2 for components)
#cc_after_broken_cc, keep_labels, toss_labels = way_2
#
#overlay_broken_by_WS = overlay.overlay(frame_gray, cc_after_broken_cc,
#                                       keep_labels, (0, 255, 0), alpha=0.5)
#
#overlay_broken_by_WS = overlay.overlay(overlay_broken_by_WS,
#                                       cc_after_broken_cc, toss_labels,
#                                       (0, 0, 255), alpha=0.5)
#
#
##  overlay (after separating into keep and toss) (WAY #3 for components)
#cc_small_WS_comp_mask, keep_labels, toss_labels = way_3
#
#overlay_WS_comp_kept = overlay.overlay(frame_gray,
#                                       cc_small_WS_comp_mask, keep_labels,
#                                       (0, 255, 0), alpha=0.5)
#
#overlay_WS_comp_kept = overlay.overlay(overlay_WS_comp_kept,
#                                       cc_small_WS_comp_mask, toss_labels,
#                                       (0, 0, 255), alpha=0.5)

# column indices into features
FEATURE_B_HIST_LEFT = 0
FEATURE_B_HIST_RIGHT = 255
FEATURE_G_HIST_LEFT = 256
FEATURE_G_HIST_RIGHT = 511
FEATURE_R_HIST_LEFT = 512
FEATURE_R_HIST_RIGHT = 767

FEATURE_H_HIST_LEFT = 768
FEATURE_H_HIST_RIGHT = 947
FEATURE_S_HIST_LEFT = 948
FEATURE_S_HIST_RIGHT = 1203
FEATURE_V_HIST_LEFT = 1204
FEATURE_V_HIST_RIGHT = 1459

FEATURE_Y_HIST_LEFT = 1460
FEATURE_Y_HIST_RIGHT = 1715
FEATURE_Cr_HIST_LEFT = 1716
FEATURE_Cr_HIST_RIGHT = 1971
FEATURE_Cb_HIST_LEFT = 1972
FEATURE_Cb_HIST_RIGHT = 2227

FEATURE_B_PDF_LEFT = 2228
FEATURE_B_PDF_RIGHT = 2483
FEATURE_G_PDF_LEFT = 2484
FEATURE_G_PDF_RIGHT = 2739
FEATURE_R_PDF_LEFT = 2740
FEATURE_R_PDF_RIGHT = 2995

FEATURE_H_PDF_LEFT = 2996
FEATURE_H_PDF_RIGHT = 3175
FEATURE_S_PDF_LEFT = 3176
FEATURE_S_PDF_RIGHT = 3431
FEATURE_V_PDF_LEFT = 3432
FEATURE_V_PDF_RIGHT = 3687

FEATURE_Y_PDF_LEFT = 3688
FEATURE_Y_PDF_RIGHT = 3943
FEATURE_Cr_PDF_LEFT = 3944
FEATURE_Cr_PDF_RIGHT = 4199
FEATURE_Cb_PDF_LEFT = 4200
FEATURE_Cb_PDF_RIGHT = 4455

FEATURE_CANNY_EDGE_TO_AREA_RATIO = 4456

FEATURE_DFT_MAG_SPEC_HIST_LEFT = 4457
FEATURE_DFT_MAG_SPEC_HIST_RIGHT = 4520

FEATURE_DFT_PHASE_SPEC_HIST_LEFT = 4521
FEATURE_DFT_PHASE_SPEC_HIST_RIGHT = 4584

FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_LEFT = 4585
FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_RIGHT = 4592

FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_LEFT = 4593
FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_RIGHT = 4600

FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_LEFT = 4601
FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_RIGHT = 4608

FEATURE_AREA = 4609
FEATURE_PERIMETER = 4610

FEATURE_MIN_AREA_RECT_ORIENT = 4611
FEATURE_AREA_TO_MIN_AREA_RECT = 4612
FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO = 4613

features_vector_size = FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO + 1
   
# append another element to hold features (fill background with -1)
cc_dt_flow_keep.append([])
cc_dt_flow_keep[-1].append(-1 * np.ones((1, features_vector_size), dtype=np.float32))

# convert the color array to YCrCb and HSV
frame_HSV = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)
frame_YCrCb = cv2.cvtColor(frame, cv2.COLOR_BGR2YCrCb)


# BGRHSVYCrCb
combine = [frame, frame_HSV, frame_YCrCb]
BGRHSVYCrCb = np.stack(combine, axis=2)
BGRHSVYCrCb = np.reshape(BGRHSVYCrCb,(frame.shape[0], frame.shape[1],
                                      frame.shape[2] * len(combine)))

# prep to go through the channels of BGRHSVYCrCb and get histograms

# bin edges
bin_edges_0to255 = np.arange(0, 257, 1)
bin_edges_0to180 = np.arange(0, 181, 1)
bin_edges_arrays = [bin_edges_0to255 for i in range(3)] + \
                   [bin_edges_0to180] + [bin_edges_0to255 for i in range(5)]
            
# indices: start inclusive, end exclusive
start_end_pairs_hist = [[FEATURE_B_HIST_LEFT, FEATURE_B_HIST_RIGHT + 1],
                        [FEATURE_G_HIST_LEFT, FEATURE_G_HIST_RIGHT + 1],
                        [FEATURE_R_HIST_LEFT, FEATURE_R_HIST_RIGHT + 1],
                   
                        [FEATURE_H_HIST_LEFT, FEATURE_H_HIST_RIGHT + 1],
                        [FEATURE_S_HIST_LEFT, FEATURE_S_HIST_RIGHT + 1],
                        [FEATURE_V_HIST_LEFT, FEATURE_V_HIST_RIGHT + 1],
                   
                        [FEATURE_Y_HIST_LEFT, FEATURE_Y_HIST_RIGHT + 1],
                        [FEATURE_Cr_HIST_LEFT, FEATURE_Cr_HIST_RIGHT + 1],
                        [FEATURE_Cb_HIST_LEFT, FEATURE_Cb_HIST_RIGHT + 1]]
                   
start_end_pairs_pdf = [[FEATURE_B_PDF_LEFT, FEATURE_B_PDF_RIGHT + 1],
                       [FEATURE_G_PDF_LEFT, FEATURE_G_PDF_RIGHT + 1],
                       [FEATURE_R_PDF_LEFT, FEATURE_R_PDF_RIGHT + 1],
                   
                       [FEATURE_H_PDF_LEFT, FEATURE_H_PDF_RIGHT + 1],
                       [FEATURE_S_PDF_LEFT, FEATURE_S_PDF_RIGHT + 1],
                       [FEATURE_V_PDF_LEFT, FEATURE_V_PDF_RIGHT + 1],
                   
                       [FEATURE_Y_PDF_LEFT, FEATURE_Y_PDF_RIGHT + 1],
                       [FEATURE_Cr_PDF_LEFT, FEATURE_Cr_PDF_RIGHT + 1],
                       [FEATURE_Cb_PDF_LEFT, FEATURE_Cb_PDF_RIGHT + 1]]


# canny
'''https://docs.opencv.org/3.1.0/da/d22/tutorial_py_canny.html'''
minVal = 50
maxVal = 150
edges = cv2.Canny(frame_YCrCb[:, :, 0], minVal, maxVal)



# now loop through the rest of the components and gather features
for label in np.unique(cc_dt_flow_keep[1])[1:]:
    
    features_vector_append = []

    features_vector = np.zeros((1, features_vector_size), dtype=np.float32)
        
    cc_bool_mask = (cc_dt_flow_keep[1] == label)
    
    dilated_cc_mask = cv2.dilate(cc_bool_mask.astype(np.uint8),
                                 kernel=np.ones((3,3)))
    
    cc_dilated_cc = cv2.connectedComponentsWithStats(dilated_cc_mask)
    cc_dilated_bool_mask = cc_dilated_cc[1].astype(np.bool)
    cc_dilated_mask = cc_dilated_bool_mask.astype(np.uint8)
    
    # form the color space histograms    
    for i, info in enumerate(zip(bin_edges_arrays, start_end_pairs_hist)):
        
        bin_edges, start_end = info

        hist, bin_edges = np.histogram(BGRHSVYCrCb[:, :, i] \
                                       [cc_dilated_bool_mask],
                                       bin_edges)
    
        features_vector[:, start_end[0]:start_end[1]] = hist 
        
        features_vector_append.append(hist)

    # form the color space PDFs    
    for i, info in enumerate(zip(bin_edges_arrays, start_end_pairs_pdf)):
        
        bin_edges, start_end = info

        hist, bin_edges = np.histogram(BGRHSVYCrCb[:, :, i] \
                                       [cc_dilated_bool_mask],
                                       bin_edges)
    
        features_vector[:, start_end[0]:start_end[1]] = hist / np.sum(hist)
        
        features_vector_append.append(hist / np.sum(hist))
                        
    
    # get the canny edge to area ratio (using the dilated component area)
    edges_in_dilated_cc = cv2.bitwise_and(edges, edges, mask=cc_dilated_mask)
    
    features_vector[:, FEATURE_CANNY_EDGE_TO_AREA_RATIO] = \
        np.count_nonzero(edges_in_dilated_cc) / \
        np.count_nonzero(cc_dilated_mask)
        
    features_vector_append.append(np.count_nonzero(edges_in_dilated_cc) / \
                                  np.count_nonzero(cc_dilated_mask))

    # discrete Fourier transform (DFT) magnitude and phase spectrum histograms
    
    # get the grayscale image crop with the component
    crop = frame_YCrCb[:, :, 0][cc_dilated_cc[2][1, cv2.CC_STAT_TOP] : \
                                cc_dilated_cc[2][1, cv2.CC_STAT_TOP] + \
                                cc_dilated_cc[2][1, cv2.CC_STAT_HEIGHT], 
                                cc_dilated_cc[2][1, cv2.CC_STAT_LEFT] : \
                                cc_dilated_cc[2][1, cv2.CC_STAT_LEFT] + \
                                cc_dilated_cc[2][1, cv2.CC_STAT_WIDTH]]
 
    mag_spec_hist, phase_spec_hist, _, _, _ = \
        dft_hist_fns.get_radians_per_pixel_histograms(crop)
        
    features_vector[:, FEATURE_DFT_MAG_SPEC_HIST_LEFT : \
                       FEATURE_DFT_MAG_SPEC_HIST_RIGHT + 1] = \
                    np.reshape(mag_spec_hist, (1, -1))
                    
    features_vector_append.append(np.reshape(mag_spec_hist, (1, -1)))                
                    
    features_vector[:, FEATURE_DFT_PHASE_SPEC_HIST_LEFT : \
                       FEATURE_DFT_PHASE_SPEC_HIST_RIGHT + 1] = \
                    np.reshape(phase_spec_hist, (1, -1))
                    
    features_vector_append.append(np.reshape(phase_spec_hist, (1, -1)))

        
    # flow phase bins to flow magnitude sums 'histogram'; using dilated mask
    bin_sums, bin_sums_div_by_count, bin_sums_div_by_area = \
        get_flow_mag_in_phase_bins(flow, cc_dilated_mask,
                                   -np.pi / 8, np.pi / 4)
        
    features_vector[:, FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_LEFT : \
                       FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_RIGHT + 1] = \
        bin_sums
    
    features_vector_append.append(bin_sums)
    
    features_vector[:, FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_LEFT : \
                       FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_RIGHT \
                       + 1] = bin_sums_div_by_count
                    
    features_vector_append.append(bin_sums_div_by_count)
                    
    features_vector[:, FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_LEFT : \
                       FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_RIGHT \
                       + 1] = bin_sums_div_by_area

    features_vector_append.append(bin_sums_div_by_area)
                    
    # area
    area = np.count_nonzero(cc_dilated_mask)
    features_vector[:, FEATURE_AREA] = area
    
    features_vector_append.append(area)
    
    # feature perimeter
    kernel = np.ones((3, 3))
    
    features_vector[:, FEATURE_PERIMETER] = \
        np.count_nonzero(cv2.subtract(cv2.dilate(cc_dilated_mask, kernel),
                                      cv2.erode(cc_dilated_mask, kernel)))
        
    features_vector_append.append(np.count_nonzero(cv2.subtract(\
                                  cv2.dilate(cc_dilated_mask, kernel),
                                  cv2.erode(cc_dilated_mask, kernel))))

    # minimum area rectangle features
    
    # get the external contour points 
    contours = cv2.findContours(cc_dilated_mask, cv2.RETR_EXTERNAL,
                                cv2.CHAIN_APPROX_SIMPLE)
    
    # reshape the points into N x 2 for passing to cv2.minAreaRect()
    contour_points = contours[1][0].reshape((contours[1][0].shape[0], -1))
    
    # get the minimum area rectangle info
    rect_center_point, rect_shape, rotation = cv2.minAreaRect(contour_points)
    
    # separate the rotations
    width_along_rotated_x, height_along_rotated_y = rect_shape
    
    '''when width_along_rotated_x is smaller than height_along_rotated_y, 
    adjust the rotation to distinguish orientation'''
    if width_along_rotated_x < height_along_rotated_y:
        
        if abs(rotation) != 90:
            
            rotation = -rotation
            
        else:
            
            rotation = 0
            
    #-force -90 to become 90'''            
    elif rotation == -90:
        
        rotation = 90

    # save the orientation feature
    features_vector[:, FEATURE_MIN_AREA_RECT_ORIENT] = rotation
    
    features_vector_append.append(rotation)
    
    # save the ratio of component area to minimum area rectangle
    features_vector[:, FEATURE_AREA_TO_MIN_AREA_RECT] = \
        area / (width_along_rotated_x * height_along_rotated_y)
        
    features_vector_append.append(area / (width_along_rotated_x * \
                                          height_along_rotated_y))
        
    # save ratio of short side to long side of minimum area rectangle
    features_vector[:, FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO] = \
        min(rect_shape) / max(rect_shape)
        
    features_vector_append.append(min(rect_shape) / max(rect_shape))



 
    features_vector_check_pieces = []
    for e in features_vector_append:
        if type(e) == float:
            features_vector_check_pieces.append(np.array([e]))
        elif type(e) == int:
            features_vector_check_pieces.append(np.array([e]))
        elif len(e.shape) == 1:
            features_vector_check_pieces.append(e)
        elif len(e.shape) == 2:
            features_vector_check_pieces.append(e[0])
        else:
            print('DID NOT HAVE THIS CASE')
    features_vector_check = np.hstack(features_vector_check_pieces)
    features_vector_check = features_vector_check.reshape((1, -1))
    
    print(np.all(features_vector_check.astype(np.float32) == \
                 features_vector))




# display
titles = ['components thus far',
          
#          'components broken by watershed',
#          
#          'watershed components kept',
          
          'canny edges'

          ]

images = [overlay_motion_keep,
          
#          overlay_broken_by_WS,
#          
#          overlay_WS_comp_kept,
          
          edges

         ]

        
for title, image in zip(titles, images):
    
    cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(title, 0, -60) 
    cv2.imshow(title, image)

k = cv2.waitKey(0)

cv2.destroyAllWindows()
        
# escape
if k == 27:
    
    escape = True
    
    


























