# -*- coding: utf-8 -*-
"""
Created on Sun Feb 18 12:56:50 2018

@author: Callie Darsey

Example of Discrete Fourier Transform (DFT) of image

example_of_DFT_01
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt
import pickle

def get_radians_per_pixel_histograms(image):
    '''
    Uhg. I just don't feel like writing a description
    
    image - one channel image
    '''
    
    dft = cv2.dft(np.float32(image),flags = cv2.DFT_COMPLEX_OUTPUT)
    
    dft_mag = cv2.magnitude(dft[:,:,0], dft[:,:,1])
    dft_phase = cv2.phase(dft[:,:,0], dft[:,:,1])
    
    #def get_radians_per_pixel_grid(rows, cols):
    '''
    For a given size of (rows, cols), find the radians per pixel associated
    with each pixel in each dimension.
    
    See my scratch notebook for details.
    '''
    # row and column indices
    rows, cols = image.shape
    rind = np.arange(0, rows)
    cind = np.arange(0, cols)
    
    # make row values
    with_row_value = 2 * np.pi * rind / rows
    
    # adjust for greater than pi
    gtpi_ind = np.where(with_row_value > np.pi)[0]
    with_row_value[gtpi_ind] = (-2*np.pi + with_row_value)[gtpi_ind]
    
    # adjust for less than -pi
    ltnpi_ind = np.where(with_row_value < -np.pi)[0]
    with_row_value[ltnpi_ind] = (2 * np.pi + with_row_value)[ltnpi_ind]
    
    # make column values
    with_col_value = 2 * np.pi * cind / cols
    
    # adjust for greater than pi
    gtpi_ind = np.where(with_col_value > np.pi)[0]
    with_col_value[gtpi_ind] = (-2*np.pi + with_col_value)[gtpi_ind]
    
    # adjust for less than -pi
    ltnpi_ind = np.where(with_col_value < -np.pi)[0]
    with_col_value[ltnpi_ind] = (2 * np.pi + with_col_value)[ltnpi_ind]
    
    # So here is the grid
    col_rad_per_pix, row_rad_per_pix = np.meshgrid(with_col_value, with_row_value)
    
    # now form the magnitude histogram
    divisions = 8
    start = -np.pi / 8
    bin_edges = np.arange(start, start + 2 * np.pi, 2 * np.pi / divisions)
    
    # force between -pi and pi
    gtpi_ind = np.where(bin_edges > np.pi)[0]
    
    bin_edges[gtpi_ind] = (-2 * np.pi + bin_edges)[gtpi_ind]
    
    # bin masks
    mag_spec_hist = np.zeros((len(bin_edges), len(bin_edges)), dtype=np.float32)
    phase_spec_hist = np.zeros_like(mag_spec_hist)
    
    bin_masks = []
    color = 0
    count = 0
    
    for col_ind, dim_1 in enumerate(zip(bin_edges, np.hstack([bin_edges[1:],
                                                              bin_edges[0]]))):
        
        dim_1_edge_1, dim_1_edge_2 = dim_1
        
        for row_ind, dim_2 in enumerate(zip(bin_edges,np.hstack([bin_edges[1:],
                                                                 bin_edges[0]]))):
            
            dim_2_edge_1, dim_2_edge_2 = dim_2
            
            count = count + 1
            
            color = color + 3
            
            parts = []
            
        
            if dim_1_edge_1 < dim_1_edge_2:
                
                # columns
                parts.append(np.all([dim_1_edge_1 <= col_rad_per_pix,
                                    col_rad_per_pix < dim_1_edge_2], axis=0))
        
                
            else:
                
                # columns
                parts.append(np.any([np.all([dim_1_edge_1 <= col_rad_per_pix,
                                           col_rad_per_pix <= np.pi], axis=0),
                             np.all([col_rad_per_pix < dim_1_edge_2,
                                    -np.pi <= col_rad_per_pix], axis=0)], axis=0))
                
                
            if dim_2_edge_1 < dim_2_edge_2:
                            
                # rows
                parts.append(np.all([dim_2_edge_1 <= row_rad_per_pix,
                                     row_rad_per_pix < dim_2_edge_2], axis=0))            
                
            else:
                
                # rows
                parts.append(np.any([np.all([dim_2_edge_1 <= row_rad_per_pix,
                                             row_rad_per_pix <= np.pi], axis=0),
                                     np.all([row_rad_per_pix < dim_2_edge_2,
                                             -np.pi <= row_rad_per_pix], axis=0)],
                                    axis=0))
            
            # save for visualizing the masks later
            bin_masks.append(np.all(parts, axis=0).astype(np.uint8)*color)
        
            # mask for bin
            bin_mask = np.all(parts, axis=0).astype(np.uint8) * 255
            
            # sum of magnitude spectrum values for bin
            dft_mag_spec_sum = np.sum(cv2.bitwise_and(dft_mag,
                                                      dft_mag, mask=bin_mask))
            
            # sum of phase spectrum values for the bin
            dft_phase_spec_sum = np.sum(cv2.bitwise_and(dft_phase,
                                                        dft_phase, mask=bin_mask))
            
            # place the sums
            mag_spec_hist[row_ind, col_ind] = dft_mag_spec_sum
            phase_spec_hist[row_ind, col_ind] = dft_phase_spec_sum
            
            
        
    
    
        
    show_bin_masks = np.sum(bin_masks, axis=0, dtype=np.uint8)
    
    return (mag_spec_hist, phase_spec_hist, dft_mag, dft_phase, show_bin_masks)


if __name__ == '__main__':
    
    example = '1'
    
    if example == '1':
        rows, cols = (6, 7)
        u, v = (3, 1)
        
        J, I = np.meshgrid(np.arange(0, cols), np.arange(0, rows))
        
        A = (255 / 2)
        image = A * np.sin(2 * np.pi *(u / rows * I + v / cols * J)) + A
        image = image.astype(np.uint8)
      
        mag_spec_hist, phase_spec_hist, dft_mag, dft_phase, show_bin_masks = \
            get_radians_per_pixel_histograms(image)
            
    elif example == '2':
        
        '''
        path to pickled the resized color frame, its flow, and exemplar(s) information
        '''
        path_specific = 'STI WA Mar07 A/00011_trans/'
        video_name = '00011_trans__320_360.mp4'
        target_frame = 2
        
        path_common = './video_copies_clips_track/copies_and_clips_wave_02/'
        
        
        # pickle
        pickle_path = path_common + path_specific + 'frame_flow_exemplars__' + \
                      video_name[0:-4] + '__' + str(target_frame) + '.data'
                    
        with open(pickle_path, 'rb') as f:
            frame, flow, boxes = pickle.load(f)
            
        frame_gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        
        image = frame_gray[557:680, 196:315]
        
        mag_spec_hist, phase_spec_hist, dft_mag, dft_phase, show_bin_masks = \
            get_radians_per_pixel_histograms(image)
    
            
    
    cv2.imshow('title', image)
    cv2.imshow('magnitude spectrum', np.log(dft_mag + 1) / np.max(np.log(dft_mag + 1)))
    cv2.imshow('phase spectrum', dft_phase / np.max(dft_phase))
    cv2.imshow('bin masks', show_bin_masks)
    cv2.imshow('magnitude spectrum 2D histogram', np.log(mag_spec_hist) / np.max(np.log(mag_spec_hist)))
    cv2.imshow('phase spectrum 2D histogram', phase_spec_hist / np.max(phase_spec_hist))
    
    cv2.waitKey(0)
    cv2.destroyAllWindows()




