# -*- coding: utf-8 -*-
"""
Created on Sat Apr 14 22:03:22 2018

@author: Callie Darsey

save feature vectors for different patch sizes
"""

import os
import pickle
import patches_01 as patches
import numpy as np
import cv2


path = './video_copies_clips_track/ROI_masks_patch_detection/'
roi_filenames = [f for f in os.listdir(path) if os.path.isfile(path + f)]

# make the review folder
save = path + "non_skin_misclassifed/"
if not os.path.isdir(save):
    os.mkdir(save)


print('Number of frames: {}\n'.format(len(roi_filenames)))

for block_dim in [2, 3, 4, 5]:

    print('block dimension ({:d}, {:d})'.format(block_dim, block_dim))
    
    hand_feature_vectors_collect = []
    non_skin_feature_vectors_collect = []
    
    for roi_filename in roi_filenames:
        
        print(roi_filename)
        
        with open(path + roi_filename, 'rb') as f:
            
            roi_masks, frame = pickle.load(f)
            
        classified_frame = cv2.imread(path + \
            'classified_training/classified_patch_skin_mask_' + \
            roi_filename[0:-4] + 'png')
        
        classified_skin_mask = \
            (np.sum(classified_frame.astype(np.float),
                    axis=2) > 0).astype(np.uint8)
            
        hand_mask_collect = []
        non_skin_mask_collect = []
        
        for key in roi_masks.keys():
            
            if 'hand' in key:
                
                hand_mask_collect.append(roi_masks[key])
                
            elif 'non_skin' in key:
                
                non_skin_mask_collect.append(roi_masks[key] * \
                                             classified_skin_mask)
                
        hand_mask = np.any(hand_mask_collect, axis=0)       
        non_skin_mask = np.any(non_skin_mask_collect, axis=0)
        
        hand_feature_vectors = \
            patches.get_valid_blocks_as_feature_vectors(frame, hand_mask,
                                                        block_dim)
    
        non_skin_feature_vectors = \
            patches.get_valid_blocks_as_feature_vectors(frame, non_skin_mask,
                                                        block_dim)
        print('Number of hand feature vector:     {}'.format(\
              hand_feature_vectors.shape[0]))
        print('Number of non-skin feature vector: {}\n'.format(\
              non_skin_feature_vectors.shape[0]))
    
        with open(save+roi_filename[0:-5] + \
                  '_block_dim_{:02d}.data'.format(block_dim), 'wb') as f:
            
            pickle.dump((hand_feature_vectors, non_skin_feature_vectors), f)
