# -*- coding: utf-8 -*-
"""
Created on Fri May 11 15:37:55 2018

@author: Callie Darsey

features_from_patches_1st_01
"""

import cv2
import numpy as np
import pickle
import matplotlib.pyplot as plt
import os


def sum_all_flow_mag_in_patch(segment_flow_mag, row_1, row_2, col_1, col_2):
    '''
    '''
    collect = []
    
    for flow_mag_display in segment_flow_mag:
        
        collect.append(flow_mag_display[row_1 : row_2 + 1,
                                        col_1 : col_2 + 1,
                                        0].astype(np.float64))
    collect = np.stack(collect, axis=2)
    
    return np.sum(collect)


def combine(crop1, crop2):
    '''
    '''
    mask1 = np.any(crop1 > 50, axis=2)
    mask2 = np.any(crop2 > 50, axis=2)
    
    intersection = np.logical_and(mask1, mask2)
    xor_mask1 = np.logical_xor(mask1, intersection)
    xor_mask2 = np.logical_xor(mask2, intersection)    

    pred_side_1 = np.copy(crop1)
    pred_side_2 = np.copy(crop2)
    
    np.copyto(pred_side_1, crop2,
              where=np.stack([xor_mask2, xor_mask2, xor_mask2], axis=2))
    np.copyto(pred_side_2, crop1,
              where=np.stack([xor_mask1, xor_mask1, xor_mask1], axis=2))

    alpha = 0.5
    beta = 1.0 - alpha
    gamma = 0.0
    
    output = cv2.addWeighted(pred_side_1, alpha, pred_side_2, beta, gamma)
    
    return output


def get_seg_skin_overlays(segment_cleaned_skin, col_1, col_2, row_1, row_2):
    '''
    '''
    
    crop1 = segment_cleaned_skin[0][row_1 : row_2 + 1, col_1 : col_2 + 1, :]
    
    for index in np.arange(1, len(segment_cleaned_skin)):
        
        crop2 = segment_cleaned_skin[index][row_1 : row_2 + 1,
                                            col_1 : col_2 + 1, :]

        crop1 = combine(crop1, crop2)
        
    return crop1


places1 = [\
'part1/MVI_0069_trans/',
'STI_Polk_Apr13_C/MVI_0039_trans/',
'STI_Polk_Apr27_A/MVI_0073_trans/',
'STI_Polk_Apr27_D/MVI_0063_trans/',
'STI_Polk_May04_A/MVI_0087_trans/',
'STI_Polk_May11_C/MVI_0108_trans/',
'STI_WA_Apr11_extra_B/MVI_0036_trans/',
'STI_WA_Jun12_B/MVI_0018_trans/',
'STI_WA_Jun14_Lb/MVI_0070_trans/',
'STI_WA_Jun20_CT/MVI_0029_trans/',
'STI_WA_Jun22_De/MVI_0118_trans/',
'STI_WA_Jun23_Da/MVI_0112_trans/',
'STI_WA_May02_B/MVI_0053_trans/',
'STI_WA_May02_C/MVI_0074_trans/',
'STI_WA_May09_A/MVI_0109_trans/']

places2 = [\
'part1/MVI_0069_trans/',
'STI Polk Apr13 C/MVI_0039_trans/',
'STI Polk Apr27 A/MVI_0073_trans/',
'STI Polk Apr27 D/MVI_0063_trans/',
'STI Polk May04 A/MVI_0087_trans/',
'STI Polk May11 C/MVI_0108_trans/',
'STI WA Apr11 extra B/MVI_0036_trans/',
'STI WA Jun12 B/MVI_0018_trans/',
'STI WA Jun14 Lb/MVI_0070_trans/',
'STI WA Jun20 CT/MVI_0029_trans/',
'STI WA Jun22 De/MVI_0118_trans/',
'STI WA Jun23 Da/MVI_0112_trans/',
'STI WA May02 B/MVI_0053_trans/',
'STI WA May02 C/MVI_0074_trans/',
'STI WA May09 A/MVI_0109_trans/']

for place1, place2 in zip(places1, places2):


    directory = "./wheeler/dataset/" + place1
    
    for content in os.listdir(directory):
        
        if os.path.isfile(directory + content):
            
            if content[-12:] == '.sampled.mp4':
                
                name = content[:-12]
                
                print(name)
                
                break
                                       
    cap_flow_mag = cv2.VideoCapture(directory+'flow_'+name+'.flow_mag.mp4')
    cap_flow_ang = cv2.VideoCapture(directory+'flow_'+name+'.flow_ang.mp4')
    cap_cleaned_skin = cv2.VideoCapture(directory+name+'.cleaned_skin.mp4')
    cap_frame = cv2.VideoCapture(directory+name+'.sampled.mp4')    
    
    with open(directory + 'patches_info___' + name + '.data', 'rb') as f:
        
        patches_and_labels = pickle.load(f)
        
    keys = np.array(list(patches_and_labels.keys()))
    
    patches_and_labels_and_features = {}
    
    segment = 45
    
    segment_flow_mag = []
    segment_flow_ang = []
    segment_cleaned_skin = []
    segment_cleaned_skin_watershed = []
    segment_frame = []
    
    count = -1
    ind = -1
    
    while(cap_flow_mag.isOpened() and cap_flow_ang.isOpened() and \
          cap_cleaned_skin.isOpened() and cap_frame.isOpened()):
        
        success_flag_flow_mag, flow_mag_display     = cap_flow_mag.read()
        success_flag_flow_ang, flow_ang_display     = cap_flow_ang.read()
        success_flag_skin,     cleaned_skin_display = cap_cleaned_skin.read()
        success_frame,         frame_display        = cap_frame.read()
        
        if success_flag_flow_mag == True and success_flag_flow_ang == True \
            and success_flag_skin == True and success_frame == True:
            
            count = count + 1
            
            rows, cols, channels = frame_display.shape
            
            
            # watershed skin
            skin_mask_thr = np.any(cleaned_skin_display > 50,
                                   axis=2).astype(np.uint8)
    
            
            dilate_skin_mask_thr = cv2.dilate(skin_mask_thr, kernel=np.ones((3,3)),
                                              iterations=80)
            
            unknown_skin_watershed = cv2.subtract(dilate_skin_mask_thr,
                                                  skin_mask_thr)
            
            cc_skin_mask_thr = cv2.connectedComponents(skin_mask_thr)
            
            markers_skin_watershed = cc_skin_mask_thr[1] + 1
            
            markers_skin_watershed[unknown_skin_watershed > 0] = -1
            
            # call the watershed
            cv2.watershed(frame_display, markers_skin_watershed)
            
            cc_markers_skin_watershed = cv2.connectedComponentsWithStats(\
                markers_skin_watershed.astype(np.bool).astype(np.uint8))
                
            skin_watershed = markers_skin_watershed.copy()
            
            skin_watershed[skin_watershed == -1] = 0
            skin_watershed[skin_watershed == np.argmax(\
                cc_markers_skin_watershed[2][:, cv2.CC_STAT_AREA])] = 0
            
            skin_watershed_display = cv2.bitwise_and(frame_display,
                                                     frame_display,
                mask=skin_watershed.astype(np.bool).astype(np.uint8))
            

            segment_flow_mag.append(flow_mag_display[:, :, 0])
            segment_flow_ang.append(flow_ang_display[:, :, 0])
            segment_cleaned_skin.append(cleaned_skin_display)
            segment_cleaned_skin_watershed.append(skin_watershed_display)
            segment_frame.append(frame_display)
        
            if len(segment_flow_mag) > segment:
                segment_flow_mag.pop(0)
                segment_flow_ang.pop(0)
                segment_cleaned_skin.pop(0)
#                segment_cleaned_skin_watershed.pop(0)
                segment_frame.pop(0)
    
            else:
                ind = ind + 1


            if count != 0 and (count + 1) % segment == 0:
                
                print('segment', int(np.floor(count / segment)))
                
                # overlay the skin watershed
                skin_watershed_combined = get_seg_skin_overlays(\
                    segment_cleaned_skin_watershed, 0, cols, 0, rows)
                    
#                plt.figure(figsize=(10, 10))
#                plt.imshow(cv2.merge([skin_watershed_combined[:, :, 2],
#                                      skin_watershed_combined[:, :, 1],
#                                      skin_watershed_combined[:, :, 0]]))
#                plt.show() 

                # overlay the cleaned skin
                skin_cleaned_combined = get_seg_skin_overlays(\
                    segment_cleaned_skin, 0, cols, 0, rows)
                
                skin_cleaned_combined_gray = \
                    cv2.cvtColor(skin_cleaned_combined, cv2.COLOR_BGR2GRAY)
                    
#                plt.figure(figsize=(10, 10))
#                plt.imshow(skin_cleaned_combined_gray)
#                plt.show()                 
                
                index = np.where(keys[:, 1] == count * 4)[0]

                if len(index) == 1:
                    
                    relevant_key = (keys[index[0], 0], keys[index[0], 1])
                    
                    patches, tags = patches_and_labels[relevant_key]
                    
                    
                    # get the patch size
                    col_1, col_2 = np.sort([patches[0][0], patches[0][2]])
                
                    row_1, row_2 = np.sort([patches[0][1], patches[0][3]])
                    
                    patch_size = (row_2 - row_1 + 1, col_2 - col_1 + 1)
                    
                    '''
                    location
                    col_loc = patch center column / frame width 
                    row_loc = patch center row / frame height and 
                    '''
                    feature_location = np.zeros((patches.shape[0], 2))
                    
                    '''
                    complete flow mag. sum over patch clip
                    '''
                    feature_sum_all_flow_mag = np.zeros((patches.shape[0], 1))
                    
                    '''
                    skin watershed present in patch
                    '''
                    feature_watershed_skin_present = \
                        np.zeros((patches.shape[0], 1))

                    '''
                    skin cleaned combined sum
                    '''
                    skin_combined_sum = np.zeros((patches.shape[0], 1))
                    
                    '''
                    skin combined image
                    '''
                    skin_combined_image = np.zeros((patches.shape[0],
                        patch_size[0] * patch_size[1]))
                    
                    for i, patch in enumerate(patches):
                        
                        col_1, col_2 = np.sort([patch[0], patch[2]])
                    
                        row_1, row_2 = np.sort([patch[1], patch[3]])
                        
                        # location
                        col_loc = np.mean([col_1, col_2]) / cols
                        
                        row_loc = np.mean([row_1, row_2]) / rows
                                                
                        feature_location[i] = np.array([[col_loc, row_loc]])
                        
                        # complete flow mag. sum over patch clip
                        sum_all_flow_mag = \
                            sum_all_flow_mag_in_patch(segment_flow_mag,
                                                      row_1, row_2,
                                                      col_1, col_2)
                        feature_sum_all_flow_mag[i] = sum_all_flow_mag
                        
                        # watershed skin present in patch
                        feature_watershed_skin_present[i] = np.sum(\
                            skin_watershed_combined[row_1 : row_2 + 1,
                                                    col_1 : col_2 + 1, :])
    
                        # skin cleaned combined sum
                        skin_combined_sum[i] = np.count_nonzero(np.any(\
                            skin_cleaned_combined[\
                            row_1 : row_2 + 1, col_1 : col_2 + 1, :], axis=2))
                        
                        # skin combined image
                        skin_combined_image[i, :] = \
                            skin_cleaned_combined_gray[\
                            row_1 : row_2 + 1, col_1 : col_2 + 1].ravel()
                        
                    patches_and_labels_and_features[(relevant_key)] = []
                    
                    
                    for feature in [feature_location,
                                    feature_sum_all_flow_mag,
                                    feature_watershed_skin_present,
                                    skin_combined_sum,
                                    skin_combined_image]:
                        
                        patches_and_labels_and_features[(relevant_key) \
                                                        ].append(feature)


              
        else:
            
            print("No more frames")
            
            cap_flow_mag.release()
            cap_flow_ang.release()
                    
            break        
    else:
        
        cap_flow_mag.release()
        cap_flow_ang.release()
    
    with open(directory + 'patches_labels_and_features___' + name + '.data', \
              'wb') as f:
        
        pickle.dump((patches_and_labels_and_features), f)   