# -*- coding: utf-8 -*-
"""
Created on Wed May  9 08:28:57 2018

@author: Callie Darsey

Try to get other features from the reference
"""


import cv2
import numpy as np
import pickle
import matplotlib.pyplot as plt
import os


def get_seg_mag_sum(segment_flow_mag):
    '''
    '''
    collect = []
    
    for flow_mag_display in segment_flow_mag:
        
        collect.append(flow_mag_display[:, :, 0].astype(np.float64))
        
    segment_sum = \
        (np.sum(collect, axis=0) / len(segment_flow_mag)).astype(np.uint8)
        
    segment_sum_display = \
        np.stack([segment_sum, segment_sum, segment_sum], axis=2)
        
    return segment_sum_display


def combine(crop1, crop2):
    '''
    '''
    mask1 = np.any(crop1 > 50, axis=2)
    mask2 = np.any(crop2 > 50, axis=2)
    
    intersection = np.logical_and(mask1, mask2)
    xor_mask1 = np.logical_xor(mask1, intersection)
    xor_mask2 = np.logical_xor(mask2, intersection)    

    pred_side_1 = np.copy(crop1)
    pred_side_2 = np.copy(crop2)
    
    np.copyto(pred_side_1, crop2,
              where=np.stack([xor_mask2, xor_mask2, xor_mask2], axis=2))
    np.copyto(pred_side_2, crop1,
              where=np.stack([xor_mask1, xor_mask1, xor_mask1], axis=2))

    alpha = 0.5
    beta = 1.0 - alpha
    gamma = 0.0
    
    output = cv2.addWeighted(pred_side_1, alpha, pred_side_2, beta, gamma)
    
    return output


def get_seg_skin_overlays(segment_cleaned_skin, col_1, col_2, row_1, row_2):
    '''
    '''
    
    crop1 = segment_cleaned_skin[0][row_1 : row_2 + 1, col_1 : col_2 + 1, :]
    
    for index in np.arange(1, len(segment_cleaned_skin)):
        
        crop2 = segment_cleaned_skin[index][row_1 : row_2 + 1,
                                            col_1 : col_2 + 1, :]

        crop1 = combine(crop1, crop2)
        
    return crop1
    
    
places1 = [\
'part1/MVI_0069_trans/',
'STI_Polk_Apr13_C/MVI_0039_trans/',
'STI_Polk_Apr27_A/MVI_0073_trans/',
'STI_Polk_Apr27_D/MVI_0063_trans/',
'STI_Polk_May04_A/MVI_0087_trans/',
'STI_Polk_May11_C/MVI_0108_trans/',
'STI_WA_Apr11_extra_B/MVI_0036_trans/',
'STI_WA_Jun12_B/MVI_0018_trans/',
'STI_WA_Jun14_Lb/MVI_0070_trans/',
'STI_WA_Jun20_CT/MVI_0029_trans/',
'STI_WA_Jun22_De/MVI_0118_trans/',
'STI_WA_Jun23_Da/MVI_0112_trans/',
'STI_WA_May02_B/MVI_0053_trans/',
'STI_WA_May02_C/MVI_0074_trans/',
'STI_WA_May09_A/MVI_0109_trans/']

places2 = [\
'part1/MVI_0069_trans/',
'STI Polk Apr13 C/MVI_0039_trans/',
'STI Polk Apr27 A/MVI_0073_trans/',
'STI Polk Apr27 D/MVI_0063_trans/',
'STI Polk May04 A/MVI_0087_trans/',
'STI Polk May11 C/MVI_0108_trans/',
'STI WA Apr11 extra B/MVI_0036_trans/',
'STI WA Jun12 B/MVI_0018_trans/',
'STI WA Jun14 Lb/MVI_0070_trans/',
'STI WA Jun20 CT/MVI_0029_trans/',
'STI WA Jun22 De/MVI_0118_trans/',
'STI WA Jun23 Da/MVI_0112_trans/',
'STI WA May02 B/MVI_0053_trans/',
'STI WA May02 C/MVI_0074_trans/',
'STI WA May09 A/MVI_0109_trans/']

for place1, place2 in zip(places1, places2):


    directory = "./wheeler/dataset/" + place1
    
    directory_e = "./video_copies_clips_track/copies_and_clips_wave_04/" + place2
    
    for content in os.listdir(directory):
        
        print(content)
        
        if os.path.isfile(directory + content):
            
            if content[-12:] == '.sampled.mp4':
                
                name = content[:-12]
                
                break
                                       
    cap_flow_mag = cv2.VideoCapture(directory+'flow_'+name+'.flow_mag.mp4')
    cap_flow_ang = cv2.VideoCapture(directory+'flow_'+name+'.flow_ang.mp4')
    cap_cleaned_skin = cv2.VideoCapture(directory+name+'.cleaned_skin.mp4')
    
    
    with open(directory_e + 'exemplars_and_labels___' + name + '.data',
              'rb') as f:
        
        exemplars_and_labels = pickle.load(f)
        
    exemplar_info = {}
    
    keys = np.array(list(exemplars_and_labels.keys()))
    
    
    segment = 45
    
    segment_flow_mag = []
    segment_flow_ang = []
    segment_cleaned_skin = []
    
    count = -1
    ind = -1
                
    set_size = (100, 100)
    
    while(cap_flow_mag.isOpened() and cap_flow_ang.isOpened() and \
          cap_cleaned_skin.isOpened()):
        
        success_flag_flow_mag, flow_mag_display = cap_flow_mag.read()
        success_flag_flow_ang, flow_ang_display = cap_flow_ang.read()
        success_flag_skin, cleaned_skin_display = cap_cleaned_skin.read()
        
        if success_flag_flow_mag == True and success_flag_flow_ang == True \
            and success_flag_skin == True:
            
            count = count + 1
    
            segment_flow_mag.append(flow_mag_display)
            segment_flow_ang.append(flow_ang_display)
            segment_cleaned_skin.append(cleaned_skin_display)
        
            if len(segment_flow_mag) > segment:
                segment_flow_mag.pop(0)
                segment_flow_ang.pop(0)
                segment_cleaned_skin.pop(0)
    
            else:
                ind = ind + 1
    
            if count != 0 and (count + 1) % segment == 0:
                print(count)
                
                print('segment', int(np.floor(count / segment)))
                
                index = np.where(keys[:, 1] == count * 4)[0]
                
                if len(index) == 1:
                    
                    exemplars, tags = \
                        exemplars_and_labels[(keys[index[0], 0],
                                              keys[index[0], 1])]
                        
                    feature_vectors = np.zeros((exemplars.shape[0],
                                                set_size[0] * set_size[1] * 3))
                    
                    for i, box in enumerate(exemplars):
                        
                        col_1, col_2 = np.sort([box[0], box[2]])
                    
                        row_1, row_2 = np.sort([box[1], box[3]])
                        
                        crop = get_seg_skin_overlays(segment_cleaned_skin,
                                              col_1, col_2, row_1, row_2)
                        
                        plt.imshow(cv2.merge([crop[:, :, 2],
                                              crop[:, :, 1],
                                              crop[:, :, 0]]))
                        plt.show()
                        
                        rows, cols, channels = crop.shape
                        
                        if rows == cols:
                            pad = crop
                            
                        elif rows > cols:
                            pad = np.zeros((rows, rows, channels),
                                           dtype=np.uint8)
                            shift = int((rows - cols) / 2)
                            pad[ :, shift : shift + cols, :] = crop
                            
                        elif cols > rows:
                            pad = np.zeros((cols, cols, channels),
                                           dtype=np.uint8)
                            shift = int((cols - rows) / 2)
                            pad[ shift : shift + rows, :, :] = crop
                        
                        pad = cv2.resize(pad, set_size)
                        
                        plt.imshow(cv2.merge([pad[:, :, 2],
                                              pad[:, :, 1],
                                              pad[:, :, 0]]))
                        plt.show()
    
                        feature_vectors[i] =  pad.ravel()   
                        
                    exemplar_info[(keys[index[0], 0], keys[index[0], 1])] = \
                        [exemplars, tags, feature_vectors]
              
        else:
            
            print("No more frames")
            
            cap_flow_mag.release()
            cap_flow_ang.release()
                    
            break        
    else:
        
        cap_flow_mag.release()
        cap_flow_ang.release()
    
    with open(directory + \
              'exemplars_labels_and_stuff_as_feature_vectors' + \
              '_{:d}x{:d}___'.format(set_size[0], set_size[1]) + \
              name + '.data', 'wb') as f:
        
        pickle.dump((exemplar_info), f)   