# -*- coding: utf-8 -*-
"""
Created on Tue Apr 17 01:45:30 2018

@author: Callie Darsey

First attempt at using union over frames to make regions to capture
features and ? (perhaps ishand, isface, isbackofhead, isjunk; perhaps do for
on of the frames in a union segment; perhaps do for frame with region max
motion)

Here we want to make and save the union region maps

first_attempt_union_regions_01
"""

import cv2
import pickle
import numpy as np
import toy_problem_place_cc as place
import scipy.spatial.distance as scipy_dist

def shortest_distance(map_1, map_2):
    '''
    Shortest distance function from VJ.
    '''
    
    map_1, map_1_contours, hierarchy_1 = \
        cv2.findContours(map_1, cv2.RETR_TREE, cv2.CHAIN_APPROX_NONE)
        
    map_2, map_2_contours, hierarchy_2 = \
        cv2.findContours(map_2, cv2.RETR_TREE, cv2.CHAIN_APPROX_NONE)
        
    map_1_points = np.squeeze(np.concatenate(np.array(map_1_contours)))
    map_2_points = np.squeeze(np.concatenate(np.array(map_2_contours)))
        
    if len(map_1_points.shape) == 1:
        map_1_points = np.array([map_1_points])
    if len(map_2_points.shape) == 1:
        map_2_points = np.array([map_2_points])
        
    all_dist = scipy_dist.cdist(map_1_points,map_2_points, 'euclidean')
    
    shortest_distance = np.amin(all_dist)
    
    rows, cols = np.where(all_dist == shortest_distance)
    
    points = np.vstack([map_2_points[i, :] for i in cols])    
    
    return (shortest_distance, points)

def locate_window(title):
    '''
    This function places the window for openCV
    
    '''
    
    move_window = [0, -30]
    cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(title, move_window[0], move_window[1]) 
    

def centroid_center_bounding_box(cc, cc_label):
    '''
    This function finds the top, left, width, and height for a bounding box
    of a connected component when the bounding box is centered around the
    centroid (centroid forced to an integer).

    cc - list holding output of cv2.connectedComponentsWithStats with
         any appended information 
    cc_label - connected component label being cropped
    '''
    
    # centroid x and y forced to int
    c_col = int(cc[3][cc_label, 0])
    c_row = int(cc[3][cc_label, 1])
    
    # points of the bounding box.
    pt1_col = cc[2][cc_label, cv2.CC_STAT_LEFT]
    pt1_row = cc[2][cc_label, cv2.CC_STAT_TOP]
    
    pt2_col = pt1_col + cc[2][cc_label, cv2.CC_STAT_WIDTH] - 1
    pt2_row = pt1_row + cc[2][cc_label, cv2.CC_STAT_HEIGHT] - 1
    
    # left and right column distance
    left_col_dist = c_col - pt1_col
    right_col_dist = pt2_col - c_col
    
    # left and right row distance
    upper_row_dist = c_row - pt1_row
    lower_row_dist = pt2_row - c_row
    
    # row and column with of one side of rectangle excluding center
    col_dist = max([left_col_dist, right_col_dist])
    row_dist = max([upper_row_dist, lower_row_dist])
    
    top = c_row - row_dist
    height = 2 * row_dist + 1
    left = c_col - col_dist
    width = 2 * col_dist + 1

    return top, height, left, width
         

def bigger_bounding_box(frame_dim, cc, cc_label, new_dim):
    '''
    Attempt to form a bounding box of dimension (new_dim[0], new_dim[1]) about
    the centroid (centroid forced to int). If a side of the bounding box found
    by cv2.connectedComponentWithStat is further, use it. If the new dimension
    takes the bounding box outside the frame dimensions, clip it.
    
    The dimensions of new_dim are expected to be odd.
    '''
    
    assert new_dim[0] % 2 == 1
    assert new_dim[1] % 2 == 1

    # centroid x and y forced to int
    c_col = int(cc[3][cc_label, 0]); c_row = int(cc[3][cc_label, 1])
    
    # distance
    row_dist, col_dist = int(new_dim[0] / 2), int(new_dim[1] / 2)
    
    # indices to the new sides
    top, left, bottom, right = c_row - row_dist, c_col - col_dist,\
                               c_row + row_dist, c_col + col_dist
                               
    # check indices
    if top < 0:
        top = 0
    if top > cc[2][cc_label, cv2.CC_STAT_TOP]:
        top  = cc[2][cc_label, cv2.CC_STAT_TOP]
    
    if left < 0:
        left = 0
    if left > cc[2][cc_label, cv2.CC_STAT_LEFT]:
        left = cc[2][cc_label, cv2.CC_STAT_LEFT]
        
    if bottom >= frame_dim[0]:
        bottom = frame_dim[0] - 1
    if bottom < cc[2][cc_label, cv2.CC_STAT_TOP] + \
                cc[2][cc_label, cv2.CC_STAT_HEIGHT] - 1:
        bottom = cc[2][cc_label, cv2.CC_STAT_TOP] + \
                 cc[2][cc_label, cv2.CC_STAT_HEIGHT] - 1
    
    if right >= frame_dim[1]:
        right = frame_dim[1] - 1
    if right < cc[2][cc_label, cv2.CC_STAT_LEFT] + \
               cc[2][cc_label, cv2.CC_STAT_WIDTH] - 1:
        right = cc[2][cc_label, cv2.CC_STAT_LEFT] + \
                cc[2][cc_label, cv2.CC_STAT_WIDTH] - 1
                               
    width = right - left + 1
    height = bottom - top + 1

    return top, left, height, width                           
                               

def bigger_bounding_box_02(frame_dim, cc_top, cc_height, cc_left, cc_width,
                           cc_centroid, new_dim):
    '''
    Attempt to form a bounding box of dimension (new_dim[0], new_dim[1]) about
    the centroid (centroid forced to int). If a side of the bounding box found
    by cv2.connectedComponentWithStat is further, use it. If the new dimension
    takes the bounding box outside the frame dimensions, clip it.
    
    The dimensions of new_dim are expected to be odd.
    '''
    
    assert new_dim[0] % 2 == 1
    assert new_dim[1] % 2 == 1

    # centroid x and y forced to int
    c_col = int(cc_centroid[0]); c_row = int(cc_centroid[1])
    
    # distance
    row_dist, col_dist = int(new_dim[0] / 2), int(new_dim[1] / 2)
    
    # indices to the new sides
    top, left, bottom, right = c_row - row_dist, c_col - col_dist,\
                               c_row + row_dist, c_col + col_dist
                               
    # check indices
    if top < 0:
        top = 0
    if top > cc_top:
        top = cc_top
    
    if left < 0:
        left = 0
    if left > cc_left:
        left = cc_left
        
    if bottom >= frame_dim[0]:
        bottom = frame_dim[0] - 1
    if bottom < cc_top + cc_height - 1:
        bottom = cc_top + cc_height - 1
    
    if right >= frame_dim[1]:
        right = frame_dim[1] - 1
    if right < cc_left + cc_width - 1:
        right = cc_left + cc_width - 1
                               
    width = right - left + 1
    height = bottom - top + 1

    return top, left, height, width    


# specify paths to where the frame and flow are pickled
common = './video_copies_clips_track/copies_and_clips_facilitator_C/'
specific = 'STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/'
videoname = 'MVI_0028_trans__200_208.mp4'


''' We have the pickled frame and flow have names in the form
    flow_<video clip name>_<####>_<####>.data, and those frames
    have 3 frames dropped in between frames where flow is calculated.
'''
framesToAdvance = 4

# number of frames to take union over flow mag. thresholded
union_num = 10

# threshold on flow mag. thresholded
thresh = 7.0

# collect
thresh_flow_mag_collect = []
flow_mag_collect = []
Y_collect = []

# loop through
count = 0
for fn in range(0, framesToAdvance * union_num, framesToAdvance):
    
    fn1 = '{:04d}'.format(fn)
    fn2 = '{:04d}'.format(fn + framesToAdvance)
    
    count = count + 1
    print(count)
        
    # load the frame and flow
    with open(common + specific + 'flow_' + videoname[0:-4] + \
              '_{}_{}.data'.format(fn1, fn2), 'rb') as f:
        
        p = pickle.load(f)
        
    frame1, frame2, flow = p[0], p[1], p[2]
    
    # flow magnitude
    flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
    
    # collect flow magnitude
    flow_mag_collect.append(flow_mag)
    
    # collect the thresholded flow
    thresh_flow_mag_collect.append(flow_mag > thresh)
    
    # collect gray frame
    Y_collect.append(cv2.cvtColor(frame1, \
                                  cv2.COLOR_BGR2GRAY).astype(np.uint16))
    

# union mask
union_mask = np.any(thresh_flow_mag_collect, axis=0).astype(np.uint8) * 255

# flow magnitude sum and flow magnitude sum for display
flow_mag_sum = np.sum(flow_mag_collect, axis=0)


clip_val = 7 * union_num
flow_mag_sum_disp = np.copy(flow_mag_sum)
clip = np.where(flow_mag_sum > clip_val)

if clip[0].size or clip[1].size:
    
    flow_mag_sum_disp[clip[0], clip[1]] = clip_val

flow_mag_sum_disp = (255 * flow_mag_sum_disp / clip_val). astype(np.uint8)



# gray frame sum for display
Y_sum = (  255 * np.sum(Y_collect, axis=0).astype(np.float) / \
         ( 255 * union_num )).astype(np.uint8)

# display gray sum
title = 'Y sum'; locate_window(title); cv2.imshow(title, Y_sum)

# display flow mag. sum
title = 'flow magnitude sum'
locate_window(title); cv2.imshow(title, flow_mag_sum_disp)

# display union mask
title = 'union'; locate_window(title); cv2.imshow(title, union_mask)


'''
reckon I need some type of clean up descisions

1) reject components near edges of image

2) smaller regions get bigger bounding boxes

3) if bounding boxes overlap and regions are small, combine region

'''

e1 = cv2.getTickCount()

'''don't deal with unions near the border'''
# boarder mask
border = 20

border_mask = np.ones_like(union_mask)
border_mask[border:border_mask.shape[0] - border,
            border:border_mask.shape[1] - border] = 0

# get connected component info
cc_unions = list(cv2.connectedComponentsWithStats(union_mask))

# flag for removal
remove = []
for label in range(0, cc_unions[0]):
    
    label_matrix = cc_unions[1] == label
    
    # take action when cc overlaps border
    overlap = np.count_nonzero(np.all([label_matrix, border_mask], axis=0))
    if overlap > 0:
        
       # over laps border completely
       if overlap == cc_unions[2][label, cv2.CC_STAT_AREA]:
           remove.append(label)
           
       # overlaps partially
       elif overlap > 0.25 * cc_unions[2][label, cv2.CC_STAT_AREA]:
           remove.append(label)
    
r, c = np.where(np.any([cc_unions[1] == label for label in remove], axis=0))
cc_unions[1][r, c] = 0

#union_mask = cc_unions[1].astype(np.bool).astype(np.uint8) * 255








m = np.array([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0],
              [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0],
              [0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0],
              [0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0],
              [0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0],
              [0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0],
              [0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0],
              [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0],
              [0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0],
              [0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0],
              [0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
              [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]],
             dtype=np.uint8) * 255


union_mask = np.zeros((300, 300), dtype=np.uint8)
union_mask[0:20, 0:24] = m
#union_mask[16:33, 16:33] = 255
union_mask[10:13, 21:115] = 255
union_mask[13:17, 21:22] = 255
union_mask[16:17, 16:21] = 255
union_mask[21:42, 0:14] = 255












cc_unions = list(cv2.connectedComponentsWithStats(union_mask))
    
# display union mask after removing components near border
title = 'union'; locate_window(title); cv2.imshow(title, union_mask)

# print the areas
[print('{:10}'.format(area)) \
    for area in cc_unions[2][1:, cv2.CC_STAT_AREA]]


'''bounding boxes, smaller regions get larger bounding boxes'''
union_mask_boxes = np.stack([union_mask, union_mask, union_mask], axis=2)


small_cc = np.where(cc_unions[2][0:, cv2.CC_STAT_AREA]  < 200)[0]
big_cc = np.where(cc_unions[2][0:, cv2.CC_STAT_AREA]  >= 200)[0][1:]

associated = {}
for label in big_cc:
    associated[label] = []

big_cc_map = np.any([cc_unions[1] == label for label in big_cc], axis=0)





for label in small_cc:
    
    print(' ')
    print(label)
    
    label_matrix = (cc_unions[1] == label)
    
    map_1 = label_matrix.astype(np.bool).astype(np.uint8)
    map_2 = big_cc_map.astype(np.bool).astype(np.uint8)
    

    shortest_dist, points = shortest_distance(map_1, map_2)
    
    print(shortest_dist)
    print(points)
    
    if shortest_dist < 10:
        
        if len(points) > 1:
            
            print('contending')
            
            contending_labels = []
            
            for point in points:
                contending_labels.append(cc_unions[1][point[1],
                                                      point[0]])
    
            print(contending_labels)
    
            contending_labels = np.array(contending_labels)
            contending_labels = np.unique(contending_labels)
            
            if len(contending_labels) == 1:
                
                print('from same component')
                
                associated[cc_unions[1][points[0, 1],
                                        points[0, 0]]].append(label)
                
            else:
                
                print('from different components')
            
                mx = np.amax(cc_unions[2][contending_labels, cv2.CC_STAT_AREA])
                
                mx_ind = np.where(cc_unions[2][contending_labels,
                                               cv2.CC_STAT_AREA] == mx)[0]
    
                if len(mx_ind) == 1:
                    
                    print('clear area winner')
                    
                    associated[contending_labels[mx_ind[0]]].append(label)
                    
                elif len(mx_ind) > 1:
                    
                    print('Multiple should get due to same area')
                    
                    for ind in mx_ind:
                        
                        associated[contending_labels[ind]].append(label)

        else:
            
            print('one point')
            
            associated[cc_unions[1][points[0, 1], points[0, 0]]].append(label)
    
            
small_grouped = np.unique(np.hstack(list(associated.values())))  

small_rem = np.setdiff1d(small_cc, small_grouped)

bounding_box_small_rem = {}
[bounding_box_small_rem.update({label : np.zeros((1, 4))}) for label in small_rem]

# establish boxes around remaining small components
if len(small_rem) > 0:
    
    for label in small_rem:
        
        if cc_unions[2][label, cv2.CC_STAT_HEIGHT] * \
           cc_unions[2][label, cv2.CC_STAT_WIDTH] < 1225:
               
            top, left, height, width = \
                bigger_bounding_box_02(cc_unions[1].shape,
                                       cc_unions[2][label, cv2.CC_STAT_TOP],
                                       cc_unions[2][label, cv2.CC_STAT_HEIGHT],
                                       cc_unions[2][label, cv2.CC_STAT_LEFT],
                                       cc_unions[2][label, cv2.CC_STAT_WIDTH],
                                       cc_unions[3][label, :],
                                       (35, 35))

            bounding_box_small_rem[label][0, cv2.CC_STAT_TOP] = top
            bounding_box_small_rem[label][0, cv2.CC_STAT_HEIGHT] = height
            bounding_box_small_rem[label][0, cv2.CC_STAT_LEFT] = left
            bounding_box_small_rem[label][0, cv2.CC_STAT_WIDTH] = width
            
        else:
            
            bounding_box_small_rem[label][0, cv2.CC_STAT_TOP] = \
                cc_unions[3][label, cv2.CC_STAT_TOP]
            bounding_box_small_rem[label][0, cv2.CC_STAT_HEIGHT] = \
                cc_unions[3][label, cv2.CC_STAT_HEIGHT]
            bounding_box_small_rem[label][0, cv2.CC_STAT_LEFT] = \
                cc_unions[3][label, cv2.CC_STAT_LEFT]
            bounding_box_small_rem[label][0, cv2.CC_STAT_WIDTH] = \
                cc_unions[3][label, cv2.CC_STAT_WIDTH]  

# figure out i f small boxes overlap and group    
    
        
            
        
    

    
    
    
    
    
    


#for label in range(0, cc_unions[0]):
#
#    if cc_unions[2][label, cv2.CC_STAT_AREA] < 200:
#            
#        box_area = cc_unions[2][label, cv2.CC_STAT_HEIGHT] * \
#                   cc_unions[2][label, cv2.CC_STAT_WIDTH]
#       
#        # build bounding box that has an area of at least 1225
#        if cc_unions[2][label, cv2.CC_STAT_HEIGHT] * \
#           cc_unions[2][label, cv2.CC_STAT_WIDTH] < 1225:
#               
#            top, left, height, width = \
#                bigger_bounding_box(union_mask_boxes.shape,
#                                    cc_unions, label, (35, 35))
#            
#            cv2.rectangle(union_mask_boxes, (left, top),
#                          (left + width - 1, top + height - 1), (0, 170, 0), 1)
#                  
#           
# display union mask after drawing boxes
title = 'union'; locate_window(title);
cv2.imshow(title, union_mask_boxes)           
           

cv2.waitKey(0)
    
cv2.destroyAllWindows()
    
    




















