# -*- coding: utf-8 -*-
"""
Created on Tue Apr 24 00:05:08 2018

@author: Callie Darsey

Save sampled video with  flow arrows and with numbers
"""

# -*- coding: utf-8 -*-
"""
Created on Fri Mar 23 16:29:00 2018

@author: Callie Darsey

Calculate optical flow forwards and backwards for a clip.
"""

import cv2
import pickle
import numpy as np
import os
import fns_investigating_optical_flow_01 as fns


# video path
common_path = './video_copies_clips_track/copies_and_clips_wave_04/'
specific_path = 'STI Polk Apr27 A/'
out_folder = 'MVI_0073_trans/'
video_name = 'MVI_0073_trans__0_100.mp4'

video_path = common_path + specific_path + out_folder + video_name

'''Frames to advance between frames passed to optical flow'''
framesToAdvance = 4

'''Resolution rescaling factor'''
scaleFactor = 0.67


'''
Optical flow parameters for cv2.calcOpticalFlowFarneback

docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''
pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0
      
# process the mp4 video
cap = cv2.VideoCapture(video_path)

num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
fps = cap.get(cv2. CAP_PROP_FPS )

print('number of frames:', num_frames)
print('frames per second:', fps)

print('framesToAdvance:', framesToAdvance)
print('scaleFactor:', scaleFactor)


# initialize frame1
success_flag, frame1_full = cap.read()

assert success_flag, "On first attempt after construction, could \
                      not read frame from {}".format(video_path)  
                            
frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)


# write frames with arrows and frame number drawn
fourcc = cv2.VideoWriter_fourcc(*'XVID')
out = cv2.VideoWriter(video_path[0:-3:] + 'arrows.mp4',fourcc, 15,
                      frame1.shape[-2::-1])

# loop through the frames

e1 = cv2.getTickCount()

count = 0


while(cap.isOpened()):
    
    eb = cv2.getTickCount()
                
    # skip frames
    for i in range(framesToAdvance):
        success_flag, frame2_full = cap.read()
        
        count += 1

        if (not success_flag):
            print("no more frames in this capture")
            break
     
    if success_flag:
    # process upon true read frame success_flag

        #initialize frame2
        frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
        frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)
        

        # calculate the flow between frame1 and frame2    
        flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                            flow=None,
                                            pyr_scale=pyr_scale,
                                            levels=levels,
                                            winsize=winsize,
                                            iterations=iterations,
                                            poly_n=poly_n,
                                            poly_sigma=poly_sigma,
                                            flags=flags)
        
        # draw arrows
        arrows = np.copy(frame1)    
        arrows = fns.draw_flow(arrows, flow,
                               np.ones(frame1.shape[0:2], dtype=np.uint8),
                               step=4)

        # draw the text
        cv2.putText(arrows, '{}'.format(count - framesToAdvance),
                    (arrows.shape[1] - 100, 25),
                    cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2) 
        
        # save frame
        out.write(arrows)    
        
        # update frame1
        frame1 = frame2
        frame1_gray = frame2_gray
        
        
        
        ee = cv2.getTickCount()
        print('{:04d}'.format(count - framesToAdvance),
              'flow calc {:.2f} seconds'.format((ee - eb)/ cv2.getTickFrequency()))

    else:

        # release capture
        cap.release()
        out.release()

e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)
