# -*- coding: utf-8 -*-
"""
Created on Sun Jan 21 10:36:08 2018

@author: Callie Darsey

Pasting in functions (from ECE 633 course) for use or alteration and use
for investigating optical flow.

fns_investigating_optical_flow_01
"""
import cv2
import numpy as np

def draw_flow(frame, frame_flow, mask, step=1):
    """ Plot optical flow at sample points spaced step pixels apart. """
    
    # sample the points at which the vectors will be drawn
    
    # get image dimensions
    h,w = frame.shape[:2]
    
    # get row and column indices for all points in two 1D numpy arrays
    y,x = np.mgrid[np.floor(step/2):h:step, \
                   np.floor(step/2):w:step].reshape(2,-1)
    
    # get the flow components at each point
    flow_x, flow_y = frame_flow[y.astype(int), x.astype(int)].T
        
    # create line endpoints
    lines = np.vstack([x, y, x + flow_x, y + flow_y]).T.reshape(-1,2,2)
    lines = np.int32(lines)
    
    if len(frame.shape) == 2:
        frame_gray_3D = np.stack([frame, frame, frame], axis=2)
    else:
        frame_gray_3D = frame

    for (x1,y1),(x2,y2) in lines:
 
        # check if the sample point is within the regions kept after thresholding
        cond = (mask[y1, x1] == True or mask[y1, x1] != 0)
        if (cond):
            cv2.line(frame_gray_3D,(x1,y1),(x2,y2),(0,255,0),1)
            #cv2.circle(frame_gray_3D,(x1,y1),1,(0,255,0), -1)
    
    return frame_gray_3D