# -*- coding: utf-8 -*-
"""
Created on Fri May 18 14:01:11 2018

@author: Callie Darsey

gather_pieces_01
"""

import cv2
import numpy as np
import pickle
import matplotlib.pyplot as plt
import os

def get_area_overlap(patch_row_1, patch_row_2,
                     patch_col_1, patch_col_2,
                     exemplar_row_1, exemplar_row_2,
                     exemplar_col_1, exemplar_col_2):
    '''
    '''
    # rows
    if not (patch_row_2 < exemplar_row_1 or patch_row_1 > exemplar_row_2):
        
        row_1 = np.max([patch_row_1, exemplar_row_1])
        row_2 = np.min([patch_row_2, exemplar_row_2])
        
    else:
        
        row_1 = -1
        row_2 = -1
    
    # cols    
    if not (patch_col_2 < exemplar_col_1 or patch_col_1 > exemplar_col_2):
        
        col_1 = np.max([patch_col_1, exemplar_col_1])
        col_2 = np.min([patch_col_2, exemplar_col_2])
        
    else:
        
        col_1 = -1
        col_2 = -1
    
    # area    
    if row_1 < 0 or row_2 < 0 or col_1 < 0 or col_2 < 0:
        
        area = 0
        
    else:
        
        area = (row_2 - row_1 + 1) * (col_2 - col_1 + 1)
        
    return area

places1 = [\
'part1/MVI_0069_trans/',
'STI_Polk_Apr13_C/MVI_0039_trans/',
'STI_Polk_Apr27_A/MVI_0073_trans/',
'STI_Polk_Apr27_D/MVI_0063_trans/',
'STI_Polk_May04_A/MVI_0087_trans/',
'STI_Polk_May11_C/MVI_0108_trans/',
'STI_WA_Apr11_extra_B/MVI_0036_trans/',
'STI_WA_Jun12_B/MVI_0018_trans/',
'STI_WA_Jun14_Lb/MVI_0070_trans/',
'STI_WA_Jun20_CT/MVI_0029_trans/',
'STI_WA_Jun22_De/MVI_0118_trans/',
'STI_WA_Jun23_Da/MVI_0112_trans/',
'STI_WA_May02_B/MVI_0053_trans/',
'STI_WA_May02_C/MVI_0074_trans/',
'STI_WA_May09_A/MVI_0109_trans/']

places2 = [\
'part1/MVI_0069_trans/',
'STI Polk Apr13 C/MVI_0039_trans/',
'STI Polk Apr27 A/MVI_0073_trans/',
'STI Polk Apr27 D/MVI_0063_trans/',
'STI Polk May04 A/MVI_0087_trans/',
'STI Polk May11 C/MVI_0108_trans/',
'STI WA Apr11 extra B/MVI_0036_trans/',
'STI WA Jun12 B/MVI_0018_trans/',
'STI WA Jun14 Lb/MVI_0070_trans/',
'STI WA Jun20 CT/MVI_0029_trans/',
'STI WA Jun22 De/MVI_0118_trans/',
'STI WA Jun23 Da/MVI_0112_trans/',
'STI WA May02 B/MVI_0053_trans/',
'STI WA May02 C/MVI_0074_trans/',
'STI WA May09 A/MVI_0109_trans/']

for place1, place2 in zip(places1, places2):


    directory_w = "./wheeler/dataset/" + place1
    
    directory = "./video_copies_clips_track/copies_and_clips_wave_04/" + place2


    for content in os.listdir(directory_w):
        
        if os.path.isfile(directory_w + content):
            
            if content[-12:] == '.sampled.mp4':
                
                name = content[:-12]
                
                print(name)
                
                break
    
    path_exemplars_and_labels = \
        directory + 'exemplars_and_labels___' + name + '.data'
        
    with open(path_exemplars_and_labels, 'rb') as f:
        
        exemplars_and_labels = pickle.load(f)
    
    
    
    
    
    '''form the patch box info'''
    patch_size = 76
    
    frame_rows, frame_cols = 724, 1286
    
    
    row_starts_1 = np.arange(0, frame_rows - patch_size,
                             patch_size)
    
    row_starts_2 = np.arange(int(patch_size / 2), frame_rows - patch_size,
                             patch_size)
    
    
    col_starts_1 = np.arange(0, frame_cols - patch_size,
                             patch_size)
    
    col_starts_2 = np.arange(int(patch_size / 2), frame_cols - patch_size,
                             patch_size)
    
    if row_starts_1[-1] != (frame_rows - patch_size) and \
        row_starts_2[-1] != (frame_rows - patch_size):
        
        row_starts = np.hstack([row_starts_1, row_starts_2,
                                np.array([frame_rows - patch_size])])
        
    else:
        
        row_starts = np.hstack([row_starts_1, row_starts_2])    
    
        
    if col_starts_1[-1] != (frame_cols - patch_size) and \
        col_starts_2[-1] != (frame_cols - patch_size):
        
        col_starts = np.hstack([col_starts_1, col_starts_2,
                                np.array([frame_cols - patch_size])])
        
    else:
        
        col_starts = np.hstack([col_starts_1, col_starts_2]) 
        
    row_starts.sort()
    col_starts.sort()
    
    patch_boxes = np.zeros((row_starts.shape[0] * col_starts.shape[0], 4),
                           dtype=np.uint32)
    
    i = -1
    
    for row_start in row_starts:
        
        for col_start in col_starts:
            
            i = i + 1
            
            patch_box = np.array([col_start, row_start,
                                  col_start + patch_size - 1,
                                  row_start + patch_size - 1], dtype=np.uint32)
        
#            print(patch_box)
            
            patch_boxes[i, :] = patch_box
    
    ''' initialize'''    
    patches_all_segments_labels = {}
    
    ''' hack to determine if we are handling a 13 segment video or a 
        33 segment video '''
    exemplar_keys = list(exemplars_and_labels.keys())
        
    if len(list(exemplars_and_labels.keys())) < 20:
        
        seg_keys = [(first, last) \
                    for first, last in zip(np.arange(0, 2160 + 180, 180),
                                           np.arange(176, 2336 + 180, 180))]
                    
    elif len(list(exemplars_and_labels.keys())) >= 20:
        
        seg_keys = [(first, last) \
                       for first, last in zip(np.arange(0, 5760 + 180, 180),
                                              np.arange(176, 5936 + 180, 180))]

    ''' initialize '''
    for key in seg_keys:

        patches_all_segments_labels[key] = [patch_boxes, [''] * len(patch_boxes)]
    
    '''fill the tags'''
    for p, patch_box in enumerate(patch_boxes):
        
        patch_col_1, patch_row_1, patch_col_2, patch_row_2 = patch_box
                
        for key in seg_keys:
            
            if key in exemplar_keys:
                
                exemplars, tags = exemplars_and_labels[key]
                
                possible_tag = []
                overlap_exemplar_ratio = []
                #overlap_patch_ratio = []
                
                for box, tag in zip(exemplars, tags):
        
                    exemplar_col_1, exemplar_col_2 = np.sort([box[0], box[2]])
                    
                    exemplar_row_1, exemplar_row_2 = np.sort([box[1], box[3]])            
            
                    area_overlap = get_area_overlap(patch_row_1, patch_row_2,
                                                    patch_col_1, patch_col_2,
                                                    exemplar_row_1, exemplar_row_2,
                                                    exemplar_col_1, exemplar_col_2)
    
                    overlap_to_exemplar = area_overlap / \
                                       ((exemplar_col_2 - exemplar_col_1 + 1) * \
                                        (exemplar_row_2 - exemplar_row_1 + 1))
                    
                    overlap_to_patch = area_overlap / \
                                       ((patch_col_2 - patch_col_1 + 1) * \
                                        (patch_row_2 - patch_row_1 + 1))
    
                    # ensure patch overlaps at least 10% of exemplar to consider
    
                    if overlap_to_exemplar >= 0.2 or overlap_to_patch > 0.8:
    
                        possible_tag.append(tag)
                        
                        overlap_exemplar_ratio.append(overlap_to_exemplar)
        
                        #overlap_patch_ratio.append(overlap_to_patch)              
    
                if len(possible_tag) == 0:
                    
                    patch_tag = 'predominately non-exemplar'
                    
                else:
                    
                    index_overlap_exemplar_ratio = np.argmax(overlap_exemplar_ratio)
                    patch_tag = possible_tag[index_overlap_exemplar_ratio]
                    #index_overlap_patch_ratio = np.argmax(overlap_patch_ratio)
                    
                    #if overlap_exemplar_ratio[index_overlap_exemplar_ratio] > \
                    #   overlap_patch_ratio[index_overlap_patch_ratio]:
                                    
                    #    patch_tag = possible_tag[index_overlap_exemplar_ratio]
                        
                    #elif overlap_patch_ratio[index_overlap_patch_ratio] > \
                    #    overlap_exemplar_ratio[index_overlap_exemplar_ratio]:
                            
                    #    patch_tag = possible_tag[index_overlap_exemplar_ratio]
                        
                    #else:
                    #    patch_tag = 'predominately both'
                
                patches_all_segments_labels[key][1][p] = patch_tag
    
            else:
                
                patches_all_segments_labels[key][1][p] = 'predominately non-exemplar'
                
    path_patches_all_segments_labels = \
        directory_w + 'patches_all_segments_labels___' + name + '.data'
    
    with open(path_patches_all_segments_labels, 'wb') as f:
        
        pickle.dump((patches_all_segments_labels), f)                
    
    print('"{}"'.format(path_patches_all_segments_labels))        