# -*- coding: utf-8 -*-
"""
Created on Mon Apr 30 10:03:44 2018

@author: Callie Darsey

getting_exemplars_labels_01
"""


import numpy as np
import cv2
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import os
import overlay_01 as overlay
import Adjustable_BB_05 as Adjustable_BB
import datetime
import pickle
import sys

# video path
path_common = './video_copies_clips_track/copies_and_clips_wave_04/'
path_specific = 'STI Polk May04 A/MVI_0087_trans/'
video_name = 'MVI_0087_trans__579_680.mp4'
video_path = path_common + path_specific + video_name

'''
bool for whether the input video already has reduced frames
'''
reduced_frames = False
             

# Setting to play with

'''Resolution rescaling factor'''
scaleFactor = 0.67
#scaleFactor = 1.0

'''
normalization of optical flow by resolution scaling factor and 
number of frames advanced
'''
normalize_scale = False
normalize_frame_advance = False

''' wait to calculate optical flow and display '''
frame_wait = 0

'''
Optical flow parameters for cv2.calcOpticalFlowFarneback

docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''

pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0


'''
write out a video
'''
video_write = False

'''
write out images
'''
image_write = False

'''
Whether to display the video
'''
displayVideo = True

'''
Whether to calculate optical flow
'''
calc_flow = False

'''
insert frame number into frame
'''
insert_frame_number = True

'''
draw existing exemplars into frame
'''
place_existing_exemplars = True

'''
hold exemplar point info
'''
exemplar_points = {}

'''pickle info old'''
use_old_exemplar = True
#exemplar_old = sys.argv[1]
if use_old_exemplar == True:
    
    #pickle_path_old = path_common + path_specific + exemplar_old
    pickle_path_old = sys.argv[1]
                  
    with open(pickle_path_old, 'rb') as f:
        exemplar_points = pickle.load(f)

'''pickle info new'''
n = datetime.datetime(1, 1, 1).now()
timestamp = 'y{:d}m{:02d}d{:02d}-h{:02d}m{:02d}s{:02d}.{:d}'. \
            format(n.year, n.month, n.day,
                   n.hour, n.minute, n.second, n.microsecond)
#pickle_path = path_common + path_specific + \
#              'exemplars___' + video_name[0:-4] + '___'+ \
#              timestamp + '.data'
pickle_path = path_common + path_specific + \
              'exemplars_and_labels___' + video_name[0:-4] + '.data'
              
exemplar_points_and_labels = {}

      
# process the mp4 video
cap = cv2.VideoCapture(video_path)

num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
fps = cap.get(cv2.CAP_PROP_FPS )

'''Frames to advance between frames passed to optical flow'''
framesToAdvance = np.int32(np.ceil(fps / 15))

print('number of frames:', num_frames, ', frames per second:', fps,
      'framesToAdvance:', framesToAdvance)



# initialize frame1
success_flag, frame1_full = cap.read()

assert success_flag, "On first attempt after construction, could \
                      not read frame from {}".format(video_path)  
                            
frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)

'''
save frames
'''
segment = True
seconds = 3
if segment == True:
    num_frames = np.int32(np.ceil(seconds * fps))
    number_saved = np.int32(np.ceil(seconds * fps / framesToAdvance))
    print('number saved:', number_saved)
    
else:
    num_frames = 300
    number_saved = 75


saved_frames = []
frame_number = []
ind = -1

# prep window
title = 'display window'
move_window = [60, -30]
cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
cv2.moveWindow(title, move_window[0], move_window[1])
#cv2.moveWindow(title, -468, -1111)
#cv2.moveWindow(title, -234, -900)

# loop through the frames and display

# announce
print("Press Esc on the image to exit")

e1 = cv2.getTickCount()

count = 0


# for when write_video is true
if video_write == True:
    
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    root, ext = os.path.splitext(video_path)
    video_path_less_frames = root + '__skipped_' + str(framesToAdvance - 1) + ext
    out = cv2.VideoWriter(video_path_less_frames, fourcc, 20, 
                          tuple(np.flip(frame1_full.shape[0:2], 0)))
    
    if out.isOpened():
        out.write(frame1_full)
    else:
        print('\nout not open\n')
        
# for when image_write is true
if image_write == True:
    
    image_path = path_common + path_specific + 'images/' + \
        video_name[0:-4] + '__'+ '{:05d}'.format(count + 1) + '.png'
    
    cv2.imwrite(image_path, frame1)

# last frame got was the last
last_frame = False

while(cap.isOpened()):
                
    # skip frames
    for i in range(framesToAdvance):
        success_flag, frame2_full = cap.read()
        
        count += 1

        if (not success_flag):
            print("no more frames in this capture")
            last_frame = True
                 
    if success_flag or last_frame == True:
    # process upon true read frame success_flag
    
        if last_frame is False:
    
            if video_write:
        
                if out.isOpened():
                    out.write(frame2_full)
                else:
                    print('\nout not open\n')
                    
            #initialize frame2
            frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
            frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)
    
            if image_write == True:    
                
                image_path = path_common + path_specific + 'images/' + \
                video_name[0:-4] + '__'+ '{:05d}'.format(count + 1) + '.png'
                
                cv2.imwrite(image_path, frame2)
            
            
            if count > frame_wait and calc_flow == True:
                # calculate the flow between frame1 and frame2    
                flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                                    flow=None,
                                                    pyr_scale=pyr_scale, levels=levels,
                                                    winsize=winsize,
                                                    iterations=iterations,
                                                    poly_n=poly_n,
                                                    poly_sigma=poly_sigma, flags=flags)
                
                mag = np.sqrt(flow[...,0]**2 + flow[..., 1]**2)
                
                bp1 = plt.boxplot(np.reshape(mag, -1))
                plt.show()
                
                mask1 = mag > bp1['whiskers'][1].get_data()[1][1]
                
                bp2 = plt.boxplot(mag[mask1])
                plt.show()
                
                mask = (mag > bp2['medians'][0].get_data()[1][0]).astype(np.uint8)
                
                kernel = np.ones((5,5),np.uint8)
                
                mask = cv2.morphologyEx(mask, cv2.MORPH_OPEN, kernel)
                mask = cv2.dilate(mask, kernel)
                
                
                
                
            if displayVideo == True:
                
                if count > frame_wait and calc_flow == True:
    
                    if normalize_scale and normalize_frame_advance:                
                        frame_to_diplay = fns.draw_flow(frame1_gray,
                                                        flow * (1 / scaleFactor) \
                                                        / framesToAdvance,
                                                        mask, step=4)
                    elif normalize_scale:                
                        frame_to_diplay = fns.draw_flow(frame1_gray,
                                                        flow * (1 / scaleFactor),
                                                        mask, step=4)
                    elif normalize_frame_advance:                
                        frame_to_diplay = fns.draw_flow(frame1_gray,
                                                        flow / framesToAdvance,
                                                        mask, step=4)
                    else:                
                        frame_to_diplay = fns.draw_flow(frame1_gray, flow,
                                                        mask, step=4)  
                elif calc_flow == False:
                    
                    frame_to_display = frame1
    
                if insert_frame_number == True:
                                    
                    # draw the text
                    cv2.putText(frame_to_display, str(count - framesToAdvance),
                                (frame_to_display.shape[1] - 100, 25),
                                 cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)
                    
                if place_existing_exemplars == True:
                    
                    if len(exemplar_points.keys()) > 0:
                        
                        frame_num = (count - framesToAdvance)
                        
                        ranges = np.array(list(exemplar_points.keys()))
                        
                        relevant = np.where(np.all([ranges[:, 0] <= int(frame_num),
                                    ranges[:, 1] >= int(frame_num)], axis=0))[0]
        
                        if len(relevant) > 0:
                            
                            for rel in relevant:
                                
                                key = tuple(ranges[rel])
                                
                                for pts in exemplar_points[key]:
                                    
                                    cv2.rectangle(frame_to_display,
                                                  tuple(pts[0:2]),
                                                  tuple(pts[2:]),
                                                  (255, 255, 0))
    
    
                cv2.imshow(title, frame_to_display)
    
            saved_frames.append(frame_to_display)
            
            if reduced_frames == True:
                frame_number.append(str((count - framesToAdvance) * 4))
            else:
                frame_number.append(str(count - framesToAdvance))
            
            if len(saved_frames) > number_saved:
                saved_frames.pop(0)
                frame_number.pop(0)
            else:
                ind = ind + 1
    
    
            # update frame1
            frame1 = frame2
            frame1_gray = frame2_gray
            
        else:
            # draw the text
            cv2.putText(saved_frames[-1], 'last', (25, 25),
                        cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)
            
            cv2.imshow(title, saved_frames[-1])
            
                
        # Esc key press will escape
        k = cv2.waitKey(1) & 0xff
        if k == 27:

            # release capture
            cap.release()

            # close all the OpenCV windows
            cv2.destroyAllWindows()
            
            break
        # space key will pause
        elif k == 32 or (segment == True and count % num_frames == 0):
            
            frame_ranges = np.array(list(exemplar_points.keys()))
            indices = \
                np.where(np.all([frame_ranges[:,0] <= (count-framesToAdvance),
                                 frame_ranges[:,1] >= (count-framesToAdvance)],
                                axis=0))[0]
    
            if len(indices) > 0:
                
                # list to hold the labels
                tags = []
                
                key = tuple(frame_ranges[indices[0]])
                
                for pts in exemplar_points[key]:
                    
                    # mark the particular exemplar purple                    
                    for i, saved_frame in enumerate(saved_frames):
                        
                        cv2.rectangle(saved_frame, tuple(pts[0:2]),
                                      tuple(pts[2:]), (163, 13, 128))
                    
                    # update display
                    cv2.imshow(title, saved_frames[ind])
                    
                    
                    # allow scrolling          
                    k = -1
                    
                    reply = None
                    
                    # until we get all the exemplars labelled
                    allow_scrolling = True
                    while(allow_scrolling):
                    
                        k = cv2.waitKey(1)
                        
                        # space or escape
                        if k == 27:
        
                            allow_scrolling = False
                        
                        # > arrow
                        elif k == 46:
        
                            if ind < len(saved_frames) - 1:
                                
                                ind = ind + 1
                                cv2.imshow(title, saved_frames[ind])
                        
                        # < arrow
                        elif k == 44:
                            
                            if ind > 0:
                                
                                ind = ind - 1
                                cv2.imshow(title, saved_frames[ind])
                                
                                
                        # e predominantly  head
                        # a predominantly  hand
                        # b both head and hand
                        
                        # e key
                        elif k == 101:
                            reply = 'e'
                            
                        # a key
                        elif k == 97:
                            reply = 'a'
                            
                        # b key
                        elif k == 98:
                            reply = 'b'
                                
                        # save tag
                        if reply is not None:

                            [tags.append(tag) for r, tag in \
                                     zip(['e', 'a', 'b'],
                                         ['predominately head',
                                          'predominately hand',
                                          'predominately both']) if r == reply]
                            
                            allow_scrolling = False                          
                    
                    else:
                        # escape
                        if k == 27:
                            
                            break
                        
                    # mark the particular exemplar purple 
                    color = [bgr for r, bgr in zip(['e', 'a', 'b', None],
                                                   [(0, 0, 255),
                                                    (0, 255, 255),
                                                    (0, 127, 255),
                                                    (255, 255, 0)]) \
                                                    if r is None or r == reply]
                    for i, saved_frame in enumerate(saved_frames):

                        cv2.rectangle(saved_frame, tuple(pts[0:2]),
                                      tuple(pts[2:]), color[0], 1)
                        
                    # update display
                    cv2.imshow(title, saved_frames[ind])
                
                # save the labels / tags
                else:
                    
                    exemplar_points_and_labels.update(\
                        {key : [exemplar_points[key], tags]})                    
                        
                        
            # allow scrolling (unless escape has been pressed)    
            if k == 27:
                allow_scrolling = False
            else:
                
                # until we get another space key
                allow_scrolling = True
                
            while(allow_scrolling):
            
                k = cv2.waitKey(1)
                
                # space or escape
                if k == 32 or k == 27:
            
                    allow_scrolling = False
                
                # > arrow
                elif k == 46:
            
                    if ind < len(saved_frames) - 1:
                        
                        ind = ind + 1
                        cv2.imshow(title, saved_frames[ind])
                
                # < arrow
                elif k == 44:
                    
                    if ind > 0:
                        
                        ind = ind - 1
                        cv2.imshow(title, saved_frames[ind])


             
#            According to "A Byte of Python", "The else block is executed when
#            the while loop condition becomes False - this may even be the first
#            time that the condition is checked. If there is an else clause for
#            a while loop, it is always executed unless you break out of the
#            loop with a break statement."
            
            else:
                
                # escape
                if k == 27:

                    # release capture
                    cap.release()
        
                    # close all the OpenCV windows
                    cv2.destroyAllWindows()
                    
                    break
                
                # space
                if k == 32:
                    
                    get_back = True
                    
                    while(get_back):
                        
                        if ind + 1 == len(saved_frames):
                            
                            get_back = False
                            
                        else:
                            
                            ind = ind + 1
                            
                            cv2.imshow(title, saved_frames[ind])
                            
                            k = cv2.waitKey(1)
                            
                            if k == 27:
                                
                                get_back = False

                    else:
                        
                        # escape
                        if k == 27:
        
                            # release capture
                            cap.release()
                
                            # close all the OpenCV windows
                            cv2.destroyAllWindows()
                            
                            break                            
             
    else:

        # release capture
        cap.release()

        # close all the OpenCV windows
        cv2.destroyAllWindows()

if video_write == True:
    
    out.release()

e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)

# pickle
with open(pickle_path, 'wb') as f:
    pickle.dump((exemplar_points_and_labels), f)