# -*- coding: utf-8 -*-
"""
Created on Wed Mar 21 17:12:42 2018

@author: Callie Darsey

Use less features in Random Forest.

Do grid search on Random Forest parameters.

"""

import component_exploration_04 as fts
import pickle
import numpy as np
import sklearn.ensemble as sk_ensemble
import sklearn.model_selection as sk_ms
import cv2



e1 = cv2.getTickCount()

files = ['way_1_cc_features.data', 'way_2_cc_features.data',
         'way_3_cc_features.data']

for path in files:
    
    print('\n\n', path, '\n')
    
    # load the pickled features
    with open(path, 'rb') as f:
        keep_features, toss_features = pickle.load(f)

    
    # take out some features
    collect_keep_features_to_use = []
    collect_toss_features_to_use = []
    
    
    '''
    starts = [fts.FEATURE_B_PDF_LEFT, fts.FEATURE_G_PDF_LEFT,
              fts.FEATURE_R_PDF_LEFT, fts.FEATURE_Y_PDF_LEFT]
    
    ends = [fts.FEATURE_B_PDF_RIGHT, fts.FEATURE_G_PDF_RIGHT,
            fts.FEATURE_R_PDF_RIGHT, fts.FEATURE_Y_PDF_RIGHT]
    '''
    '''
    starts = [fts.FEATURE_B_PDF_LEFT + combine * 2,
              fts.FEATURE_G_PDF_LEFT + combine * 2,
              fts.FEATURE_R_PDF_LEFT + combine * 2,
              fts.FEATURE_Y_PDF_LEFT + combine * 2]
    
    ends = [fts.FEATURE_B_PDF_RIGHT - combine * 6,
            fts.FEATURE_G_PDF_RIGHT - combine * 6,
            fts.FEATURE_R_PDF_RIGHT - combine * 6,
            fts.FEATURE_Y_PDF_RIGHT - combine * 6]
    '''
    starts = [fts.FEATURE_B_PDF_LEFT,
              fts.FEATURE_G_PDF_LEFT,
              fts.FEATURE_R_PDF_LEFT]
    
    ends = [fts.FEATURE_B_PDF_RIGHT,
            fts.FEATURE_G_PDF_RIGHT,
            fts.FEATURE_R_PDF_RIGHT]
    
    starts = [fts.FEATURE_Y_PDF_LEFT,
              fts.FEATURE_Cr_PDF_LEFT,
              fts.FEATURE_Cb_PDF_LEFT]
    
    ends = [fts.FEATURE_Y_PDF_RIGHT,
            fts.FEATURE_Cr_PDF_RIGHT,
            fts.FEATURE_Cb_PDF_RIGHT]    

    starts = [fts.FEATURE_H_PDF_LEFT,
              fts.FEATURE_S_PDF_LEFT,
              fts.FEATURE_V_PDF_LEFT]
    
    ends = [fts.FEATURE_H_PDF_RIGHT,
            fts.FEATURE_S_PDF_RIGHT,
            fts.FEATURE_V_PDF_RIGHT]
    
    combines = [10, 16, 16]
    for start, end, combine in zip(starts, ends, combines):
        
        for s in range(start, end + 1, combine):
            
            combined = np.sum(keep_features[:, s : s + combine], axis=1)
            combined = combined.reshape((-1, 1))

            collect_keep_features_to_use.append(combined)
            
            combined = np.sum(toss_features[:, s : s + combine], axis=1)
            combined = combined.reshape((-1, 1))

            collect_toss_features_to_use.append(combined)
    
    '''
    print('dft and flow normalized by bin count')
    
    starts = [fts.FEATURE_DFT_MAG_SPEC_HIST_LEFT,
              fts.FEATURE_DFT_PHASE_SPEC_HIST_LEFT,
              fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_LEFT]
    
    ends = [fts.FEATURE_DFT_MAG_SPEC_HIST_RIGHT,
            fts.FEATURE_DFT_PHASE_SPEC_HIST_RIGHT,
            fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_RIGHT]
    '''
    '''
    print('dft and flow normalized by area')
    
    starts = [fts.FEATURE_DFT_MAG_SPEC_HIST_LEFT,
              fts.FEATURE_DFT_PHASE_SPEC_HIST_LEFT,
              fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_LEFT]
    
    ends = [fts.FEATURE_DFT_MAG_SPEC_HIST_RIGHT,
            fts.FEATURE_DFT_PHASE_SPEC_HIST_RIGHT,
            fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_RIGHT]
    '''
    '''
    print('flow normalized by area')
    
    starts = [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_LEFT]
    
    ends = [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_AREA_RIGHT]
    '''
    
    
    print('flow normalized by bin count')
    
    starts = [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_LEFT]
    
    ends = [fts.FEATURE_FLOW_PHASE_BIN_MAG_SUM_HIST_DIV_COUNT_RIGHT]  
    
    
    for start, end in zip(starts, ends):
        
        collect_keep_features_to_use.append(keep_features[:, start : end + 1])
        collect_toss_features_to_use.append(toss_features[:, start : end + 1])
        
        
    for index in [fts.FEATURE_CANNY_EDGE_TO_AREA_RATIO,
                  fts.FEATURE_AREA,
                  fts.FEATURE_PERIMETER,
                  fts.FEATURE_MIN_AREA_RECT_ORIENT,
                  fts.FEATURE_AREA_TO_MIN_AREA_RECT,
                  fts.FEATURE_MIN_AREA_RECT_SHORT_TO_LONG_SIDES_RATIO]:
        
        column = keep_features[:, index].reshape((-1, 1))
        
        collect_keep_features_to_use.append(column)
        
        column = toss_features[:, index].reshape((-1, 1))
        
        collect_toss_features_to_use.append(column)        
     
    # use
    keep_features_used = np.hstack(collect_keep_features_to_use)
    toss_features_used = np.hstack(collect_toss_features_to_use)


        
    # print the training set size
    n_keep_samples, n_keep_features = keep_features_used.shape
    n_toss_samples, n_toss_features = toss_features_used.shape
    
    assert n_keep_features == n_toss_features

    n_features = n_keep_features
    
    print('Number of features: {}'.format(n_features))
    print('Number of keep samples available: {}'.format(n_keep_samples))
    print('Number of toss samples available: {}'.format(n_toss_samples))

    out_of_bag_score = []
    
    trials = 1
    for i in range(trials):
        '''
        Force the class with more samples to have just as many samples by 
        randomly choosing a subset.
        '''
        if n_keep_samples > n_toss_samples:
            
            np.random.shuffle(keep_features_used)
            
            X = np.vstack([keep_features_used[0:n_toss_samples, :],
                           toss_features_used])
        
            y = np.hstack([np.ones(n_toss_samples, dtype=np.int),
                           np.zeros(n_toss_samples, dtype=np.int)])
    
        elif n_keep_samples < n_toss_samples:
            
            np.random.shuffle(toss_features_used)
            
            X = np.vstack([keep_features_used, 
                           toss_features_used[0:n_keep_samples, :]])
        
            y = np.hstack([np.ones(n_keep_samples, dtype=np.int),
                           np.zeros(n_keep_samples, dtype=np.int)])
        
        else:
        
            X = np.vstack([keep_features_used, toss_features_used])
        
            y = np.hstack([np.ones(n_keep_samples, dtype=np.int),
                           np.zeros(n_toss_samples, dtype=np.int)])
    
        # shuffle the training set and targets
        n_samples = X.shape[0]
        
        shuffled_indices = np.arange(n_samples)
        np.random.shuffle(shuffled_indices)
        
        X = X[shuffled_indices, :]
        y = y[shuffled_indices]
        
        # make the classifier
        n_estimators=200
        print(n_estimators, 'trees in the forest')
        rf_clf = sk_ensemble.RandomForestClassifier(n_estimators=n_estimators,
                                                    oob_score=True)
        
        # train
        rf_clf.fit(X, y)
        
        # pickle the classifier 'way_1_cc_features.data'
        with open(path[0:9] + '_' + '{:04d}'.format(n_features) + \
                  '_' + '_RandomForestClassifier' + '.data', 'wb') as f:
            pickle.dump((rf_clf), f)
            
        
        # look at the out-of-bag scores
        print('out-of-bag accuracy score: {:.4f}'.format(rf_clf.oob_score_))
        
        out_of_bag_score.append(rf_clf.oob_score_)
    
    print('For', trials, 'trials: ')
    print('   min out-of-bag accuracy score:', np.min(out_of_bag_score))
    print('  mean out-of-bag accuracy score:', np.mean(out_of_bag_score))
    print('median out-of-bag accuracy score:', np.median(out_of_bag_score))
    print('   max out-of-bag accuracy score:', np.max(out_of_bag_score))
                
    

e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)

