# -*- coding: utf-8 -*-
"""
Created on Fri May  4 14:34:56 2018

@author: Callie Darsey

make_cleaned_skin_videos_wheeler_01
"""


import cv2
import pickle
import attempt_log_reg_AND_knn_with_Cleanup_01 as skin
import sklearn.linear_model as sklm
import sklearn.neighbors as skn

def function():
    
    # video path
    common_path = './'
    video_name = 'MVI_0108_trans__0_42.mp4'
    
    video_path = common_path + video_name
    
    '''Frames to advance between frames passed to optical flow'''
    framesToAdvance = 4
    
    '''Resolution rescaling factor'''
    scaleFactor = 0.67
    
              
    # process the mp4 video
    cap = cv2.VideoCapture(video_path)
    
    num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    fps = cap.get(cv2. CAP_PROP_FPS )
    
    print('number of frames:', num_frames)
    print('frames per second:', fps)
    
    print('framesToAdvance:', framesToAdvance)
    print('scaleFactor:', scaleFactor)
    
    
    # initialize frame1
    success_flag, frame1_full = cap.read()
    print('success_flag', success_flag)
    
    assert success_flag, "On first attempt after construction, could " + \
                          "not read frame from {}".format(video_path)  
                                
    frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
    frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)
    
    
    '''
    We will sample the video.
    Write frames with cleaned skin detection.
    Write sampled frames to a video.
    '''
    fourcc = cv2.VideoWriter_fourcc(*'XVID')
    
    out_skin = cv2.VideoWriter(video_path[0:-3:] + 'cleaned_skin.mp4',
                               fourcc, 15, frame1.shape[-2::-1])
    out_sampled = cv2.VideoWriter(video_path[0:-3:] + 'sampled.mp4',
                                  fourcc, 15, frame1.shape[-2::-1])
    
    # loop through the frames
    
    e1 = cv2.getTickCount()
    
    count = 0
    
    # stuff we need for skin detection by patches
    with open('model_knn_for_block_dim_2.data', 'rb') as f:
        model_knn_for_block_dim_2 = pickle.load(f)
    
    with open('model_log_reg_for_block_dim_2.data', 'rb') as f:
        model_log_reg_for_block_dim_2 = pickle.load(f)
    
    block_dim = 2
    
    while(cap.isOpened()):
        
        eb = cv2.getTickCount()
                    
        # skip frames
        for i in range(framesToAdvance):
            success_flag, frame2_full = cap.read()
            
            count += 1
    
            if (not success_flag):
                print("no more frames in this capture")
                break
         
        if success_flag:
        # process upon true read frame success_flag
    
            #initialize frame2
            frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
            frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)
            
            
            eis = cv2.getTickCount()
            
            # cleaned skin mask
            skin_mask = skin.get_skin_mask(frame1,
                                           model_knn_for_block_dim_2,
                                           model_log_reg_for_block_dim_2,
                                           block_dim)
    
            cleanup = skin.cleanup(skin_mask)
            
            cleaned_skin = cv2.bitwise_and(frame1, frame1, mask=cleanup)
            
            ees = cv2.getTickCount()
            print('{:04d}'.format(count - framesToAdvance),
                  'skin calc {:.2f} seconds'.format(\
                             (ees - eis)/ cv2.getTickFrequency()))
        
            
            # save frame
            out_skin.write(cleaned_skin)
            out_sampled.write(frame1)
            
            # update frame1
            frame1 = frame2
            frame1_gray = frame2_gray
            
            
            
            ee = cv2.getTickCount()
            print('{:04d}'.format(count - framesToAdvance),
                  'processing calc {:.2f} seconds'.format(\
                                   (ee - eb)/ cv2.getTickFrequency()))
    
        else:
    
            # release capture
            cap.release()
            out_skin.release()
            out_sampled.release()
    
    e2 = cv2.getTickCount()
    time = (e2 - e1)/ cv2.getTickFrequency()
    print("Execution time in seconds = ", time)