# -*- coding: utf-8 -*-
"""
Created on Sat Apr 14 14:19:28 2018

@author: Callie Darsey

Break into patches.

patches_01
"""

import numpy as np

def print_C1_mat(a):
    
    for line in a:
        s = ''
        for el in line:
            s = s + '{:3d}'.format(el) + ', '
        print(s)
        
def get_valid_blocks_as_feature_vectors(frame, mask, block_dim):
    '''
    Returns a (N, F) numpy array where N is the number of valid blocks found
    and F = 3 * block_dim * block_dim. Blocks are reckoned starting from the
    upper-left corner of the mask and have size (block_dim, block_dim). A
    valid block is where all elements of the mask have are non-zero.
    '''

    assert len(frame.shape) == 3
    assert frame.shape[2] == 3    
    assert frame.shape[0:2] == mask.shape[0:2]
    
    orig_rows, orig_cols = mask.shape

    mask_trimmed = mask[0 : int( orig_rows / block_dim) * block_dim,
                        0 : int( orig_cols / block_dim) * block_dim]    

    mask_collect = []
    features_collect = []
    
    for r in np.arange(0, mask_trimmed.shape[0], block_dim):
        
        for c in np.arange(0, mask_trimmed.shape[1], block_dim):
            
            # mask
            mask_block_as_row = \
                mask_trimmed[r:r+block_dim, c:c+block_dim].reshape((1, -1))
            
            mask_collect.append(mask_block_as_row)
            
            # frame
            C0_block_as_row = \
                frame[r:r+block_dim, c:c+block_dim, 0].reshape((1, -1))
            C1_block_as_row = \
                frame[r:r+block_dim, c:c+block_dim, 1].reshape((1, -1))
            C2_block_as_row = \
                frame[r:r+block_dim, c:c+block_dim, 2].reshape((1, -1))
                
            block_features_vector = np.hstack([C0_block_as_row,
                                               C1_block_as_row,
                                               C2_block_as_row])
        
            features_collect.append(block_features_vector)
    
    # stack the mask block rows
    mask_blocks_as_rows = np.vstack(mask_collect)

    # find the valid blocks    
    block_nonzero_counts = np.count_nonzero(mask_blocks_as_rows, axis=1)    
    valid_block_indices = \
        np.where(block_nonzero_counts == block_dim * block_dim)[0]
    
    # get the valid features to return
    feature_vectors = np.vstack(features_collect)
    valid_feature_vectors = feature_vectors[valid_block_indices, :]
    
    return valid_feature_vectors

def build_mask_from_patch_predictions(frame, predicted_label, block_dim):
    '''
    '''
    
    dest_rows, dest_cols, channels = frame.shape

    trimmed_rows, trimmed_cols = (int( dest_rows / block_dim) * block_dim,
                                  int( dest_cols / block_dim) * block_dim)
    
    count = -1
    
    mask = np.zeros((dest_rows, dest_cols), dtype=np.uint8)
    
    for r in np.arange(0, trimmed_rows, block_dim):
        
        for c in np.arange(0, trimmed_cols, block_dim):
            
            count = count + 1
            
            if predicted_label[count] == 1.0:
                
                mask[r : r + block_dim, c : c + block_dim] = \
                    255 * np.ones((block_dim, block_dim), dtype=np.uint8)
                    
    return mask
    
if __name__ == '__main__':    

    a = np.random.randint(0, 256, (5, 9, 3), dtype=np.uint8)
    print('channel 0')
    print_C1_mat(a[:, :, 0])
    print('\nchannel 1')
    print_C1_mat(a[:, :, 1])
    print('\nchannel 2')
    print_C1_mat(a[:, :, 2])
    
    m = np.array([[  0,   1,   1,   1,   1,   1,   1,   1,   1], 
                  [  1,   1,   1,   1,   1,   0,   1,   1,   1], 
                  [  1,   1,   0,   1,   1,   1,   0,   1,   1], 
                  [  1,   1,   0,   0,   1,   1,   1,   1,   1], 
                  [  1,   1,   1,   1,   1,   1,   1,   1,   1]])
            
    feature_vectors = get_valid_blocks_as_feature_vectors(a, m, 2)
    print('\nfeature vectors')
    print_C1_mat(feature_vectors)

