# -*- coding: utf-8 -*-
"""
Created on Sat Apr 14 23:07:03 2018

@author: Callie Darsey

Display the classified and misclassied
"""


import pickle
import os
import numpy as np
import sklearn.neighbors as skn
import sklearn.linear_model as sklm
import sklearn.model_selection as skm
import sklearn.metrics as skmet
import matplotlib.pyplot as plt
import cv2


def make_image(feature_vectors, block_dim):
    
    rows, cols = feature_vectors.shape
    
    sqrt_dim = np.int(np.sqrt(rows))
    num_rem_blocks = rows - sqrt_dim * sqrt_dim
    fitting = int(num_rem_blocks / sqrt_dim)
    remaining = np.mod(num_rem_blocks, sqrt_dim)
    add = sqrt_dim - remaining
    
    if remaining > 0:
        
        z = np.zeros((int(add), cols), dtype=feature_vectors.dtype)
        
        feature_vectors = np.vstack([feature_vectors, z])
        
        display_rows = sqrt_dim + fitting + 1
            
    else:
        
        display_rows = sqrt_dim + fitting
        
    display_image = np.zeros((display_rows * block_dim,
                              sqrt_dim * block_dim,
                              int(cols / (block_dim ** 2))),
                                dtype=feature_vectors.dtype)
    
    count = -1
    
    for r in np.arange(0, display_rows * block_dim, block_dim):
        
        for c in np.arange(0, sqrt_dim * block_dim, block_dim):
            
            count = count + 1
            
            feature_vector = feature_vectors[count, :]
            
            area = block_dim ** 2

            C0 = feature_vector[0 * area : 0 * area + area]
            C1 = feature_vector[1 * area : 1 * area + area]
            C2 = feature_vector[2 * area : 2 * area + area]
            
            C0 = np.reshape(C0, (block_dim, block_dim))
            C1 = np.reshape(C1, (block_dim, block_dim))
            C2 = np.reshape(C2, (block_dim, block_dim))
            
            C = np.stack([C0, C1, C2], axis=2)
            
            display_image[r : r + block_dim, c : c + block_dim, :] = C
    
    return display_image
    
    


path = './video_copies_clips_track/ROI_masks_patch_detection/'
save = path + "block_feature_vectors_non_skin_misclassified_patches/"

#roi_path = './video_copies_clips_track/ROI_masks_patch_detection/'
#filenames = [f for f in os.listdir(roi_path) if os.path.isfile(roi_path + f)]
'''second training where non-skin patches were misclassified as skin by
initial training'''
filenames = [\
'STI Polk Apr13 E-MVI_0044_trans__350_365_0416.data',
'STI Polk Apr20 C-MVI_0053_trans__50_72_0080.data',
#'STI Polk Mar30 F-MVI_0025_trans__250_280_0388.data',
'STI Polk May04 F-MVI_0079_trans__390_415_0148.data',
#'STI WA Apr11 A-MVI_0023_trans__100_200_2132.data',
'STI WA Apr18 B-MVI_0029_trans__75_125_0212.data',
#'STI WA Apr25 A-STI WA Apr25 AMVI_0068_trans__860_910_1772.data',
'STI WA Mar28 C-MVI_0013_trans__110_140_0768.data',
#'STI WA May06 C-MVI_0095_trans__0_100_0540.data',
'STI WA May06 Polk B-MVI_0091_trans__60_85_0480.data',
#'STI WA May06 Polk C-MVI_0087_trans__600_700_1224.data',
'STI WA May09 A-MVI_0110_trans__100_200_2384.data',
#'STI WA May09 B-MVI_0006_trans__600_700_0496.data',
]

print('Number of frames: {}\n'.format(len(filenames)))

block_dim = 5
print('block dimension ({:d}, {:d})'.format(block_dim, block_dim))

hand_feature_vectors_collect = []
non_skin_feature_vectors_collect = []

for filename in filenames:
    
    print(filename)

    with open(save+filename[0:-5] + \
              '_block_dim_{:02d}.data'.format(block_dim), 'rb') as f:
        
        hand_feature_vectors, non_skin_feature_vectors = pickle.load(f)
    
    # I assume there are more non-skin than hand, so verify
    assert hand_feature_vectors.shape[0] < non_skin_feature_vectors.shape[0]

    # shuffle, then take a subset of non-skin
    np.random.shuffle(non_skin_feature_vectors)
    
    non_skin_feature_vectors_take = \
        non_skin_feature_vectors[0:hand_feature_vectors.shape[0], :]
    
    # save for further stacking
    hand_feature_vectors_collect.append(hand_feature_vectors)
    non_skin_feature_vectors_collect.append(non_skin_feature_vectors_take)
    
feature_vectors_hand = np.vstack(hand_feature_vectors_collect)
feature_vectors_non_skin = np.vstack(non_skin_feature_vectors_collect)

# shuffle and split into a training set and a testing set
sind = np.arange(0, feature_vectors_hand.shape[0])
np.random.shuffle(sind)
feature_vectors_hand = feature_vectors_hand[sind, :]

sind = np.arange(0, feature_vectors_hand.shape[0])
np.random.shuffle(sind)
feature_vectors_non_skin = feature_vectors_non_skin[sind, :]

hand_train = feature_vectors_hand[0:int(feature_vectors_hand.shape[0] / 2), :]
hand_val = feature_vectors_hand[int(feature_vectors_hand.shape[0] / 2):, :]

non_skin_train = \
    feature_vectors_non_skin[0:int(feature_vectors_non_skin.shape[0] / 2), :]
non_skin_val = \
    feature_vectors_non_skin[int(feature_vectors_non_skin.shape[0] / 2):, :]
    
# get the training labels
y_train = np.hstack([np.ones(hand_train.shape[0]),
                     np.zeros(non_skin_train.shape[0])])

# feature vectors
X_train = np.vstack([hand_train, non_skin_train])

# shuffle
indices_shuffled = np.arange(0, y_train.shape[0])
np.random.shuffle(indices_shuffled)

y_train = y_train[indices_shuffled]
X_train = X_train[indices_shuffled, :]
print('X_train.shape', X_train.shape)


#clf_knn = skn.KNeighborsClassifier(n_neighbors=5)
#clf_knn.fit(X_train, y_train)

model = sklm.LogisticRegression()
model.fit(X_train, y_train)

# scores
predict_label = model.predict(np.vstack([hand_val, non_skin_val]))
y_label = np.hstack([np.ones(hand_val.shape[0]),
                     np.zeros(non_skin_val.shape[0])])
print('non-skin')
print('accuracy:', skmet.accuracy_score(y_label, predict_label))
print('confusion matrix:\n',
      skmet.confusion_matrix(y_label, predict_label))
print('precision:', skmet.precision_score(y_label, predict_label))
print('recall:', skmet.recall_score(y_label, predict_label))
print('F1:', skmet.f1_score(y_label, predict_label))

# hand
predict_hand_val = model.predict(hand_val)

good = np.where(predict_hand_val == 1)[0]
bad = np.where(predict_hand_val == 0)[0]
hand_predicted_hand = make_image(hand_val[good, :], block_dim)
hand_predicted_non_skin = make_image(hand_val[bad, :], block_dim)

cv2.imshow('hand predicted hand', hand_predicted_hand)
cv2.imshow('hand predicted non-skin', hand_predicted_non_skin)

# non-skin
predict_non_skin_val = model.predict(non_skin_val)

good = np.where(predict_non_skin_val == 0)[0]
bad = np.where(predict_non_skin_val == 1)[0]
non_skin_predicted_non_skin = make_image(non_skin_val[good, :], block_dim)
non_skin_predicted_hand = make_image(non_skin_val[bad, :], block_dim)

cv2.imshow('non-skin predicted non-skin', non_skin_predicted_non_skin)
cv2.imshow('non_skin_predicted_hand', hand_predicted_non_skin)

cv2.waitKey(0)
cv2.destroyAllWindows()