# -*- coding: utf-8 -*-
"""
Created on Tue Feb  6 11:42:04 2018

@author: Callie Darsey (cj.darsey@gmail.com)

I want to pickle the resized color frame and the optical flow for the subset
of frames I've specified in lists.
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import os
import overlay_01 as overlay
import pickle

# info for the subset of frames
paths_specific = ['STI WA Apr25 A/STI WA Apr25 AMVI_0068_trans/',
                  'STI Polk Mar02 E/00005_trans/',
                  'STI Polk Apr13 E/MVI_0044_trans/',
                  'STI Polk Mar02 B/00006_trans/',
                  'STI WA Mar28 C/MVI_0013_trans/',
                  'STI WA May06 Polk B/MVI_0091_trans/',
                  'STI Polk Feb16 C/00054_trans/',
                  'STI Polk May04 F/MVI_0079_trans/',
                  'STI WA Apr18 D/MVI_0056_trans/',
                  'STI WA Feb28 xtraB/00011_trans/',
                  'STI Polk Apr20 C/MVI_0053_trans/',
                  'STI WA Apr18 B/MVI_0029_trans/',
                  'STI WA Feb28 C/00089_trans/']
video_names = ['STI WA Apr25 AMVI_0068_trans__860_910.mp4',
               '00005_trans__605_628.mp4',
               'MVI_0044_trans__350_365.mp4',
               '00006_trans__567_600.mp4',
               'MVI_0013_trans__110_140.mp4',
               'MVI_0091_trans__60_85.mp4',
               '00054_trans__452_505.mp4',
               'MVI_0079_trans__390_415.mp4',
               'MVI_0056_trans__205_240.mp4',
               '00011_trans__290_335.mp4',
               'MVI_0053_trans__50_72.mp4',
               'MVI_0029_trans__75_125.mp4',
               '00089_trans__110_165.mp4']
frames_target = [[ 260,  748, 1200, 1480, 1648, 1660, 1780],
                 [  78,  164,  190,  202,  206,  322,  406,  442,  508,  554],
                 [  92,  248,  344,  404,  480,  544,  784],
                 [  84,  170,  316,  408,  500,  792,  870],
                 [   4,  272,  316,  620,  704,  940, 1140, 1260],
                 [  40,  212,  396,  480,  620,  792,  880],
                 [ 122,  360,  478,  632,  876,  906,  938, 1144, 1164, 1196],
                 [   0,  144,  688,  964],
                 [  76,  126,  814],
                 [ 134,  190,  202,  770,  778,  908, 1110],
                 [  76,  736,  800,  860, 1132, 1264],
                 [ 172,  324,  472,  596, 1136, 1720, 2000, 2128, 2396, 2576],
                 [  16,   22,  462,  478,  560,  712,  776,  784,  836, 1066, 1238]]

# common part of path
path_common = './video_copies_clips_track/copies_and_clips/'

# settings not per video clip

# Settings
'''Resolution rescaling factor'''
scaleFactor = 0.7
'''normalization of optical flow by resolution scaling factor'''
normalize_scale = True
''' normalization of optical flow by number of frames advanced'''
normalize_frame_advance = False

'''Optical flow parameters for cv2.calcOpticalFlowFarneback
docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''
pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0

# begin counting time
e1 = cv2.getTickCount()

# loop though the clips
for path_specific, video_name, frame_list in zip(paths_specific, video_names,
                                                 frames_target):
    # clip path
    video_path = path_common + path_specific + video_name
    
    # ensure sorted
    frame_list = list(np.sort(frame_list))
    
    # get the first target frame number
    if len(frame_list) > 0:
        target_frame = frame_list.pop(0)
             
    # make the VideoCapture object
    cap = cv2.VideoCapture(video_path)
    
    num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    fps = cap.get(cv2.CAP_PROP_FPS )
    
    '''Frames to advance between frames passed to optical flow'''
    framesToAdvance = np.int32(np.ceil(fps / 15))
    
    print('number of frames:', num_frames, ', frames per second:', fps,
          'framesToAdvance:', framesToAdvance)

    # initialize frame1
    success_flag, frame1_full = cap.read()
    
    assert success_flag, "On first attempt after construction, could \
                          not read frame from {}".format(video_path)  
                                
    frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
    frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)

    # count frames
    count = 0

    # loop through the frames
    while(cap.isOpened()):
                    
        # skip frames
        for i in range(framesToAdvance):
            success_flag, frame2_full = cap.read()
            
            count += 1
    
            if (not success_flag):
                print("no more frames in this capture")
                break
         
        if success_flag:
        # process upon true read frame success_flag
    
            #initialize frame2
            frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
            frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)
    
            if count - framesToAdvance == target_frame:
                
                # calculate the flow between frame1 and frame2    
                flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                                    flow=None,
                                                    pyr_scale=pyr_scale, levels=levels,
                                                    winsize=winsize,
                                                    iterations=iterations,
                                                    poly_n=poly_n,
                                                    poly_sigma=poly_sigma, flags=flags)
                
                # pickle
                pickle_path = video_path[0:-4] + '__' + str(target_frame) + '.data'
                print(pickle_path)
                
                with open(pickle_path, 'wb') as f:
                    pickle.dump((frame1, flow), f)
                
                
                # update target_frame            
                if len(frame_list) > 0:
                    
                    target_frame = frame_list.pop(0)
                    
                else:
                    
                    # release the capture
                    cap.release()
                    
                    break
            
            # update frames
            frame1 = frame2
            frame1_gray = frame2_gray
            
    else:
        
        # release the capture
        cap.release()

# finish counting time
e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)    
    
    
    