# -*- coding: utf-8 -*-
"""
Created on Tue Feb  6 11:42:04 2018

@author: Callie Darsey (cj.darsey@gmail.com)

I want to pickle the resized color frame and the optical flow for the subset
of frames I've specified in lists.
"""

import numpy as np
import cv2
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import os
import overlay_01 as overlay
import pickle

# info frames
paths_specific = ['STI WA Feb28 C/00089_trans/']
video_names = ['00089_trans__110_165.mp4']
exemplars_file_names = \
['exemplars___00089_trans__110_165___y2018m03d19-h06m43s51.214354__for_now.data']

# common part of path
path_common = './video_copies_clips_track/copies_and_clips/'

# settings not per video clip

# Settings
'''Resolution rescaling factor'''
scaleFactor = 0.67
'''normalization of optical flow by resolution scaling factor'''
normalize_scale = True
''' normalization of optical flow by number of frames advanced'''
normalize_frame_advance = False

'''Optical flow parameters for cv2.calcOpticalFlowFarneback
docs.opencv.org/2.4/modules/video/doc/motion_analysis_and_object_tracking.html#calcopticalflowfarneback
'''
pyr_scale = 0.5 
levels = 3
winsize = 15
iterations = 3
poly_n = 5
poly_sigma = 1.2
flags = 0

# begin counting time
e1 = cv2.getTickCount()

# loop though the clips
for path_specific, video_name, exemplars_file_name in zip(paths_specific,
                                                          video_names,
                                                          exemplars_file_names):
    # load the pickled exemplars info
    pickle_path_load = path_common + path_specific + exemplars_file_name
                  
    with open(pickle_path_load, 'rb') as f:
        exemplar_info = pickle.load(f)
    
    
    # get the frame_list
    frame_list = []
    for frame_number, boxes in exemplar_info.items():
        
        if boxes != None:
            frame_list.append(int(frame_number))

    # ensure sorted
    frame_list = list(np.sort(frame_list))
    
    # clip path
    video_path = path_common + path_specific + video_name

    # get the first target frame number
    if len(frame_list) > 0:
        target_frame = frame_list.pop(0)
             
    # make the VideoCapture object
    cap = cv2.VideoCapture(video_path)
    
    num_frames = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    fps = cap.get(cv2.CAP_PROP_FPS )
    
    '''Frames to advance between frames passed to optical flow'''
    framesToAdvance = np.int32(np.ceil(fps / 15))
    
    print('number of frames:', num_frames, ', frames per second:', fps,
          'framesToAdvance:', framesToAdvance)

    # initialize frame1
    success_flag, frame1_full = cap.read()
    
    assert success_flag, "On first attempt after construction, could \
                          not read frame from {}".format(video_path)  
                                
    frame1 = cv2.resize(frame1_full, (0,0), fx=scaleFactor, fy=scaleFactor) 
    frame1_gray = cv2.cvtColor(frame1,cv2.COLOR_BGR2GRAY)

    # count frames
    count = 0

    # loop through the frames
    while(cap.isOpened()):
                    
        # skip frames
        for i in range(framesToAdvance):
            success_flag, frame2_full = cap.read()
            
            count += 1
    
            if (not success_flag):
                print("no more frames in this capture")
                break
         
        if success_flag:
        # process upon true read frame success_flag
    
            #initialize frame2
            frame2 = cv2.resize(frame2_full, (0,0), fx=scaleFactor, fy=scaleFactor)             
            frame2_gray = cv2.cvtColor(frame2, cv2.COLOR_BGR2GRAY)
    
            if count - framesToAdvance == target_frame:
                
                # calculate the flow between frame1 and frame2    
                flow = cv2.calcOpticalFlowFarneback(frame1_gray, frame2_gray,
                                                    flow=None,
                                                    pyr_scale=pyr_scale, levels=levels,
                                                    winsize=winsize,
                                                    iterations=iterations,
                                                    poly_n=poly_n,
                                                    poly_sigma=poly_sigma, flags=flags)
                
                # pickle
                pickle_path = path_common + path_specific + \
                              'frame_flow_exemplars__' + video_name[0:-4] + \
                              '__' + str(target_frame) + '.data'
                print(pickle_path)
                
                with open(pickle_path, 'wb') as f:
                    pickle.dump((frame1, flow,
                                 exemplar_info[str(target_frame)]), f)
                
                
                # update target_frame            
                if len(frame_list) > 0:
                    
                    target_frame = frame_list.pop(0)
                    
                else:
                    
                    # release the capture
                    cap.release()
                    
                    break
            
            # update frames
            frame1 = frame2
            frame1_gray = frame2_gray
            
        else:
            
            # release the capture
            cap.release()

# finish counting time
e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)    
    
    
    