# -*- coding: utf-8 -*-
"""
Created on Wed Mar 28 12:11:21 2018

@author: Callie Darsey


Arg. Look into trigger and predict hand regions
"""

import cv2
import numpy as np
import pickle
import os
import fns_investigating_optical_flow_01 as fns
from matplotlib import pyplot as plt
import get_lessened_features_and_save_steps_01 as get
import overlay_01 as overlay
import shutil
import component_functions_01 as comp_fns

e1 = cv2.getTickCount()

# video path
common_path = './video_copies_clips_track/copies_and_clips_facilitator_C/'
specific_path = 'STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/'
video_name = 'MVI_0028_trans__200_208.mp4'


directory = common_path + specific_path

dir_list = os.listdir(directory)

collect_listings = []

for listing in dir_list:
    
    if 'flow_' + video_name[0:-4] in listing and listing[-5:] == '.data':
        
        collect_listings.append(listing)

collect_listings.sort()

# prep window
title = 'display window'
move_window = [60, -30]
cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
cv2.moveWindow(title, move_window[0], move_window[1])

with open(common_path + specific_path[0:32] + \
          'flow_mag_samples.data', 'rb') as f:
    
    flow_mag_samples, specific_paths, video_names, exemplar_names = \
        pickle.load(f)


bp1 = plt.boxplot(\
    np.reshape(np.sort(flow_mag_samples), -1)[int(len(flow_mag_samples) / 2):])
plt.show()


Q1 = np.min(bp1['boxes'][0].get_data()[1])
Q3 = np.max(bp1['boxes'][0].get_data()[1])
median = bp1['medians'][0].get_data()[1][0]
Q3_plus_1pt5_IQR = np.max(bp1['caps'][1].get_data()[1]) # IQR = Q3 - Q1


# make the review folder
predict_review_02 = common_path + specific_path + "trigger_and_3_way_overlap/"
if not os.path.isdir(predict_review_02):
    os.mkdir(predict_review_02)
    
# make the pickle folder
pickle_folder = predict_review_02 + "pickle_folder/"
if not os.path.isdir(pickle_folder):
    os.mkdir(pickle_folder)

# load the classification models
with open('./rfc_models/from_wheeler/' + \
          'way_1_gs_rf_clf_64y2018m03d26-h15m12s08.211352.data', \
          'rb') as f:
    way_1_model = pickle.load(f)    

with open('./rfc_models/from_wheeler/' + \
          'way_2_gs_rf_clf_64y2018m03d27-h00m19s13.508701.data', \
          'rb') as f:
    way_2_model = pickle.load(f)   
    
with open('./rfc_models/from_wheeler/' + \
          'way_3_gs_rf_clf_64y2018m03d26-h18m14s15.655086.data', \
          'rb') as f:
    way_3_model = pickle.load(f)   

'''flow_MVI_0028_trans__200_208_0000_0004.data'''


save_trigger_frames = []

for listing in collect_listings:
    
    eib = cv2.getTickCount()
    
    nframe1 = listing[-14:-10]
    
    # pickle result
    pickle_path = common_path + specific_path + listing
        
    with open(pickle_path, 'rb') as f:
        frame1, frame2, flow, display = pickle.load(f)
        
    flow_mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
    flow_mag_trigger_mask = np.logical_and(flow_mag > Q1, flow_mag < Q3)
    
    trigger_arrows = np.copy(frame1)
    fns.draw_flow(trigger_arrows, flow, flow_mag_trigger_mask, step=4)
    
    # draw the text
    cv2.putText(trigger_arrows, str(int(nframe1)),
                (trigger_arrows.shape[1] - 100, 25),
                cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)    

    cv2.imshow(title, trigger_arrows)
    
    # escape key
    if cv2.waitKey(1) == 27:
        
        break

    cv2.imwrite(predict_review_02 + 'flow_trigger_arrows_' + nframe1 + '.png', \
                trigger_arrows)

    if np.count_nonzero(flow_mag_trigger_mask) > 0:
        
        save_trigger_frames.append(nframe1)
        
        eie = cv2.getTickCount()
        print(nframe1,
              'load flow, threshold, draw flow, display time (seconds)',
              (eie - eib)/ cv2.getTickFrequency(), 'triggered')
        
#        intermediate_tag, intermediate_arr = \
#            get.get_components_and_intermediate_steps(frame1, flow, Q1)
#            
#        filename_skin_detection, \
#        filename_flow_mask, \
#        filename_flow_mag_thresh, \
#        filename_skin_and_flow_mag_thresh, \
#        filename_DT_skin_and_flow_mag_thresh, \
#        filename_DT_skin_and_flow_mag_thresh_AND_flow_mag_thresh, \
#        filename_way_1_components_mask, \
#        filename_DT_from_way_1_components_mask, \
#        filename_less_small_connectors, \
#        filename_per_cc_top_percent_mask, \
#        filename_local_max_mask, \
#        filename_markers, \
#        filename_watershed_lines, \
#        filename_way_2_components_mask, \
#        filename_watershed_components, \
#        filename_way_3_components_mask =  intermediate_tag
#            
#        skin_detection, \
#        flow_mask, \
#        flow_mag_thresh, \
#        skin_and_flow_mag_thresh, \
#        DT_skin_and_flow_mag_thresh, \
#        DT_skin_and_flow_mag_thresh_AND_flow_mag_thresh, \
#        way_1_components_mask, \
#        DT_from_way_1_components_mask, \
#        less_small_connectors, \
#        per_cc_top_percent_mask, \
#        local_max_mask, \
#        markers, \
#        watershed_lines, \
#        way_2_components_mask, \
#        watershed_components, \
#        way_3_components_mask = intermediate_arr
#
#        # get all the arrows
#        tmp = np.copy(frame1)
#        fns.draw_flow(tmp, flow, flow_mag != 0, step=4)

        # way components
        connectivity = 4
        
#        cc_way_1 = cv2.connectedComponentsWithStats(way_1_components_mask,
#                                                    connectivity,
#                                                    cv2.CV_32S)
#        
#        cc_way_2 = cv2.connectedComponentsWithStats(way_2_components_mask,
#                                                    connectivity,
#                                                    cv2.CV_32S)
#
#        cc_way_3 = cv2.connectedComponentsWithStats(way_3_components_mask,
#                                                    connectivity,
#                                                    cv2.CV_32S)       
#        
#        # classify with the models
#        cc_way_1 = get.get_features(cc_way_1, frame1, flow)
#        cc_way_2 = get.get_features(cc_way_2, frame1, flow)
#        cc_way_3 = get.get_features(cc_way_3, frame1, flow)
#                
#        cc_way_1 = get.classify(cc_way_1, way_1_model)
#        cc_way_2 = get.classify(cc_way_2, way_2_model)
#        cc_way_3 = get.classify(cc_way_3, way_3_model)
#        
#        # save the images for the classified
#        keep_classified = []
#        for tag, cc in zip(['way_1', 'way_2', 'way_3'],
#                           [cc_way_1, cc_way_2, cc_way_3]):
#            
#            toss_labels = np.unique(cc[1])[np.where(cc[5] == 0)[0]]
#            keep_labels = np.unique(cc[1])[np.where(cc[5] == 1)[0]]
#                
#            prediction_overlay = overlay.overlay(frame1, cc, toss_labels,
#                                                 (0, 0, 255), alpha=0.5)
#            
#            prediction_overlay, overlay_mask = get.overlay(prediction_overlay,
#                                                           cc, keep_labels,
#                                                           (0, 255, 0),
#                                                           alpha=0.5)
#            keep_classified.append(overlay_mask)
#
#            cv2.imwrite(predict_review_02 + tag + '_class_prediction' + '_' + \
#                        nframe1 +'.png', prediction_overlay)
#        
#        keep_sum = np.sum(keep_classified, axis=0)
#        keep_sum_adjusted_for_viewing = \
#            ((keep_sum + 1) ** 4 - 1).astype(np.uint8)
#        
        cc_trigger = cv2.connectedComponents(\
             flow_mag_trigger_mask.astype(np.uint8), connectivity, cv2.CV_32S)

        # pickle None as the non-existing trigger mask
        with open(pickle_folder + 'cc_trigger_' + nframe1 + '.data', \
                  'wb') as f:
            
            pickle.dump((cc_trigger), f)   

#        overlap_trigger_keep_ways = overlay.overlay(\
#                                                keep_sum_adjusted_for_viewing,
#                                                cc_trigger,
#                                                np.unique(cc_trigger[-1])[1:],
#                                                (127, 0, 255), alpha=0.5)
#
#        cv2.imwrite(predict_review_02 + 'overlap_trigger_keep_ways' + '_' + \
#                    nframe1 +'.png', overlap_trigger_keep_ways)
#        
#        cc_summed_ways_3 = \
#            cv2.connectedComponents((keep_sum[:, :, 0] == 3).astype(np.uint8),
#                                    connectivity, cv2.CV_32S)
#        
#        regions = [np.zeros(cc_summed_ways_3[1].shape)]
#        for label_3 in np.unique(cc_summed_ways_3[1])[1:]:
#            
#            for label_trigger in np.unique(cc_trigger[1])[1:]:
#                
#                matrix_summed_ways_3 = (cc_summed_ways_3[1] == label_3)
#                matrix_trigger = (cc_trigger[1] == label_trigger)
#                
#                if np.count_nonzero(np.all([matrix_summed_ways_3,
#                                            matrix_trigger], axis=0)):
#                    
#                    regions.append(matrix_summed_ways_3)
#                    regions.append(matrix_trigger)
#        
#        regions_mask = np.any(regions, axis=0)
#          
#        if np.count_nonzero(regions_mask) > 0:
#            
#            regions_mask = comp_fns.fill_holes(regions_mask.astype(np.uint8))
#            
#            cc_regions = cv2.connectedComponents(regions_mask.astype(np.uint8),
#                                                 connectivity, cv2.CV_32S)            
#            cc_regions = list(cc_regions)
#            cc_regions.append([])
#
#            regions_regard_flow_holes_filled = \
#                overlay.overlay(frame1, cc_regions,
#                                np.unique(cc_regions[1])[1:], (178, 0, 255),
#                                alpha=0.5)
#            
#            cv2.imwrite(predict_review_02 + \
#                        'regions_regard_flow_holes_filled' + '_' + \
#                        nframe1 +'.png', regions_regard_flow_holes_filled)
#            
#            # pickle the regions mask cc
#            with open(pickle_folder + 'cc_regions_' + nframe1 + '.data', \
#                      'wb') as f:
#                
#                pickle.dump((cc_regions), f)
#        else:
#            # pickle None as the non-existing regions mask
#            with open(pickle_folder + 'cc_regions_' + nframe1 + '.data', \
#                      'wb') as f:
#                
#                pickle.dump((None), f)            
        
#        cv2.imwrite(predict_review_02 + filename_flow_mask + '_arrows_' + \
#            nframe1 + '.png', tmp)
                     
        eib = cv2.getTickCount()
        print(nframe1,
              'calculating features, classifying, ' + \
              'and saving images time (seconds)',
              (eib - eie)/ cv2.getTickFrequency(), 'triggered')
    else:

        # pickle None as the non-existing trigger mask
        with open(pickle_folder + 'cc_trigger_' + nframe1 + '.data', \
                  'wb') as f:
            
            pickle.dump((None), f)   
        
#        # pickle None as the non-existing regions mask
#        with open(pickle_folder + 'cc_regions_' + nframe1 + '.data', \
#                  'wb') as f:
#            
#            pickle.dump((None), f)   
        
        eie = cv2.getTickCount()
        print(nframe1,
              'load flow, threshold, draw flow, display time (seconds)',
              (eie - eib)/ cv2.getTickFrequency(), 'triggered')
    
cv2.destroyAllWindows()

# pickle None as the non-existing regions mask
with open(pickle_folder + 'Q1_Q3_triggered_frame_strings.data', 'wb') as f:
    
    pickle.dump((Q1, Q3, save_trigger_frames), f)   

e2 = cv2.getTickCount()
time = (e2 - e1)/ cv2.getTickFrequency()
print("Execution time in seconds = ", time)