# -*- coding: utf-8 -*-
"""
Created on Fri Apr 13 10:32:57 2018

@author: Callie Darsey

Slightly different implementation of 

C:\\Users\\Callie\\ECE_633_Adv_Topics_in_Image_Processing\\Assignment_01\\
my_solution_02_turn_in\\select_roi_and_pickle_tool.py

"""

import cv2
import numpy as np
import roipoly
from matplotlib import pyplot as plt
import argparse
import pickle

parser = argparse.ArgumentParser(prog='Build Videos')
subparsers = parser.add_subparsers(help='sub-command help')

subparser_obj_imgs = subparsers.add_parser('images_start',
                                           help='starting with list of images')
subparser_obj_imgs.add_argument('--obj_imgs', nargs='+', type=str, 
                                help="object image paths",
                                metavar='pth1 pth2 ... pth3')
subparser_obj_imgs.add_argument('--old_pickled', type=str, 
                            help="path to pickled information we wish to load")
subparser_obj_imgs.add_argument('--pickled_out', type=str, 
                                help="filename for saving object image info")

subparser_pickled = subparsers.add_parser('pickled_start',
                            help='starting with pickled image and object data')
subparser_pickled.add_argument('--pickled_in', type=str, 
                               help="pickled data filepath")

args = parser.parse_args()
 
            
if ('pickled_in' in args) and args.pickled_in != None:
    '''Load pickled data.'''
    with open(args.pickled_in, 'rb') as f:
        obj_data = pickle.load(f)
else:
    '''Cut out regions of interest and pickle the result.'''
    if ('obj_imgs' in args) and args.obj_imgs != None:
        obj_imgs_list = args.obj_imgs
    else:
        '''Start with default object images.'''
        obj_imgs_list = ['imgid185161.jpg', 'Callie.jpg',
                         'DSC00453.JPG', 'DSC00443.JPG']
    
    if ('old_pickled' in args) and args.old_pickled is not None:
        
        with open(args.old_pickled, 'rb') as f:
            
            obj_data, bg_bgr = pickle.load(f)
            
    else:
        
        obj_data = {}
        bg_bgr = None

    for img in obj_imgs_list:
        while True:
            if bg_bgr is None:
                try:
                    print(img)
                    bg_bgr = cv2.imread(img)
                    print(bg_bgr is None)
                except:
                    print("Couldn't read {}".format(img))
                    break
                else:
                    print("Could read {}".format(img))

            bg_rgb = cv2.cvtColor(bg_bgr, cv2.COLOR_BGR2RGB)
            bg_gray = cv2.cvtColor(bg_bgr, cv2.COLOR_BGR2GRAY)
            
            plt.imshow(bg_rgb)
            plt.title("Click to place boundary points; right-click to end")
            roi = roipoly.roipoly()
            
            plt.imshow(bg_rgb)
            plt.title("Upon closing this viewing window, check the command\n \
                      line to answer whether you want to redo or continue.")
            roi.displayROI()
            plt.show()
            while True:
                response = input("Enter 'r' to redo boundary points.\n \
                                 Enter 'c' to continue.\n \
                                 ==> ")
                if response == 'r' or response == 'c':
                    break
            if response == 'c':
                mask = roi.getMask(bg_gray)
                plt.imshow(mask)
                plt.show()

                while True:
                    name = input("Enter name for object ==> ")
                    name = name + \
                        '{:010d}'.format(np.random.randint(0, bg_bgr.size))
                    print('Name:', name)
                    if name in obj_data.keys():
                        print("Name taken! Try again.")
                    else:
                        break
                obj_data[name] = mask                  
                break
    if ('pickled_out' in args) and args.pickled_out != None:                    
        with open(args.pickled_out, 'wb') as f:
            pickle.dump((obj_data, bg_bgr), f, pickle.HIGHEST_PROTOCOL)
    else:
        pickled_out_default = 'obj_data.data'
        with open(pickled_out_default, 'wb') as f:
            pickle.dump((obj_data, bg_bgr), f, pickle.HIGHEST_PROTOCOL) 

obj_img = bg_rgb
obj = np.zeros(obj_img.shape, dtype=np.uint8)

for obj_name in obj_data.keys():
    
    obj_mask = obj_data[obj_name].astype(np.uint8)
    
    
    msk = np.stack([obj_mask, obj_mask, obj_mask], axis=2)

    np.copyto(obj, obj_img, where=msk.astype(bool))
    
    cc_obj = cv2.connectedComponentsWithStats(obj_mask)
    
    # draw the text
    cv2.putText(obj, obj_name, (int(cc_obj[-1][1, 0]), int(cc_obj[-1][1, 1])),
                cv2.FONT_HERSHEY_SIMPLEX, 0.5, (0, 255, 0), 1)    
 


    
plt.imshow(obj)
plt.show()