# -*- coding: utf-8 -*-
"""
Created on Thu Feb  8 12:22:16 2018

@author: Callie Darsey

I am not sure what I am looking into here. I want to see what I can get with
the distance transform on the motion maginitude components. I want to 
enlarge components found with fancy anding.
"""


import pickle
import numpy as np
import cv2
import skin_detector_01 as skin_detector
from matplotlib import pyplot as plt
import fns_investigating_optical_flow_01 as fns
import overlay_01 as overlay
import labeling_image_components_01 as labeling_image_cc

def fill_holes(mask):
    '''
    Fill the holes in connected components in the mask.
    
    References:
        https://www.learnopencv.com/filling-holes-in-an-image-using-opencv-python-c/
        
        http://answers.opencv.org/question/9863/fill-holes-of-a-binary-image/
    '''

    assert mask.dtype == np.uint8, 'fill_holes: mask expected to be UINT8'
    
    # force nonzero values to be 255
    mask[mask != 0] = 255
    
    # make a zero border around image    
    mask_floodfill = np.zeros((mask.shape[0] + 2, mask.shape[1] + 2),
                              dtype=np.uint8)
    mask_floodfill[1:mask.shape[0] + 1, 1:mask.shape[1] + 1] = mask
    
    # starting from the zero border, do a flood fill
    cv2.floodFill(mask_floodfill, None, (0, 0), 255)
    
    # invert making the holes in the connected components 255
    holes = cv2.bitwise_not(mask_floodfill[1:mask.shape[0] + 1,
                                           1:mask.shape[1] + 1])
    
    # fill the holes via OR and return
    return cv2.bitwise_or(mask, holes)
    

def break_mask(image, mask, scale=0.0):
    '''
    Apply the distance transform the the mask to get markers and use watershed.
    Returns the marker from watershed where the contours found are -1.
    
    References:
        http://opencv-python-tutroals.readthedocs.io/en/latest/py_tutorials/py_imgproc/py_watershed/py_watershed.html
    '''
    
    # kernel
    kernel = np.ones((3,3),np.uint8)
        
    # sure background area
    sure_bg = cv2.dilate(mask, kernel, iterations=3)
    
    # replacing with precise
    dist_transform = cv2.distanceTransform(mask, distanceType=cv2.DIST_L2,
                                           maskSize=cv2.DIST_MASK_PRECISE)
    threshold = scale*dist_transform.max()
    maxval = 255
    ret, sure_fg = cv2.threshold(dist_transform, threshold, maxval,
                                 cv2.THRESH_BINARY)
    
    # Finding unknown region
    sure_fg_uint8 = np.uint8(sure_fg)
    unknown = cv2.subtract(sure_bg, sure_fg_uint8)
        
    # Marker labelling
    ret, markers = cv2.connectedComponents(sure_fg_uint8)
    
    # Add one to all labels so that sure background is not 0, but 1
    markers = markers+1
    
    # Now, mark the region of unknown with zero
    markers[unknown==255] = 0
    
    # get the watershed markers
    plt.matshow(markers)
    
    dist_trans = cv2.normalize(dist_transform ,None, 0, 255, 
                               cv2.NORM_MINMAX).astype(np.uint8)
    dist_trans = np.stack([dist_trans, dist_trans, dist_trans], axis=2)
    
    markers =  cv2.watershed(image, markers)
    
    # break up the mask
    boundaries = np.zeros_like(mask)
    boundaries[markers == -1] = 255
    boundaries = cv2.dilate(boundaries, kernel, iterations=1)
    
    boundaries = cv2.bitwise_not(boundaries)
    
    broken_mask = cv2.bitwise_and(boundaries, mask)
    
    cv2.imshow('background', sure_bg)
    cv2.imshow('distance transform', dist_transform / np.max(dist_transform))
    cv2.imshow('forground', sure_fg)
    cv2.imshow('unknown', unknown)
    
    cv2.waitKey(0)

    cv2.destroyAllWindows()
    
    plt.matshow(markers)
    
    return (broken_mask, markers)

def is_max_in_neighborhood(image, radius):
    '''
    Determine if each pixel is the maximum in a neighborhood.
    
    image - one channel image
    radius - radius of the square used as a kernel
    '''
    
    neigborhood_max = image
    
    with_border = cv2.copyMakeBorder(image, radius, radius,
                                     radius, radius, cv2.BORDER_CONSTANT,
                                     None, 0)
    
    collect = []
    
    for i in range(1, radius + 1):
        for j in range(1, radius + 1):
        
            collect.append(image > with_border[\
                                    radius + i : image.shape[0] + radius + i,
                                    radius + j : image.shape[1] + radius + j])
        
            collect.append(image > with_border[\
                                    radius - i : image.shape[0] + radius - i,
                                    radius - j : image.shape[1] + radius - j])
        
            collect.append(image > with_border[\
                                    radius - i : image.shape[0] + radius - i,
                                    radius + j : image.shape[1] + radius + j])
        
            collect.append(image > with_border[\
                                    radius + i : image.shape[0] + radius + i,
                                    radius - j : image.shape[1] + radius - j])

    mark = np.zeros(image.shape, dtype=np.uint8)
    
    mark[np.all(collect, axis=0)] = 255
    
    return mark
    
    
    # video path
paths_specific = ['/STI WA Apr25 A/STI WA Apr25 AMVI_0068_trans/',
                  'STI Polk Mar02 E/00005_trans/',
                  'STI Polk Apr13 E/MVI_0044_trans/',
                  'STI Polk Mar02 B/00006_trans/',
                  'STI WA Mar28 C/MVI_0013_trans/',
                  'STI WA May06 Polk B/MVI_0091_trans/',
                  'STI Polk Feb16 C/00054_trans/',
                  'STI Polk May04 F/MVI_0079_trans/',
                  'STI WA Apr18 D/MVI_0056_trans/',
                  'STI WA Feb28 xtraB/00011_trans/',
                  'STI Polk Apr20 C/MVI_0053_trans/',
                  'STI WA Apr18 B/MVI_0029_trans/',
                  'STI WA Feb28 C/00089_trans/']
video_names = ['STI WA Apr25 AMVI_0068_trans__860_910.mp4',
               '00005_trans__605_628.mp4',
               'MVI_0044_trans__350_365.mp4',
               '00006_trans__567_600.mp4',
               'MVI_0013_trans__110_140.mp4',
               'MVI_0091_trans__60_85.mp4',
               '00054_trans__452_505.mp4',
               'MVI_0079_trans__390_415.mp4',
               'MVI_0056_trans__205_240.mp4',
               '00011_trans__290_335.mp4',
               'MVI_0053_trans__50_72.mp4',
               'MVI_0029_trans__75_125.mp4',
               '00089_trans__110_165.mp4']
frames_target = [[ 260,  748, 1200, 1480, 1648, 1660, 1780],
                 [  78,  164,  190,  202,  206,  322,  406,  442,  508,  554],
                 [  92,  248,  344,  404,  480,  544,  784],
                 [  84,  170,  316,  408,  500,  792,  870],
                 [   4,  272,  316,  620,  704,  940, 1140, 1260],
                 [  40,  212,  396,  480,  620,  792,  880],
                 [ 122,  360,  478,  632,  876,  906,  938, 1144, 1164, 1196],
                 [   0,  144,  688,  964],
                 [  76,  126,  814],
                 [ 134,  190,  202,  770,  778,  908, 1110],
                 [  76,  736,  800,  860, 1132, 1264],
                 [ 172,  324,  472,  596, 1136, 1720, 2000, 2128, 2396, 2576],
                 [  16,   22,  462,  478,  560,  712,  776,  784,  836, 1066, 1238]]

frames_target_subset = [[1200],
                        [322],
                        [344, 404],
                        [792],
                        [940],
                        [792],
                        [1144],
                        [144],
                        [126],
                        [778],
                        [860],
                        [1720],
                        [712]]

path_common = './video_copies_clips_track/copies_and_clips/'

escape = False

# make object for labeling
set_target = labeling_image_cc.CCLabelingModified(['hands or forearm', 'face',
                                                   'other'])

for path_specific, video_name, frame_list in zip(paths_specific, video_names,
                                                 frames_target):
    
    if escape == True:
        
        cv2.destroyAllWindows()
        
        break
    
    for target_frame in frame_list:
        
        if escape == True:
        
            cv2.destroyAllWindows()
            
            break
        
        print(target_frame )
    
        # pickle
        pickle_path = path_common + path_specific + video_name[0:-4] + \
                      '__' + str(target_frame) + '.data'
                    
        with open(pickle_path, 'rb') as f:
            frame, flow = pickle.load(f)
            
        skin_mask = skin_detector.DetectSkinRegions(frame)
        kernel = np.array([[0, 0, 1, 0, 0],
                           [0, 1, 1, 1, 0],
                           [1, 1, 1, 1, 1],
                           [0, 1, 1, 1, 0],
                           [0, 0, 1, 0, 0]], dtype=np.uint8)
        #skin_mask = cv2.dilate(skin_mask, np.ones(3))
        #skin_mask = fill_holes(skin_mask)
        
        #skin_mask_broken, markers = break_mask(frame, skin_mask)
        skin_mask_display = cv2.bitwise_and(frame, frame, mask=skin_mask)
        
        mag = cv2.magnitude(flow[:, :, 0], flow[:, :, 1])
        
        # first box plot, take the upper whisker
        bp1 = plt.boxplot(np.reshape(mag, -1))
        plt.close()
        flow_mask1 = mag > bp1['whiskers'][1].get_data()[1][1]
        
        # second box plot on the uper whisker, threshold with the median
        bp2 = plt.boxplot(mag[flow_mask1])
        plt.close()
        flow_mask = (mag > bp2['medians'][0].get_data()[1][0]).astype(np.uint8) * 255
        
        # arrows
        frame_gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        scaleFactor = 0.7
        arrows = fns.draw_flow(frame_gray, flow * (1 / scaleFactor), flow_mask, step=4)
        
        # overlay masked motion magnitude
        connectivity = 3
        cc = cv2.connectedComponentsWithStats(flow_mask, connectivity, cv2.CV_32S)
        overlay_motion = overlay.overlay(frame_gray, cc, np.unique(cc[1])[1:], (255, 0, 0))
        
        # anding
        and_mask = cv2.bitwise_and(skin_mask, flow_mask)
        fancy_and = cv2.bitwise_and(frame, frame, mask=and_mask)
        
        
        # thresholded motion magnitude
        motion_magnitude = cv2.bitwise_and(mag, mag, mask=flow_mask)


        # distance transform from fancy-and mask
        dt_fancy_and = cv2.distanceTransform(cv2.bitwise_not(and_mask),
                                             distanceType=cv2.DIST_L2,
                                             maskSize=cv2.DIST_MASK_PRECISE)
        dt_fancy_and_flow_mask = cv2.bitwise_and(dt_fancy_and, dt_fancy_and,
                                                 mask=flow_mask)

        # grow around fancy and
        dt_thresh = 14.5

        # threshold using the distance from the fancy and mask
        dt_thresh, from_fancy_keep = cv2.threshold(dt_fancy_and_flow_mask,
                                                   dt_thresh, 255,
                                                   cv2.THRESH_BINARY_INV)
        from_fancy_keep = from_fancy_keep.astype(np.uint8)
        from_fancy_keep = cv2.bitwise_or(from_fancy_keep, and_mask,
                                         mask=flow_mask)  
        
        # fill holes
        from_fancy_keep  = fill_holes(from_fancy_keep)

        # overlay
        cc_dt_flow_keep = cv2.connectedComponentsWithStats(from_fancy_keep,
                                                           connectivity,
                                                           cv2.CV_32S)
        overlay_motion_keep = overlay.overlay(frame_gray, cc_dt_flow_keep,
                                              np.unique(cc_dt_flow_keep[1])[1:],
                                              (0, 255, 0))


        # distance transform on from_fancy_keep
        dt_from_fancy_keep = cv2.distanceTransform(from_fancy_keep,
                                                   distanceType=cv2.DIST_L2,
                                                   maskSize=cv2.DIST_MASK_PRECISE)
        
        # get rid of small connectors        
        th = 0.15 * np.max(dt_from_fancy_keep)
        th, less_small_connectors = cv2.threshold(dt_from_fancy_keep, th, 255,
                                                  cv2.THRESH_BINARY)
        less_small_connectors = less_small_connectors.astype(np.uint8)
        
        # smooth        
        less_small_connectors = cv2.morphologyEx(less_small_connectors,
                                                 cv2.MORPH_OPEN, np.ones(3),
                                                 iterations=1)
        less_small_connectors = cv2.dilate(less_small_connectors, np.ones(3),
                                           iterations=5)

        # components
        final_mask = cv2.bitwise_and(from_fancy_keep, less_small_connectors)
        
        # overlay in image
        cc_final = cv2.connectedComponentsWithStats(final_mask)
#        overlay_final_mask = overlay.overlay(frame_gray, cc_final,
#                                             np.unique(cc_final[1])[1:],
#                                             (255, 255, 0))
#        for col, row in cc_final[3][1:]:
#            point = (int(col), int(row))
#            cv2.line(overlay_final_mask, point, point, (0, 0, 255), 5)
#        
#        
#
#
#        # display
#        titles = ['overlay of final mask']
#        images = [overlay_final_mask]
#
#                
#        for title, image in zip(titles, images):
#            
#            cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
#            cv2.moveWindow(title, 0, -60)    
#            cv2.imshow(title, image)
#        
#        k = cv2.waitKey(0)
#                
#        # escape
#        if k == 27:
#            
#            escape = True

        set_target.set_anew_run([frame], 0, list(cc_final))
        
        frame, cc_final, flag = set_target.get_results()
        
        print(cc_final[-1])
        
        # end program excecution
        if flag == 'quit':
            
            escape = True

cv2.destroyAllWindows()