# -*- coding: utf-8 -*-
"""
Created on Sat Feb 10 09:55:15 2018

@author: Callie Darsey

tkinter checkbox example
"""

#!/usr/bin/python3

import tkinter
from functools import partial

class ChooseDescriptorsSimple():
    '''
    This class allows one to check the descriptors that apply. To leave the
    window, press the 'done' button.
    '''
    
    def __init__(self, checkbox_strings):
        '''
        Initializer method
        
        checkbox_strings - list of descriptor strings associated with each
                           checkbox
        '''
        
        # save to a instance variable
        padded = [descr + ' ' * \
                  (max([len(descr) for descr in checkbox_strings]) - 
                   len(descr)) for descr in checkbox_strings]
        self.checkbox_strings = padded
        
    def _done(self):
        '''
        Call when Done button presses.
        '''        
        self.tk.destroy()
        
    def get_target(self):
        '''
        Return the list with the strings (stripped of whitespace) indicating
        whether the descriptor applied (initial string stripped of whitespace)
        or not (empty string).
        '''
        
        return_list = []
    
        for entry in self.results:
            return_list.append(entry.get().strip())
        
        return return_list
        
        
    def make_menu(self, checkbox_status):
        '''
        Make the menu.
        
        checkbox_status - list of True and False entrys indicating
                          whether checkbox should be checked in the menu
        '''
        assert len(checkbox_status) == len(self.checkbox_strings)
        
        # make object
        self.tk = tkinter.Tk()
               
        # put a frame in it        
        frame = tkinter.Frame(self.tk, relief=tkinter.RIDGE, borderwidth=10,
                              width=100) 
        frame.pack(fill=tkinter.BOTH, expand=1)

        # Done button
        button = tkinter.Button(frame, text='Done', command=self._done) 
        button.pack(side=tkinter.BOTTOM)  
        
        # give info
        label = tkinter.Label(frame, text='Which apply?') 
        label.pack()
        
        # make the checkboxes                
        # instance variable to hold the result from the checkbox
        self.results = []
        
        for text, status in zip(self.checkbox_strings, checkbox_status):
            
            self.results.append(tkinter.StringVar())
            
            if status == False:
                self.results[-1].set('')
                
            elif status == True:
                self.results[-1].set(text)
                
            chk = tkinter.Checkbutton(frame, offvalue='', onvalue=text,
                                      font='consolas', text=text,
                                      variable=self.results[-1])
            chk.pack()

        self.tk.mainloop()

        
 
if __name__ == '__main__':    
    obj = ChooseDescriptorsSimple(['hand or forearm', 'face', 'other'])
    obj.make_menu([True, False, True])
    target = obj.get_target()
    print(target)

