# -*- coding: utf-8 -*-
"""
Created on Fri Apr 20 17:41:05 2018

@author: Callie Darsey

The contents of this script

wenjing_01
"""

import cv2
import numpy as np
import pickle
import scipy as sp
import scipy.ndimage
import time
import os
import subprocess
from sklearn.preprocessing import StandardScaler
from sklearn.neighbors import KNeighborsClassifier
from sklearn.metrics import accuracy_score
from scipy.ndimage.interpolation import shift
from matplotlib import pyplot as plt
from sklearn.pipeline import Pipeline
import glob
import os
import warnings

def locate_window(title):
    '''
    This function places the window for openCV
    
    '''
    
    move_window = [0, -30]
    cv2.namedWindow(title, cv2.WINDOW_GUI_NORMAL | cv2.WINDOW_AUTOSIZE)
    cv2.moveWindow(title, move_window[0], move_window[1]) 

def flood_fill(test_array,h_max=255):
    input_array = np.copy(test_array) 
    el = sp.ndimage.generate_binary_structure(2,2).astype(np.int)
    inside_mask = sp.ndimage.binary_erosion(~np.isnan(input_array), structure=el)
    output_array = np.copy(input_array)
    output_array[inside_mask]=h_max
    output_old_array = np.copy(input_array)
    output_old_array.fill(0)   
    el = sp.ndimage.generate_binary_structure(2,1).astype(np.int)
    while not np.array_equal(output_old_array, output_array):
        output_old_array = np.copy(output_array)
        output_array = np.maximum(input_array,
                                  sp.ndimage.grey_erosion(output_array,
                                                          size=(3,3),
                                                          footprint=el))
    return output_array


def backFM(ori, mag, pha, show=True):
    '''
    
    '''
    size = 100
    
    start_time = time.time()
    
    if type(ori) == str:
        ori = cv2.imread(ori)
    else:
        assert type(ori) == np.ndarray
    
    if type(mag) == str:
        mag = cv2.imread(mag, 0)
    else:
        assert type(mag) == np.ndarray
        
    if type(pha) == str:
        pha = cv2.imread(pha, 0)
    else:
        assert type(pha) == np.ndarray
    
    if show == True:
        title = '1'; locate_window(title); cv2.imshow(title, ori)
        title = '2'; locate_window(title); cv2.imshow(title, mag)
        title = '3'; locate_window(title); cv2.imshow(title, pha)
    
    ret,thresh1 = cv2.threshold(pha, 150, 255,cv2.THRESH_BINARY)
    if show == True:
        title = 'pha_binary'; locate_window(title); cv2.imshow(title, thresh1)
    
    ret,thresh2 = cv2.threshold(mag, 100, 255,cv2.THRESH_BINARY)
    if show == True:    
        title = 'mag_binary'; locate_window(title); cv2.imshow(title, thresh2)
    
    edges = cv2.Canny(thresh1, 10, 100)
    fill_hole = flood_fill(edges)
    if show == True:    
        title = 'fill_hole'; locate_window(title); cv2.imshow(title, fill_hole)
    
    dot_comp = (fill_hole - edges)
    if show == True:    
        title = 'dot_comp'; locate_window(title); cv2.imshow(title, dot_comp)
    
    AMFM_comb = (255-thresh2) & dot_comp    
    if show == True:    
        title = 'AMFM_comb'; locate_window(title); cv2.imshow(title, AMFM_comb)

    rows, cols = mag.shape
    
    col_sum = AMFM_comb.sum(axis=0)
    ind = np.argpartition(col_sum, -60)[-60:]
    largest_col = np.zeros([rows, cols])
    for col in range(0, len(ind)):
        largest_col[:,ind] = AMFM_comb[:,ind]
    if show == True:    
        title = 'Largest_60'; locate_window(title); cv2.imshow(title, largest_col)
    
    compare = 0
    row = 0
    col = 0
    for i in range(0, rows-size, 10):
        for j in range(0, cols-size, 10):
            array = largest_col[i:i+size, j:j+size]
            Sum = np.sum(array)
            if compare < Sum:
                compare = Sum
                row = j
                col = i
            else:
                compare = compare
                row = row
                col = col
            
#    print(compare, row, col)
    
    back_cord = [row, col, size]
        
#    print("--- %s seconds ---" % (time.time() - start_time))
    
    if show == True:
        ori_with_box = np.copy(ori)
        cv2.rectangle(ori_with_box, (row, col), (row + size, col + size),
                      (255, 0, 0), 2)        
        title = 'final'; locate_window(title); cv2.imshow(title, ori_with_box)
    
    if show == True:    
        cv2.waitKey(0)
        cv2.destroyAllWindows()
            

    if compare == 0:
    
        return (None)
    
    else:
        
        return (back_cord)


def backFM_boy(ori, mag, pha, show=True):
    '''
    '''
    size = 100
    
    if type(ori) == str:
        ori = cv2.imread(ori)
    else:
        assert type(ori) == np.ndarray
    
    if type(mag) == str:
        mag = cv2.imread(mag, 0)
    else:
        assert type(mag) == np.ndarray
        
    if type(pha) == str:
        pha = cv2.imread(pha, 0)
    else:
        assert type(pha) == np.ndarray
    
    if show == True:    
        title = '1'; locate_window(title); cv2.imshow(title, ori)
        title = '2'; locate_window(title); cv2.imshow(title, mag)
        title = '3'; locate_window(title); cv2.imshow(title, pha)    
    
    rows, cols = pha.shape
    
    ''' Threshold mag and phase'''
    ret, thresh1 = cv2.threshold(pha, 100, 255,cv2.THRESH_BINARY)
    if show == True:    
        title = 'pha_binary'; locate_window(title); cv2.imshow(title, thresh1)
    
    ret, thresh2 = cv2.threshold(mag, 200, 255,cv2.THRESH_BINARY)
    if show == True:    
        title = 'mag_binary'; locate_window(title); cv2.imshow(title, thresh2)

    
    ''' Find all round parts of phase threshold component '''
    circles = cv2.HoughCircles(pha, cv2.HOUGH_GRADIENT, 1, 70, param1=50,
                               param2=30, minRadius=35, maxRadius=60)
    
    empty = np.zeros([rows, cols])
    
    ori_copy = np.copy(ori)

    if circles is not None:
        
        circles = np.float16(np.around(circles))
    
        for i in circles[0,:]:
            # draw the outer circle
            cv2.circle(ori_copy, (i[0], i[1]), i[2], (0, 255, 0), 2)
            
            # draw the center of the circle
            cv2.circle(ori_copy, (i[0],i[1]), 2, (0, 0, 255), 3)
            
            empty[int((i[1]-i[2])):int((i[1]+i[2])),
                  int((i[0]-i[2])):int((i[0]+i[2]))] = \
                  thresh1[int((i[1]-i[2])):int((i[1]+i[2])),
                          int((i[0]-i[2])):int((i[0]+i[2]))]
                  
    if show == True:    
        title = 'detected circles'; locate_window(title);
        cv2.imshow(title, ori_copy)
    
    ''' Do AND with round part and mag threshold component '''
    comb_AMFM = np.uint8(empty) & (255-thresh2) 
    if show == True:    
        title='detected boy back';locate_window(title);
        cv2.imshow(title,comb_AMFM)
    

    ''' Enrode phase threshold component '''
    kernel = np.ones((5,5),np.uint8)
    thresh1 = cv2.erode(thresh1, kernel, iterations=1)
    if show == True:    
        title='erode_pha_binary'; locate_window(title);
        cv2.imshow(title, thresh1)
    
    ''' Same step as girls' head detection '''
    
    ''' Canny edge detector '''
    edges = cv2.Canny(thresh1, 10, 100)
    ''' fill holes '''
    fill_hole = flood_fill(edges)    
    if show == True:    
        title='fill_holes'; locate_window(title);
        cv2.imshow(title, fill_hole)
    
    dot_comp = (fill_hole - edges)
    if show == True:    
        title='dot_comp'; locate_window(title); cv2.imshow(title, dot_comp)
    
    AMFM_comb = (255-thresh2) & dot_comp
    if show == True:    
        title='AMFM_comb'; locate_window(title);
        cv2.imshow(title, AMFM_comb)
   
    ''' Combine two results '''
    final = comb_AMFM & AMFM_comb
    if show == True:    
        title='final1'; locate_window(title);
        cv2.imshow(title, final)
    
     
    compare = 0
    row = 0
    col = 0
    for i in range(0, rows-size, 10):
        for j in range(0, cols-size, 10):
            array = final[i:i+size, j:j+size]
            Sum = np.sum(array)
            #print(Sum)
            if compare < Sum:
                compare = Sum
                row = j
                col = i
            else:
                compare = compare
                row = row
                col = col
             
    back_cord = [row, col, size]

    if show == True:
        
        final[row:row+size, col:col+size] = np.zeros([size,size])

        cv2.rectangle(ori_copy, (row, col), (row + size, col + size),
                      (255, 0, 0), 2)
        
        title='final'; locate_window(title);
        cv2.imshow(title, ori_copy)
        
    if compare == 0:
    
        return (None)
    
    else:
        
        return (back_cord)


def get_face_squares(img, model):
    
    
    if type(img) == str:
        img = cv2.imread(img, 0)
    else:
        assert type(img) == np.ndarray

    size = 60

    skip = 30
    
    row, col = img.shape
    
    mask = np.zeros([row, col], dtype=np.uint8)
    
    collect_indices = []
    collect_features = []
    
    for i in range (0, row, skip):
        for j in range (0, col, skip):
            
            (start_row, start_col) = (i, j)

            if (start_row + size > row) or (start_col + size > col):
                break
            
            else:
                
                collect_indices.append((start_row, start_row + size,
                                        start_col, start_col + size))
                
                crop_img = img[start_row:(start_row + size),
                               start_col:(start_col + size)]
                
                KNN_test = crop_img.ravel().reshape((1, -1))
                
                collect_features.append(KNN_test)
                
                
    collect_features = np.vstack(collect_features)            
    pred_result = model.predict(collect_features)
    
    for index in np.where(pred_result)[0]:
        
        start_row, start_row_thru, start_col, start_col_thru = \
            collect_indices[index]

        mask[start_row : start_row_thru,
                start_col : start_col_thru] = 255
    
    return mask

    
def stitch(left, right, frame):
    '''
    
    
    
    '''
    
    assert left.shape[0] == left.shape[1]
    assert right.shape[0] == right.shape[1]
    assert left.shape[0] == right.shape[0]
    
    rows, cols, channels = frame.shape
    
    side, side, ch = left.shape
    
    left = cv2.resize(left, (rows, rows))
    right = cv2.resize(right, (rows, rows))
    
#    print('left.shape:', left.shape)
#    print('right.shape:', right.shape)
    
    stitched = np.zeros_like(frame)
    
    stitched[:, 0:rows, :] = left
    
#    print('', stitched[:, rows:, :].shape)
#    print('', right[:, rows - (cols - rows):, :].shape)
    
    
    stitched[:, rows:, :] = right[:, rows - (cols - rows):, :]
    
    stitched_resized = cv2.resize(stitched, (int(side * cols / rows), side))
    
    return stitched, stitched_resized



#if __name__ == '__main__':    
#    common = "./video_copies_clips_track/copies_and_clips_facilitator_C/"
#    specific = "STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/PNGs/"
#    mag_pha = "AM_FM_by_Wenjing/"
#    file = "MVI_0028_trans__200_208_0036.png"
#    
#    ori_path = common + specific + mag_pha + file[0:-3] + 'resized.png'
#    mag_path = common + specific + mag_pha + file[0:-3] + 'mag.resized.png'
#    pha_path = common + specific + mag_pha + file[0:-3] + 'phas.resized.png'
#    
#    back_cord, ori_with_box = backFM(ori_path, mag_path, pha_path, show=False)
#    
#    title='back off head detection'; locate_window(title);
#    cv2.imshow(title, ori_with_box)
#    
#    back_cord, ori_copy = backFM_boy(ori_path, mag_path, pha_path, show=False)
#      
#    title='boy back off head detection'; locate_window(title);
#    cv2.imshow(title, ori_copy)
#    
#    with open('./wheeler/svc/all_train_data1.pkl', 'rb') as f:
#        all_train_data1 = pickle.load(f)
#    
#    with open('./wheeler/svc/label_train_data1.pkl', 'rb') as f:
#        label_train_data1 = pickle.load(f)
#    
#    K=3
#    knn_clf = \
#        KNeighborsClassifier(n_jobs=-1, weights='distance', n_neighbors=K)    
#    knn_clf.fit(all_train_data1, label_train_data1.ravel())
#        
#    new_img = get_face_squares(pha_path, knn_clf)
#
#    title='faces'; locate_window(title);
#    cv2.imshow(title, new_img)
#    
#    cv2.waitKey(0)
#    cv2.destroyAllWindows()

#if __name__ == '__main__':    
#    common = "./video_copies_clips_track/copies_and_clips_facilitator_C/"
#    specific = "STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/PNGs/"
#    mag_pha = "AM_FM_by_Wenjing/"
#    file = "MVI_0028_trans__200_208_0036.png"
#    
#    ori_path = common + specific + mag_pha + file[0:-3] + 'resized.png'
#    mag_path = common + specific + mag_pha + file[0:-3] + 'mag.resized.png'
#    pha_path = common + specific + mag_pha + file[0:-3] + 'phas.resized.png'
#    
#    back_cord, ori_with_box = backFM(ori_path, mag_path, pha_path, show=True)
#    
#    title='back off head detection'; locate_window(title);
#    cv2.imshow(title, ori_with_box)
#    
#    back_cord, ori_copy = backFM_boy(ori_path, mag_path, pha_path, show=False)
#      
#    title='boy back off head detection'; locate_window(title);
#    cv2.imshow(title, ori_copy)
#    
#    with open('./wheeler/svc/all_train_data1.pkl', 'rb') as f:
#        all_train_data1 = pickle.load(f)
#    
#    with open('./wheeler/svc/label_train_data1.pkl', 'rb') as f:
#        label_train_data1 = pickle.load(f)
#    
#    K=3
#    knn_clf = \
#        KNeighborsClassifier(n_jobs=-1, weights='distance', n_neighbors=K)    
#    knn_clf.fit(all_train_data1, label_train_data1.ravel())
#        
#    new_img = get_face_squares(pha_path, knn_clf)
#
#    title='faces'; locate_window(title);
#    cv2.imshow(title, new_img)
#    
#    cv2.waitKey(0)
#    cv2.destroyAllWindows()






if __name__ == '__main__': 
    
    e1 = cv2.getTickCount()
    
    common = "./video_copies_clips_track/copies_and_clips_facilitator_C/"
    specific = "STI Polk Apr06 C/MVI_0028_trans/MVI_0028_trans__200_208/PNGs/"
    name = "MVI_0028_trans__200_208_0036.png"
    
    assemble = common + specific + "AM_FM_by_Wenjing/"
    if not os.path.isdir(assemble):
        os.mkdir(assemble)
    
    frame = cv2.imread(common + specific + name)
    
    rows, cols, channels = frame.shape
    
    frame_left, frame_right = frame[:, 0: rows, :], frame[:, -rows:, :]
    
    print('frame_left.shape', frame_left.shape)
    print('frame_right.shape', frame_right.shape)
    
    name = 'tmp.'
    
    file_left = assemble + name + 'left.png'
    file_right = assemble + name + 'right.png'
    
    cv2.imwrite(file_left, frame_left)
    cv2.imwrite(file_right, frame_right)
    
    executable = \
        "../VisualStudio/Gabor_by_Wenjing_Shi/x64/Release/Gabor_by_Wenjing_Shi.exe"
    subprocess.call([executable, file_left])
    subprocess.call([executable, file_right])
    
    file_mag_left = assemble + name + 'left.mag.png'
    file_phas_left = assemble + name + 'left.phas.png'
    
    file_mag_right = assemble + name + 'right.mag.png'
    file_phas_right = assemble + name + 'right.phas.png'
    
    mag_left = cv2.imread(file_mag_left)
    phas_left = cv2.imread(file_phas_left)
    
    mag_right = cv2.imread(file_mag_right)
    phas_right = cv2.imread(file_phas_right)
    
    stitched_mag, stitched_mag_resized = stitch(mag_left, mag_right, frame)
    stitched_phas, stitched_phas_resized = stitch(phas_left, phas_right, frame)
    
    with open('./wheeler/svc/all_train_data1.pkl', 'rb') as f:
        all_train_data1 = pickle.load(f)
    
    with open('./wheeler/svc/label_train_data1.pkl', 'rb') as f:
        label_train_data1 = pickle.load(f)
    
    K=3
    knn_clf = \
        KNeighborsClassifier(n_jobs=-1, weights='distance', n_neighbors=K)    
    knn_clf.fit(all_train_data1, label_train_data1.ravel())
        
    am_face_mask = get_face_squares(stitched_phas_resized[:, :, 0], knn_clf)
    
    
    am_face_mask_resized = cv2.resize(am_face_mask, (stitched_phas.shape[1],
                                                     stitched_phas.shape[0]))
    
    
    title='faces'; locate_window(title);
    cv2.imshow(title, cv2.bitwise_and(stitched_phas, stitched_phas,
                                      mask=am_face_mask_resized))
    
    frame_downsized = cv2.resize(frame, (stitched_phas.shape[1],
                                         stitched_phas.shape[0]))
    
    back_info = backFM(frame_downsized, stitched_mag_resized[:, :, 0],
                       stitched_phas_resized[:, :, 0], show=False)
    
    if back_info is not None:
        
        back_mask = np.zeros_like(stitched_mag_resized)
    
        cv2.rectangle(back_mask, (back_info[0], back_info[1]),
                      (back_info[0] + back_info[2], back_info[1] + back_info[2]),
                      (255, 255, 255), -1)

        back_mask = cv2.resize(back_mask, (stitched_phas.shape[1],
                                           stitched_phas.shape[0]))
        
        title='back of head'; locate_window(title);
        cv2.imshow(title, cv2.bitwise_and(frame, frame,
                                          mask=back_mask[:, :, 0]))
        
    
    boy_back_info = backFM_boy(frame_downsized, stitched_mag_resized[:, :, 0],
                               stitched_phas_resized[:, :, 0], show=False)
    
    if boy_back_info is not None:
        
        boy_back_mask = np.zeros_like(stitched_mag_resized)
    
        cv2.rectangle(boy_back_mask, (boy_back_info[0], boy_back_info[1]),
                                     (boy_back_info[0] + boy_back_info[2],
                                      boy_back_info[1] + boy_back_info[2]),
                      (255, 255, 255), -1)
                                     
        boy_back_mask = cv2.resize(boy_back_mask, (stitched_phas.shape[1],
                                                   stitched_phas.shape[0]))
        
        title='boy back of head'; locate_window(title);
        cv2.imshow(title, cv2.bitwise_and(frame, frame,
                                          mask=boy_back_mask[:, :, 0]))
    
    e2 = cv2.getTickCount()
    print('Execution time per frame: {:.1f}'.format(\
          (e2 - e1) / cv2.getTickFrequency()))
    
    cv2.waitKey(0)
    cv2.destroyAllWindows()










